@@ -15,7 +15,9 @@
 #ifndef _ID_H
 #define _ID_H
 
+/* see ipsec_policy.h for ID_* values, which are used in the kernel too */
 struct id {
+    bool       has_wildcards;  /* if there is wildcard buired in DN */
     int kind;		/* ID_* value */
     ip_address ip_addr;	/* ID_IPV4_ADDR, ID_IPV6_ADDR */
     chunk_t name;	/* ID_FQDN, ID_USER_FQDN (with @) */
@@ -59,6 +61,7 @@ extern void unshare_id_content(struct id *id);
 extern void free_id_content(struct id *id);
 extern bool any_id(const struct id *a);
 extern bool same_id(const struct id *a, const struct id *b);
+extern bool same_exact_id(const struct id *a, const struct id *b);
 #define MAX_WILDCARDS	15
 extern bool match_id(const struct id *a, const struct id *b, int *wildcards);
 extern int id_count_wildcards(const struct id *id);
@@ -78,3 +81,9 @@ extern const char *pluto_ip_str(const ip_address *src);
 
 #endif /* _ID_H */
 
+/*
+ * Local Variables:
+ * c-basic-offset:4
+ * c-style: pluto
+ * End:
+ */
