@@ -1,6 +1,7 @@
 /* Openswan config file parser (confread.h)
  * Copyright (C) 2001-2002 Mathieu Lafon - Arkoon Network Security
  * Copyright (C) 2003-2006 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2012 Paul Wouters <paul@libreswan.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -20,13 +21,19 @@
 
 #ifndef _OPENSWAN_H
 #include <openswan.h>
-#include "constants.h"  
+#include "constants.h"
 #endif
+#include "sysqueue.h"
 
+#include <limits.h>  /* for PATH_MAX */
+extern char rootdir[PATH_MAX];        /* when evaluating paths, prefix this to them */
+extern char rootdir2[PATH_MAX];       /* when evaluating paths, prefix this to them */
 
 /* define an upper limit to number of times also= can be used */
 #define ALSO_LIMIT 32
 
+struct whack_message;
+
 enum keyword_set {
     k_unset   =FALSE,
     k_set     =TRUE,
@@ -46,7 +53,7 @@ struct starter_end {
     ip_subnet subnet;
     char *iface;
     char *id;
-    
+
     enum pubkey_source rsakey1_type, rsakey2_type;
     unsigned char *rsakey1;
     unsigned char *rsakey2;
@@ -71,7 +78,7 @@ struct starter_conn {
     TAILQ_ENTRY(starter_conn) link;
     struct starter_comments_list comments;
     char *name;
-    char *connalias;			      
+    char *connalias;
 
     ksf   strings;
     knf   options;
@@ -81,9 +88,10 @@ struct starter_conn {
     bool  changed;
 
     bool  manualkey;         /* TRUE if this conn is going to be manually keyed */
-    
+
     lset_t policy;
     char **alsos;
+    char **also_flips;
 
     struct starter_end left, right;
 
@@ -114,7 +122,7 @@ struct starter_config {
 	knf   options;
 	str_set strings_set;
 	int_set options_set;
-	
+
 	/* derived types */
 	char **interfaces;
 	bool strictcrlpolicy;
@@ -138,6 +146,9 @@ struct starter_config {
 
     char *ctlbase;  /* location of pluto control socket */
 
+    /* abstract the sending part for building unit tests */
+    int (*send_whack_msg)(struct starter_config *cfg, struct whack_message *msg);
+
     /* connections list (without %default) */
     TAILQ_HEAD(, starter_conn) conns;
 };
@@ -149,18 +160,6 @@ extern struct starter_config *confread_load(const char *file
 					    , bool setuponly);
 extern struct starter_conn *alloc_add_conn(struct starter_config *cfg
 					   , char *name, err_t *perr);
-extern int init_load_conn(struct starter_config *cfg
-			  , struct config_parsed *cfgp
-			  , struct section_list *sconn
-			  , bool alsoprocessing
-			  , bool defaultconn
-			  , bool resolvip
-			  , err_t *perr);
-extern bool translate_conn (struct starter_conn *conn
-			    , struct section_list *sl
-			    , enum keyword_set   assigned_value
-			    , err_t *error);
-
 void confread_free(struct starter_config *cfg);
 
 void ipsecconf_default_values (struct starter_config *cfg);
