@@ -49,29 +49,13 @@ enum keyword_remotepeertype {
     CISCO  = 1,
 };
 
-enum keyword_sha2_truncbug {
-    SHA2_TRUNCBUG_NO = 0,
-    SHA2_TRUNCBUG_YES = 1,
-};
-
-/* Network Manager support */
-enum keyword_nmconfigured {
-    NM_NO = 0,
-    NM_YES = 1,
-};
-
-enum keyword_loopback {
-    LB_NO = 0,
-    LB_YES = 1,
-};
-
-enum keyword_labeled_ipsec {
-    LI_NO = 0,
-    LI_YES = 1,
+enum keyword_xauthby {
+	XAUTHBY_FILE = 0,
+	XAUTHBY_PAM = 1,
+	XAUTHBY_ALWAYSOK = 2,
 };
 
 /* Timer events */
-
 enum event_type {
     EVENT_NULL,	/* non-event */
     EVENT_REINIT_SECRET,	/* Refresh cookie secret */
@@ -91,6 +75,7 @@ enum event_type {
     EVENT_v2_RETRANSMIT,   /* Retransmit v2 packet */
 
     EVENT_PENDING_DDNS, /* try to start connections where DNS failed at init */
+    EVENT_SA_DELETE,    /* SA delete was sent, but never acknowledged */
 };
 
 #define EVENT_REINIT_SECRET_DELAY		3600 /* 1 hour */
@@ -98,6 +83,10 @@ enum event_type {
 #define EVENT_RETRANSMIT_DELAY_0		10   /* 10 seconds */
 #define EVENT_GIVEUP_ON_DNS_DELAY               300  /* 5 minutes for DNS */
 
+/* how long to keep a failed state open in case of retransmits */
+#define EVENT_HALF_OPEN_TIMEOUT                 60
+
+
 /*
  * cryptographic helper operations.
  */
@@ -178,7 +167,7 @@ typedef enum {
 #define DBG_CONTROL	LELEM(4)	/* control flow within Pluto */
 #define DBG_LIFECYCLE	LELEM(5)	/* SA lifecycle */
 #define DBG_KLIPS	LELEM(6)	/* messages with the kernel */
-#define DBG_NETKEY	LELEM(6)	/* same as previous entry!! */ 
+#define DBG_NETKEY	LELEM(6)	/* same as previous entry!! */
 #define DBG_DNS		LELEM(7)	/* DNS activity */
 #define DBG_OPPO	LELEM(8)	/* opportunism */
 #define DBG_CONTROLMORE LELEM(9)	/* more detailed debugging */
@@ -239,6 +228,7 @@ typedef enum {
 
 enum state_kind {
     STATE_UNDEFINED=0,	/* 0 -- most likely accident */
+    STATE_DELETING,     /* indicates state is dead, but not yet freed */
 
     /*  Opportunism states: see "Opportunistic Encryption" 2.2 */
 
@@ -298,11 +288,15 @@ enum state_kind {
      * for all work states. */
     STATE_PARENT_R1,
     STATE_PARENT_R2,
+    STATE_CHILD_C1_KEYED,      /* terminal state for a CHILD SA that has been successfully keyed */
+
+    /* INITIATOR child (re)key states */
+    STATE_CHILD_C1_REKEY,      /* initial state for a CHILD SA that is beginning a rekey */
 
     /* IKEv2 Delete States */
     STATE_IKESA_DEL,
     STATE_CHILDSA_DEL,
-    
+
     STATE_IKEv2_ROOF,
 };
 
@@ -311,13 +305,6 @@ enum phase1_role {
   RESPONDER=2
 };
 
-enum ikev2_msgtype {
-  req_sent=1,
-  req_recd=2,
-  response_sent=3,
-  response_recd=4
-};
-
 
 #define STATE_IKE_FLOOR	STATE_MAIN_R0
 
@@ -325,7 +312,8 @@ enum ikev2_msgtype {
 				  |LELEM(STATE_MAIN_I3) | LELEM(STATE_MAIN_I4) \
 				  |LELEM(STATE_AGGR_I1) | LELEM(STATE_AGGR_I2) \
 				  |LELEM(STATE_XAUTH_I0) | LELEM(STATE_XAUTH_I1) \
-				  | LELEM(STATE_MODE_CFG_I1))
+				  | LELEM(STATE_MODE_CFG_I1) \
+                                  | LELEM(STATE_PARENT_I1))
 #define IS_PHASE1_INIT(s)         ((s) == STATE_MAIN_I1 \
 				   || (s) == STATE_MAIN_I2 \
 				   || (s) == STATE_MAIN_I3 \
@@ -339,20 +327,16 @@ enum ikev2_msgtype {
 #define IS_PHASE15(s) (STATE_XAUTH_R0 <= (s) && (s) <= STATE_XAUTH_I1)
 #define IS_QUICK(s) (STATE_QUICK_R0 <= (s) && (s) <= STATE_QUICK_R2)
 #define IS_ISAKMP_ENCRYPTED(s)     (STATE_MAIN_R2 <= (s) && STATE_AGGR_R0!=(s) && STATE_AGGR_I1 != (s) && STATE_INFO != (s))
-#define IS_ISAKMP_AUTHENTICATED(s) (STATE_MAIN_R3 <= (s) && STATE_AGGR_R0!=(s) && STATE_AGGR_I1 != (s))
-#define IS_ISAKMP_SA_ESTABLISHED(s) ((s) == STATE_MAIN_R3 || (s) == STATE_MAIN_I4 \
-				  || (s) == STATE_AGGR_I2 || (s) == STATE_AGGR_R2 \
-				  || (s) == STATE_XAUTH_R0 || (s) == STATE_XAUTH_R1 \
-				  || (s) == STATE_MODE_CFG_R0 || (s) == STATE_MODE_CFG_R1 \
-				  || (s) == STATE_MODE_CFG_R2 || (s) == STATE_MODE_CFG_I1 \
-                                  || (s) == STATE_XAUTH_I0 || (s) == STATE_XAUTH_I1)
+#define IS_ISAKMP_AUTHENTICATED(s) ((STATE_MAIN_R3 <= (s) && STATE_AGGR_R0!=(s) && STATE_AGGR_I1 != (s)) || STATE_PARENT_I3 == (s) || STATE_PARENT_R2)
+#define IS_ISAKMP_SA_ESTABLISHED(s) LHAS(ISAKMP_SA_ESTABLISHED_STATES, (s))
 #define ISAKMP_SA_ESTABLISHED_STATES  (LELEM(STATE_MAIN_R3) | \
 				       LELEM(STATE_MAIN_I4) | \
 				       LELEM(STATE_AGGR_I2) | \
 				       LELEM(STATE_AGGR_R2) | \
 				       LELEM(STATE_XAUTH_R0) | \
 				       LELEM(STATE_XAUTH_R1) | \
-				       LELEM(STATE_MODE_CFG_R0) | \
+                                       LELEM(STATE_PARENT_I3) | LELEM(STATE_PARENT_R2) | \
+				       LELEM(STATE_MODE_CFG_R0) |       \
 				       LELEM(STATE_MODE_CFG_R1) | \
 				       LELEM(STATE_MODE_CFG_R2) | \
 				       LELEM(STATE_MODE_CFG_I1) | \
@@ -365,12 +349,12 @@ enum ikev2_msgtype {
 #define IS_MODE_CFG_ESTABLISHED(s) ((s) == STATE_MODE_CFG_R2)
 #endif
 
-#define IS_PARENT_SA_ESTABLISHED(s) ((s) == STATE_PARENT_I3 || (s) == STATE_PARENT_R2)
-/* 
- * Issue here is that our child sa appears as a STATE_PARENT_I3/STATE_PARENT_R2 state which it should not 
+#define IS_PARENT_SA_ESTABLISHED(s) ((s) == STATE_PARENT_I3 || (s) == STATE_PARENT_I2 || (s) == STATE_PARENT_R1 || (s) == STATE_PARENT_R2 || (s) == STATE_IKESA_DEL)
+/*
+ * Issue here is that our child sa appears as a STATE_PARENT_I3/STATE_PARENT_R2 state which it should not
  * So we fall back to checking if it is cloned, and therefor really a child
  */
-#define IS_CHILD_SA_ESTABLISHED(st) (((st->st_state) == STATE_PARENT_I3 || (st->st_state) == STATE_PARENT_R2 || (st->st_state) == STATE_CHILDSA_DEL) && (st->st_childsa != NULL))
+#define IS_CHILD_SA_ESTABLISHED(st) ( (((st->st_state == STATE_PARENT_I3) || (st->st_state == STATE_PARENT_R2)) && (st->st_clonedfrom != SOS_NOBODY)) || (st->st_state == STATE_CHILDSA_DEL) || (st->st_state == STATE_CHILD_C1_KEYED))
 
 #define IS_CHILD_SA(st)  ((st)->st_clonedfrom != SOS_NOBODY)
 #define IS_PARENT_SA(st) (!IS_CHILD_SA(st))
@@ -421,6 +405,11 @@ enum certpolicy {
 /* this is the default setting. */
 #define cert_defaultcertpolicy cert_alwayssend
 
+enum ikev1_natt_policy {
+	natt_both = 0, /* the default */
+	natt_rfc = 1,
+	natt_drafts = 2 /* Workaround for Cisco NAT-T bug */
+};
 
 enum four_options {
 	fo_never   = 0,  /* do not propose, do not permit */
@@ -429,12 +418,20 @@ enum four_options {
 	fo_insist  = 3   /* propose, and only accept if peer agrees */
 };
 
+enum ynf_options {
+	ynf_no   = 0,
+	ynf_yes  = 1,
+	ynf_force = 2,
+};
+
 enum saref_tracking {
 	sat_yes = 0, /* SAref tracking via _updown - the default */
 	sat_no = 1, /* no SAref tracking - third party will handle this */
 	sat_conntrack = 2, /* Saref tracking using connmark optimizations */
 };
 
+
+
 /* Policies for establishing an SA
  *
  * These are used to specify attributes (eg. encryption) and techniques
@@ -445,18 +442,18 @@ enum saref_tracking {
 
 extern const char *prettypolicy(lset_t policy);
 
-/* 
+/*
  * ISAKMP auth techniques (none means never negotiate)
  * a pluto policy is stored in a lset_t which is an unsigned long long,
  * so we should have 64 bits to play with
  */
 enum pluto_policy {
-	POLICY_PSK     = LELEM(0), 
+	POLICY_PSK     = LELEM(0),
 	POLICY_RSASIG  = LELEM(1),
 #define POLICY_ISAKMP_SHIFT	0	/* log2(POLICY_PSK) */
 
 /* policies that affect ID types that are acceptable - RSA, PSK, XAUTH */
-        POLICY_ID_AUTH_MASK=LRANGES(POLICY_PSK, POLICY_RSASIG), 
+        POLICY_ID_AUTH_MASK=LRANGES(POLICY_PSK, POLICY_RSASIG),
 
 /* policies that affect choices of proposal, note, does not include XAUTH */
 #define POLICY_ISAKMP(x,xs,xc)	(((x) & LRANGES(POLICY_PSK, POLICY_RSASIG)) + \
@@ -477,7 +474,7 @@ enum pluto_policy {
 	POLICY_SHUNT_SHIFT = 8,	/* log2(POLICY_SHUNT_PASS) */
 	POLICY_SHUNT_MASK  = (03ul << POLICY_SHUNT_SHIFT),
 	POLICY_SHUNT_TRAP  = (0ul << POLICY_SHUNT_SHIFT), /* default: negotiate */
-	POLICY_SHUNT_PASS  = (1ul << POLICY_SHUNT_SHIFT), 
+	POLICY_SHUNT_PASS  = (1ul << POLICY_SHUNT_SHIFT),
 	POLICY_SHUNT_DROP  = (2ul << POLICY_SHUNT_SHIFT),
 	POLICY_SHUNT_REJECT=(3ul << POLICY_SHUNT_SHIFT),
 
@@ -639,6 +636,10 @@ enum keyword_host {
     KH_IPHOSTNAME   = 7,                /* host_addr invalid, only string */
     KH_IPADDR       = LOOSE_ENUM_OTHER,
 };
+/* keyword_name(&kw_host_list, type, buffer[KEYWORD_NAME_BUFLEN]) */
+struct keyword_enum_values kw_host_list;
+#define KH_ISWILDCARD(type)  ((type) == KH_ANY || (type) == KH_DEFAULTROUTE)
+#define KH_ISKNOWNADDR(type) ((type) == KH_IPADDR || (type)==KH_IFACE)
 
 /* BIND enumerated types */
 
@@ -673,5 +674,8 @@ enum PrivateKeyKind {
     PPK_XAUTH=5,
 };
 
+#define XAUTH_PROMPT_TRIES 3
+#define XAUTH_MAX_NAME_LENGTH 128
+#define XAUTH_MAX_PASS_LENGTH 128
 
 
