@@ -1,6 +1,6 @@
 /*
  * Openswan config file writer (confwrite.c)
- * Copyright (C) 2004-2006 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2004-2015 Michael Richardson <mcr@xelerance.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -27,8 +27,8 @@
 
 void confwrite_list(FILE *out, char *prefix, int val, struct keyword_def *k)
 {
-    struct keyword_enum_values *kevs = k->validenum;
-    struct keyword_enum_value  *kev  = kevs->values;
+    const struct keyword_enum_values *kevs = k->validenum;
+    const struct keyword_enum_value  *kev  = kevs->values;
     int i=0;
     unsigned int mask=1;
     char *sep="";
@@ -54,22 +54,23 @@ void confwrite_int(FILE *out,
     struct keyword_def *k;
 
     for(k=ipsec_conf_keywords_v2; k->keyname!=NULL; k++) {
-	
+
 	if((k->validity & KV_CONTEXT_MASK) != context) continue;
 	if(keying_context != 0 && (k->validity & keying_context)==0) continue;
 
 	/* do not output aliases */
 	if(k->validity & kv_alias) continue;
-	
+
 	/* do not output policy settings handled elsewhere */
 	if(k->validity & kv_policy) continue;
 	if(k->validity & kv_processed) continue;
+        if(k->validity & kv_obsolete)  continue;
 
 #if 0
 	printf("#side: %s  %s validity: %08x & %08x=%08x vs %08x\n", side,
 	       k->keyname, k->validity, KV_CONTEXT_MASK, k->validity&KV_CONTEXT_MASK, context);
 #endif
-	
+
 	switch(k->type) {
 	case kt_string:
 	case kt_appendstring:
@@ -106,16 +107,17 @@ void confwrite_int(FILE *out,
 
 	case kt_enum:
 	case kt_loose_enum:
+	case kt_loose_enumarg:
 	    /* special enumeration */
 	    if(options_set[k->field]) {
 		int val = options[k->field];
 		fprintf(out, "\t%s%s=",side, k->keyname);
-		
-		if(k->type == kt_loose_enum && val == LOOSE_ENUM_OTHER) {
+
+                if(k->type == kt_loose_enum && val == LOOSE_ENUM_OTHER) {
 		    fprintf(out, "%s\n", strings[k->field]);
 		} else {
-		    struct keyword_enum_values *kevs = k->validenum;
-		    struct keyword_enum_value  *kev  = kevs->values;
+		    const struct keyword_enum_values *kevs = k->validenum;
+		    const struct keyword_enum_value  *kev  = kevs->values;
 		    int i=0;
 
 		    while(i <kevs->valuesize) {
@@ -125,11 +127,15 @@ void confwrite_int(FILE *out,
 			}
 			i++;
 		    }
+                    if(k->type == kt_loose_enumarg && strings[k->field]) {
+                        fprintf(out, "%c%s", k->deliminator, strings[k->field]);
+                    }
+
 		    fprintf(out, "\n");
 		}
 	    }
 	    continue;
-	
+
 	case kt_list:
 	    /* special enumeration */
 	    if(options_set[k->field]) {
@@ -149,17 +155,14 @@ void confwrite_int(FILE *out,
 
 	case kt_comment:
 	    continue;
-
-	case kt_obsolete:
-	    continue;
 	}
 
 	if(options_set[k->field]) {
 	    fprintf(out, "\t%s%s=%d\n",side, k->keyname, options[k->field]);
 	}
-    }	
-}    
-		   
+    }
+}
+
 void confwrite_str(FILE *out,
 		   char   *side,
 		   int     context,
@@ -176,24 +179,24 @@ void confwrite_str(FILE *out,
 
 	/* do not output aliases */
 	if(k->validity & kv_alias) continue;
-	
+
 	/* do not output policy settings handled elsewhere */
 	if(k->validity & kv_policy) continue;
 	if(k->validity & kv_processed) continue;
-	
+
 	switch(k->type) {
 	case kt_appendlist:
 	    if(strings_set[k->field]) {
 		fprintf(out, "\t%s%s={%s}\n",side, k->keyname, strings[k->field]);
 	    }
-	    continue;	    
+	    continue;
 
 	case kt_string:
 	case kt_appendstring:
 	case kt_filename:
 	case kt_dirname:
 	    /* these are strings */
-	    break; 
+	    break;
 
 	case kt_rsakey:
 	case kt_ipaddr:
@@ -207,7 +210,8 @@ void confwrite_str(FILE *out,
 	case kt_enum:
 	case kt_list:
 	case kt_loose_enum:
-	    /* special enumeration */
+	case kt_loose_enumarg:
+	    /* special enumeration, sorta a string */
 	    continue;
 
 	case kt_time:
@@ -220,25 +224,22 @@ void confwrite_str(FILE *out,
 
 	case kt_comment:
 	    continue;
-
-	case kt_obsolete:
-	    continue;
 	}
 
 	if(strings_set[k->field]) {
 	    char *quote="";
 
 	    if(strchr(strings[k->field],' ')) quote="\"";
-	    
+
 	    fprintf(out, "\t%s%s=%s%s%s\n",side, k->keyname
 		    , quote
 		    , strings[k->field]
 		    , quote);
 	}
-    }	
-}    
-		   
-		   
+    }
+}
+
+
 void confwrite_side(FILE *out,
 		    struct starter_conn *conn,
 		    struct starter_end *end,
@@ -246,6 +247,7 @@ void confwrite_side(FILE *out,
 {
     char databuf[2048];  /* good for a 12288 bit rsa key */
     int  keyingtype;
+    bool addrargument = TRUE;
 
     if(conn->manualkey) {
 	keyingtype=kv_manual;
@@ -255,46 +257,52 @@ void confwrite_side(FILE *out,
 
     switch(end->addrtype) {
     case KH_NOTSET:
+	fprintf(out, "\t#%s= not set",side);
+        addrargument = FALSE;
 	/* nothing! */
 	break;
-	
+
     case KH_DEFAULTROUTE:
-	fprintf(out, "\t%s=%%defaultroute\n",side);
+	fprintf(out, "\t%s=%%defaultroute",side);
 	break;
-	
+
     case KH_ANY:
-	fprintf(out, "\t%s=%%any\n",side);
+	fprintf(out, "\t%s=%%any",side);
 	break;
-	
+
     case KH_IFACE:
 	if(end->strings_set[KSCF_IP]) {
-	    fprintf(out, "\t%s=%s\n",side, end->strings[KSCF_IP]);
+	    fprintf(out, "\t%s=%s",side, end->strings[KSCF_IP]);
 	}
+        addrargument = FALSE;
 	break;
-	
+
     case KH_OPPO:
-	fprintf(out, "\t%s=%%opportunistic\n",side);   
+	fprintf(out, "\t%s=%%opportunistic",side);
 	break;
-	
+
     case KH_OPPOGROUP:
-	fprintf(out, "\t%s=%%opportunisticgroup\n",side);   
+	fprintf(out, "\t%s=%%opportunisticgroup",side);
 	break;
-	
+
     case KH_GROUP:
-	fprintf(out, "\t%s=%%group\n",side);   
+	fprintf(out, "\t%s=%%group",side);
 	break;
 
     case KH_IPHOSTNAME:
-	if(end->strings_set[KSCF_IP]) {
-	    fprintf(out, "\t%s=%s\n",side, end->strings[KSCF_IP]);
-	}
+        fprintf(out, "\t%s=%%dns",side);
 	break;
 
     case KH_IPADDR:
 	addrtot(&end->addr, 0, databuf, ADDRTOT_BUF);
-	fprintf(out, "\t%s=%s\n", side, databuf);
+	fprintf(out, "\t%s=%s", side, databuf);
+        addrargument = FALSE;
 	break;
     }
+    if(addrargument && end->strings_set[KSCF_IP]) {
+        fprintf(out, "/%s", end->strings[KSCF_IP]);
+    }
+    fputc('\n', out);
 
     if(end->strings_set[KSCF_ID] && end->id) {
 	fprintf(out, "\t%sid=\"%s\"\n",     side, end->id);
@@ -304,11 +312,11 @@ void confwrite_side(FILE *out,
     case KH_NOTSET:
 	/* nothing! */
 	break;
-	
+
     case KH_DEFAULTROUTE:
 	fprintf(out, "\t%snexthop=%%defaultroute\n",side);
 	break;
-	
+
     case KH_IPADDR:
 	addrtot(&end->nexthop, 0, databuf, ADDRTOT_BUF);
 	fprintf(out, "\t%snexthop=%s\n", side, databuf);
@@ -338,7 +346,7 @@ void confwrite_side(FILE *out,
 
     if(end->port || end->protocol) {
 	char b2[32];
-	
+
 	strcpy(b2, "%any");
 	strcpy(databuf, "%any");
 
@@ -348,15 +356,11 @@ void confwrite_side(FILE *out,
 	if(end->protocol) {
 	    sprintf(databuf, "%u", end->protocol);
 	}
-	    
+
 	fprintf(out, "\t%sprotoport=%s/%s\n", side,
 		databuf, b2);
     }
 
-    if(end->cert) {
-	fprintf(out, "\t%scert=%s\n", side, end->cert);
-    }
-
     if(!isanyaddr(&end->sourceip)) {
 	addrtot(&end->sourceip, 0, databuf, ADDRTOT_BUF);
 	fprintf(out, "\t%ssourceip=%s\n", side, databuf);
@@ -380,7 +384,7 @@ void confwrite_comments(FILE *out, struct starter_conn *conn)
 	sc != NULL;
 	sc = scnext) {
 	scnext = sc->link.tqe_next;
-	
+
 	fprintf(out, "\t%s=%s\n",
 		sc->x_comment, sc->commentvalue);
     }
@@ -398,17 +402,17 @@ void confwrite_conn(FILE *out,
     }
 
     fprintf(out,"# begin conn %s\n",conn->name);
-    
+
     fprintf(out, "conn %s\n", conn->name);
-    
-    if(conn->alsos) 
+
+    if(conn->alsos)
     { /* handle also= as a comment */
-	
+
 	int alsoplace=0;
-	fprintf(out, "\t#also = ");
+	fprintf(out, "\t#also =");
 	while(conn->alsos[alsoplace] != NULL)
 	{
-	    fprintf(out, "%s ", conn->alsos[alsoplace]);
+	    fprintf(out, " %s", conn->alsos[alsoplace]);
 	    alsoplace++;
 	}
 	fprintf(out, "\n");
@@ -438,15 +442,15 @@ void confwrite_conn(FILE *out,
 	case STARTUP_POLICY:
 	    fprintf(out, "\tauto=policy\n");
 	    break;
-	    
+
 	case STARTUP_ADD:
 	    fprintf(out, "\tauto=add\n");
 	    break;
-	    
+
 	case STARTUP_ROUTE:
 	    fprintf(out, "\tauto=route\n");
 	    break;
-	    
+
 	case STARTUP_START:
 	    fprintf(out, "\tauto=start\n");
 	    break;
@@ -469,13 +473,13 @@ void confwrite_conn(FILE *out,
 	    } else {
 		fprintf(out, "\ttype=transport\n");
 	    }
-	    
+
 	    if(conn->policy & POLICY_COMPRESS) {
 		fprintf(out, "\tcompress=yes\n");
 	    } else {
 		fprintf(out, "\tcompress=no\n");
 	    }
-	    
+
 	    if(conn->policy & POLICY_PFS) {
 		fprintf(out, "\tpfs=yes\n");
 	    } else {
@@ -493,35 +497,35 @@ void confwrite_conn(FILE *out,
 	    } else {
 		fprintf(out, "\toverlapip=no\n");
 	    }
-	    
+
 	    auth_policy=(conn->policy & POLICY_ID_AUTH_MASK);
 	    switch(auth_policy) {
 	    case POLICY_PSK:
 		fprintf(out, "\tauthby=secret\n");
 		break;
-		
+
 	    case POLICY_RSASIG:
 		fprintf(out, "\tauthby=rsasig\n");
 		break;
-		
+
 	    default:
 		fprintf(out, "\tauthby=never\n");
 		break;
 	    }
-	    
+
 	    switch(phase2_policy) {
 	    case POLICY_AUTHENTICATE:
 		fprintf(out, "\tphase2=ah\n");
 		break;
-		
+
 	    case POLICY_ENCRYPT:
 		fprintf(out, "\tphase2=esp\n");
 		break;
-		
+
 	    case (POLICY_ENCRYPT|POLICY_AUTHENTICATE):
 		fprintf(out, "\tphase2=ah+esp\n");
 		break;
-		
+
 	    default:
 		break;
 	    }
@@ -529,15 +533,15 @@ void confwrite_conn(FILE *out,
 	    switch(failure_policy) {
 	    case POLICY_FAIL_NONE:
 		break;
-		
+
 	    case POLICY_FAIL_PASS:
 		fprintf(out, "\tfailureshunt=passthrough\n");
 		break;
-		
+
 	    case POLICY_FAIL_DROP:
 		fprintf(out, "\tfailureshunt=drop\n");
 		break;
-		
+
 	    case POLICY_FAIL_REJECT:
 		fprintf(out, "\tfailureshunt=reject\n");
 		break;
@@ -545,18 +549,18 @@ void confwrite_conn(FILE *out,
 
 	    switch(ikev2_policy) {
 	    case 0:
-		fprintf(out, "\tikev2=never\n"); 
+		fprintf(out, "\tikev2=never\n");
 		break;
 
 	    case POLICY_IKEV2_ALLOW:
 		/* it's the default, do not print anything */
 		/* fprintf(out, "\tikev2=permit\n"); */
 		break;
-		
+
 	    case POLICY_IKEV2_ALLOW|POLICY_IKEV2_PROPOSE:
 		fprintf(out, "\tikev2=propose\n");
 		break;
-		
+
 	    case POLICY_IKEV1_DISABLE|POLICY_IKEV2_ALLOW|POLICY_IKEV2_PROPOSE:
 		fprintf(out, "\tikev2=insist\n");
 		break;
@@ -566,20 +570,20 @@ void confwrite_conn(FILE *out,
 	case POLICY_SHUNT_PASS:
 	    fprintf(out, "\ttype=passthrough\n");
 	    break;
-	    
+
 	case POLICY_SHUNT_DROP:
 	    fprintf(out, "\ttype=drop\n");
 	    break;
-	    
+
 	case POLICY_SHUNT_REJECT:
 	    fprintf(out, "\ttype=reject\n");
 	    break;
 
 	}
-	
+
     }
 
-    
+
     fprintf(out,"# end conn %s\n\n",conn->name);
 }
 
@@ -593,10 +597,10 @@ void confwrite(struct starter_config *cfg, FILE *out)
 
 	/* output config setup section */
 	fprintf(out, "config setup\n");
-	confwrite_int(out, "", 
+	confwrite_int(out, "",
 		      kv_config, 0,
 		      cfg->setup.options, cfg->setup.options_set, cfg->setup.strings);
-	confwrite_str(out, "", 
+	confwrite_str(out, "",
 		      kv_config, 0,
 		      cfg->setup.strings, cfg->setup.strings_set);
 
