@@ -1,8 +1,12 @@
 /*
  * Openswan config file parser (keywords.c)
- * Copyright (C) 2003-2006 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2003-2006,2015 Michael Richardson <mcr@xelerance.com>
  * Copyright (C) 2007-2010 Paul Wouters <paul@xelerance.com>
  * Copyright (C) 2012 Paul Wouters <paul@libreswan.org>
+ * Copyright (C) 2013 Paul Wouters <pwouters@redhat.com>
+ * Copyright (C) 2013 D. Hugh Redelmeier <hugh@mimosa.com>
+ * Copyright (C) 2013 David McCullough <ucdevel@gmail.com>
+ * Copyright (C) 2013 Antony Antony <antony@phenome.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -33,36 +37,60 @@
 #include "parser.tab.h"
 #include "ipsecconf/parserlast.h"
 
+#define VALUES_INITIALIZER(t)	{ t, sizeof(t)/ sizeof(t[0]) }
+#undef KEYWORD_PARSE_DEBUG 
+
+/*
+ * values for keyword types  (used for debugging)
+ */
+static const struct keyword_enum_value kt_values[]={
+    KEV_LITERAL(kt_string),
+    KEV_LITERAL(kt_appendstring),
+    KEV_LITERAL(kt_appendlist),
+    KEV_LITERAL(kt_filename),
+    KEV_LITERAL(kt_dirname),
+    KEV_LITERAL(kt_bool),
+    KEV_LITERAL(kt_invertbool),
+    KEV_LITERAL(kt_enum),
+    KEV_LITERAL(kt_list),
+    KEV_LITERAL(kt_loose_enum),
+    KEV_LITERAL(kt_rsakey),
+    KEV_LITERAL(kt_number),
+    KEV_LITERAL(kt_time),
+    KEV_LITERAL(kt_percent),
+    KEV_LITERAL(kt_ipaddr),
+    KEV_LITERAL(kt_subnet),
+    KEV_LITERAL(kt_idtype),
+    KEV_LITERAL(kt_bitstring),
+    KEV_LITERAL(kt_comment),
+};
+static const struct keyword_enum_values kt_values_list = VALUES_INITIALIZER(kt_values);
 
 /*
  * Values for failureshunt={passthrough, drop, reject, none}
  */
-struct keyword_enum_value kw_failureshunt_values[]={
+static const struct keyword_enum_value kw_failureshunt_values[]={
     { "none",        POLICY_FAIL_NONE },
     { "passthrough", POLICY_FAIL_PASS },
     { "drop",        POLICY_FAIL_DROP },
     { "reject",      POLICY_FAIL_REJECT },
 };
 
-struct keyword_enum_values kw_failureshunt_list=
-    { kw_failureshunt_values, sizeof(kw_failureshunt_values)/sizeof(struct keyword_enum_value)};
-
-
+static const struct keyword_enum_values kw_failureshunt_list = VALUES_INITIALIZER(kw_failureshunt_values);
 
 /*
  * Values for keyexchange=
  */
-struct keyword_enum_value kw_keyexchange_values[]={
+static const struct keyword_enum_value kw_keyexchange_values[]={
     { "ike",  KE_IKE },
 };
 
-struct keyword_enum_values kw_keyexchange_list=
-    { kw_keyexchange_values, sizeof(kw_keyexchange_values)/sizeof(struct keyword_enum_value)};
+static const struct keyword_enum_values kw_keyexchange_list = VALUES_INITIALIZER(kw_keyexchange_values);
 
 /*
  * Values for Four-State options, such as ikev2
  */
-struct keyword_enum_value kw_fourvalued_values[]={
+static const struct keyword_enum_value kw_fourvalued_values[]={
     { "never",     fo_never  },
     { "permit",    fo_permit },
     { "propose",   fo_propose},
@@ -71,25 +99,35 @@ struct keyword_enum_value kw_fourvalued_values[]={
     { "always",    fo_insist },
     { "no",        fo_never  }
 };
-struct keyword_enum_values kw_fourvalued_list=
-{ kw_fourvalued_values, sizeof(kw_fourvalued_values)/sizeof(struct keyword_enum_value)};
+
+static const struct keyword_enum_values kw_fourvalued_list = VALUES_INITIALIZER(kw_fourvalued_values);
+
+/*
+ * Values for yes/no/force, used by ike_frag
+ */
+static const struct keyword_enum_value kw_ynf_values[] = {
+	{ "never",     ynf_no },
+	{ "no",        ynf_no },
+	{ "yes",       ynf_yes },
+	{ "insist",     ynf_force },
+	{ "force",     ynf_force },
+};
+static const struct keyword_enum_values kw_ynf_list = VALUES_INITIALIZER(kw_ynf_values);
 
 /*
  * Values for authby={rsasig, secret}
  */
-struct keyword_enum_value kw_authby_values[]={
+static const struct keyword_enum_value kw_authby_values[]={
     { "never",     0},
     { "rsasig",    POLICY_RSASIG},
     { "secret",    POLICY_PSK},
 };
-
-struct keyword_enum_values kw_authby_list=
-    { kw_authby_values, sizeof(kw_authby_values)/sizeof(struct keyword_enum_value)};
+static const struct keyword_enum_values kw_authby_list = VALUES_INITIALIZER(kw_authby_values);
 
 /*
- * Values for dpdaction={hold,clear,restart} 
+ * Values for dpdaction={hold,clear,restart}
  */
-struct keyword_enum_value kw_dpdaction_values[]={
+static const struct keyword_enum_value kw_dpdaction_values[]={
     { "hold",    DPD_ACTION_HOLD},
     { "clear",   DPD_ACTION_CLEAR},
     { "restart",   DPD_ACTION_RESTART},
@@ -103,7 +141,7 @@ struct keyword_enum_values kw_dpdaction_list=
 /*
  * Values for auto={add,start,route,ignore}
  */
-struct keyword_enum_value kw_auto_values[]={
+static const struct keyword_enum_value kw_auto_values[]={
     { "ignore", STARTUP_NO },
     { "add",    STARTUP_ADD },
     { "route",  STARTUP_ROUTE },
@@ -111,24 +149,22 @@ struct keyword_enum_value kw_auto_values[]={
     { "up",     STARTUP_START }, /* alias */
 };
 
-struct keyword_enum_values kw_auto_list=
-    { kw_auto_values, sizeof(kw_auto_values)/sizeof(struct keyword_enum_value)};
+static const struct keyword_enum_values kw_auto_list = VALUES_INITIALIZER(kw_auto_values);
 
 /*
  * Values for connaddrfamily={ipv4,ipv6}
  */
-struct keyword_enum_value kw_connaddrfamily_values[]={
+static const struct keyword_enum_value kw_connaddrfamily_values[]={
     { "ipv4",   AF_INET },
     { "ipv6",   AF_INET6 },
 };
 
-struct keyword_enum_values kw_connaddrfamily_list=
-    { kw_connaddrfamily_values, sizeof(kw_connaddrfamily_values)/sizeof(struct keyword_enum_value)};
+static const struct keyword_enum_values kw_connaddrfamily_list = VALUES_INITIALIZER(kw_connaddrfamily_values);
 
 /*
  * Values for type={tunnel,transport,udpencap}
  */
-struct keyword_enum_value kw_type_values[]={
+static const struct keyword_enum_value kw_type_values[]={
     { "tunnel",    KS_TUNNEL },
     { "transport", KS_TRANSPORT },
     { "udp",       KS_UDPENCAP },
@@ -139,33 +175,28 @@ struct keyword_enum_value kw_type_values[]={
     { "drop",      KS_DROP },
 };
 
-struct keyword_enum_values kw_type_list=
-    { kw_type_values, sizeof(kw_type_values)/sizeof(struct keyword_enum_value)};
-
-
+static const struct keyword_enum_values kw_type_list = VALUES_INITIALIZER(kw_type_values);
 
 /*
  * Values for rsasigkey={%dnsondemand, %dns, literal }
  */
-struct keyword_enum_value kw_rsasigkey_values[]={
+static const struct keyword_enum_value kw_rsasigkey_values[]={
     { "",             PUBKEY_PREEXCHANGED },
     { "%cert",        PUBKEY_CERTIFICATE },
     { "%dns",         PUBKEY_DNS },
     { "%dnsondemand", PUBKEY_DNSONDEMAND },
 };
 
-struct keyword_enum_values kw_rsasigkey_list=
-    { kw_rsasigkey_values, sizeof(kw_rsasigkey_values)/sizeof(struct keyword_enum_value)};
-
+static const struct keyword_enum_values kw_rsasigkey_list = VALUES_INITIALIZER(kw_rsasigkey_values);
 
 /*
  * Values for protostack={klips, none, auto, klipsmast, netkey }
  */
-struct keyword_enum_value kw_proto_stack_list[]={
+static const struct keyword_enum_value kw_proto_stack_list[]={
     { "none",         NO_KERNEL },
     { "auto",         AUTO_PICK },
     { "klips",        USE_KLIPS },
-    { "mast",         USE_MASTKLIPS }, 
+    { "mast",         USE_MASTKLIPS },
     { "netkey",       USE_NETKEY },
     { "native",       USE_NETKEY },
     { "bsd",          USE_BSDKAME },
@@ -174,71 +205,34 @@ struct keyword_enum_value kw_proto_stack_list[]={
     { "win2k",        USE_WIN2K },
 };
 
-struct keyword_enum_values kw_proto_stack=
-    { kw_proto_stack_list, sizeof(kw_proto_stack_list)/sizeof(struct keyword_enum_value)};
+static const struct keyword_enum_values kw_proto_stack = VALUES_INITIALIZER(kw_proto_stack_list);
 
 /*
  * Values for sareftrack={yes, no, conntrack }
  */
-struct keyword_enum_value kw_sareftrack_values[]={
+static const struct keyword_enum_value kw_sareftrack_values[]={
     { "yes",          sat_yes },
     { "no",           sat_no },
     { "conntrack",    sat_conntrack },
 };
 
-struct keyword_enum_values kw_sareftrack_list=
-    { kw_sareftrack_values, sizeof(kw_sareftrack_values)/sizeof(struct keyword_enum_value)};
+static const struct keyword_enum_values kw_sareftrack_list = VALUES_INITIALIZER(kw_sareftrack_values);
 
 /*
  *  Cisco interop: remote peer type
  */
 
-struct keyword_enum_value kw_remote_peer_type_list[]={
+static const struct keyword_enum_value kw_remote_peer_type_list[]={
     { "cisco",         CISCO },
 };
+static const struct keyword_enum_values kw_remote_peer_type = VALUES_INITIALIZER(kw_remote_peer_type_list);
 
-
-struct keyword_enum_values kw_remote_peer_type=
-    { kw_remote_peer_type_list, sizeof(kw_remote_peer_type_list)/sizeof(struct keyword_enum_value)};
-
-struct keyword_enum_value kw_sha2_truncbug_list[]={
-    { "yes",         SHA2_TRUNCBUG_YES },
-    { "no",         SHA2_TRUNCBUG_NO },
+static const struct keyword_enum_value kw_xauthby_list[] = {
+	{ "file",        XAUTHBY_FILE },
+	{ "pam",         XAUTHBY_PAM },
+	{ "alwaysok",    XAUTHBY_ALWAYSOK },
 };
-
-struct keyword_enum_values kw_sha2_truncbug=
-    { kw_sha2_truncbug_list, sizeof(kw_sha2_truncbug_list)/sizeof(struct keyword_enum_value)};
-
- /*
-  *  Network Manager support
-  */ 
-#ifdef HAVE_NM
-struct keyword_enum_value kw_nm_configured_list[]={
-    { "yes",         NM_YES },
-};
-
-struct keyword_enum_values kw_nm_configured=
-    { kw_nm_configured_list, sizeof(kw_nm_configured_list)/sizeof(struct keyword_enum_value)};
-#endif
-
-#ifdef HAVE_LABELED_IPSEC
-struct keyword_enum_value kw_loopback_list[]={
-    { "yes",         LB_YES },
-    { "no",         LB_NO },
-};
-
-struct keyword_enum_values kw_loopback=
-    { kw_loopback_list, sizeof(kw_loopback_list)/sizeof(struct keyword_enum_value)};
-
-
-struct keyword_enum_value kw_labeled_ipsec_list[]={
-    { "yes",         LI_YES },
-    { "no",         LI_NO },
-};
-
-struct keyword_enum_values kw_labeled_ipsec=
-    { kw_labeled_ipsec_list, sizeof(kw_labeled_ipsec_list)/sizeof(struct keyword_enum_value)};
-#endif
+static const struct keyword_enum_values kw_xauthby = VALUES_INITIALIZER(kw_xauthby_list);
 
 /*
  * Values for right= and left=
@@ -246,8 +240,7 @@ struct keyword_enum_values kw_labeled_ipsec=
 extern struct keyword_enum_values kw_host_list;
 
 
-#ifdef DEBUG
-struct keyword_enum_value kw_plutodebug_values[]={
+static const struct keyword_enum_value kw_plutodebug_values[]={
     { "none",     DBG_NONE },
     { "all",      DBG_ALL },
     { "raw",      DBG_RAW },
@@ -263,8 +256,8 @@ struct keyword_enum_value kw_plutodebug_values[]={
     { "controlmore", DBG_CONTROLMORE },
     { "private",  DBG_PRIVATE },
     { "x509",     DBG_X509 },
-    { "dpd",      DBG_DPD }, 
-    { "pfkey",    DBG_PFKEY }, 
+    { "dpd",      DBG_DPD },
+    { "pfkey",    DBG_PFKEY },
     { "natt",     DBG_NATT },
     { "nattraversal", DBG_NATT },
 
@@ -279,7 +272,7 @@ struct keyword_enum_values kw_plutodebug_list=
     { kw_plutodebug_values, sizeof(kw_plutodebug_values)/sizeof(struct keyword_enum_value)};
 
 
-struct keyword_enum_value kw_klipsdebug_values[]={
+static const struct keyword_enum_value kw_klipsdebug_values[]={
     { "all",      LRANGE(KDF_XMIT, KDF_COMP) },
     { "none",     0 },
     { "verbose",  LELEM(KDF_VERBOSE) },
@@ -303,24 +296,22 @@ struct keyword_enum_value kw_klipsdebug_values[]={
 
 struct keyword_enum_values kw_klipsdebug_list=
     { kw_klipsdebug_values, sizeof(kw_klipsdebug_values)/sizeof(struct keyword_enum_value)};
-#endif
 
-struct keyword_enum_value kw_phase2types_values[]={
+static const struct keyword_enum_value kw_phase2types_values[]={
     { "ah+esp",   POLICY_ENCRYPT|POLICY_AUTHENTICATE },
     { "esp",      POLICY_ENCRYPT },
     { "ah",       POLICY_AUTHENTICATE },
     { "default",  POLICY_ENCRYPT },     /* alias, find it last */
 };
 
-struct keyword_enum_values kw_phase2types_list=
-    { kw_phase2types_values, sizeof(kw_phase2types_values)/sizeof(struct keyword_enum_value)};
+static const struct keyword_enum_values kw_phase2types_list = VALUES_INITIALIZER(kw_phase2types_values);
 
 /*
  * Values for {left/right}sendcert={never,sendifasked,always,forcedtype}
  */
-struct keyword_enum_value kw_sendcert_values[]={
+static const struct keyword_enum_value kw_sendcert_values[]={
     { "never",        cert_neversend },
-    { "sendifasked",  cert_sendifasked }, 
+    { "sendifasked",  cert_sendifasked },
     { "alwayssend",   cert_alwayssend },
     { "always",       cert_alwayssend },
     { "forcedtype",   cert_forcedtype },
@@ -334,10 +325,8 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"interfaces",     kv_config, kt_string,    KSF_INTERFACES,NOT_ENUM},
     {"myid",           kv_config, kt_string,    KSF_MYID,NOT_ENUM},
     {"syslog",         kv_config, kt_string,    KSF_SYSLOG,NOT_ENUM},
-#ifdef DEBUG
     {"klipsdebug",     kv_config, kt_list,      KBF_KLIPSDEBUG, &kw_klipsdebug_list},
     {"plutodebug",     kv_config, kt_list,      KBF_PLUTODEBUG, &kw_plutodebug_list},
-#endif
     {"plutoopts",      kv_config, kt_string,    KSF_PLUTOOPTS,NOT_ENUM},
     {"plutostderrlog", kv_config, kt_filename,  KSF_PLUTOSTDERRLOG,NOT_ENUM},
     {"plutostderrlogtime",        kv_config, kt_bool,      KBF_PLUTOSTDERRLOGTIME,NOT_ENUM},
@@ -357,51 +346,45 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"strictcrlpolicy",kv_config, kt_bool,      KBF_STRICTCRLPOLICY,NOT_ENUM},
     {"crlcheckinterval",kv_config,kt_time,      KBF_CRLCHECKINTERVAL,NOT_ENUM},
     {"force_busy",     kv_config, kt_bool,      KBF_FORCEBUSY,NOT_ENUM},
-#ifdef NAT_TRAVERSAL
     {"virtual_private",kv_config,kt_string,     KSF_VIRTUALPRIVATE,NOT_ENUM},
     {"nat_traversal", kv_config,kt_bool,        KBF_NATTRAVERSAL, NOT_ENUM},
     {"disable_port_floating", kv_config,kt_bool,KBF_DISABLEPORTFLOATING, NOT_ENUM},
     {"keep_alive", kv_config,kt_number,    KBF_KEEPALIVE, NOT_ENUM},
     {"force_keepalive", kv_config,kt_bool,    KBF_FORCE_KEEPALIVE, NOT_ENUM},
-#endif
     {"listen",     kv_config, kt_string, KSF_LISTEN,NOT_ENUM},
     {"protostack",     kv_config, kt_string,    KSF_PROTOSTACK, &kw_proto_stack},
     {"nhelpers",kv_config,kt_number, KBF_NHELPERS, NOT_ENUM},
-#ifdef HAVE_LABELED_IPSEC
     {"secctx_attr_value",kv_config,kt_number, KBF_SECCTX, NOT_ENUM},
-#endif
     /* these two options are obsoleted. Don't die on them */
-    {"forwardcontrol", kv_config, kt_obsolete, KBF_WARNIGNORE,NOT_ENUM},
-    {"rp_filter",      kv_config, kt_obsolete, KBF_WARNIGNORE,NOT_ENUM},
+    {"forwardcontrol", kv_config|kv_obsolete, kt_string, KBF_WARNIGNORE,NOT_ENUM},
+    {"rp_filter",      kv_config|kv_obsolete, kt_string, KBF_WARNIGNORE,NOT_ENUM},
 
     /* this is "left=" and "right=" */
-    {"",               kv_conn|kv_leftright, kt_loose_enum, KSCF_IP, &kw_host_list},  
+    {"",               kv_conn|kv_leftright, kt_loose_enumarg, KSCF_IP, &kw_host_list, LOOSE_ENUM_OTHER, '/'},
 
     {"ike",            kv_conn|kv_auto, kt_string, KSF_IKE,NOT_ENUM},
 
-    {"subnet",         kv_conn|kv_auto|kv_leftright|kv_processed, kt_subnet, KSCF_SUBNET,NOT_ENUM}, 
-    {"subnets",        kv_conn|kv_auto|kv_leftright, kt_appendlist, KSCF_SUBNETS,NOT_ENUM}, 
-    {"sourceip",       kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_SOURCEIP,NOT_ENUM}, 
+    {"subnet",         kv_conn|kv_auto|kv_leftright|kv_processed, kt_subnet, KSCF_SUBNET,NOT_ENUM},
+    {"subnets",        kv_conn|kv_auto|kv_leftright, kt_appendlist, KSCF_SUBNETS,NOT_ENUM},
+    {"sourceip",       kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_SOURCEIP,NOT_ENUM},
     {"nexthop",        kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_NEXTHOP,NOT_ENUM},
-#ifdef OBSOLETE
-    {"firewall",       kv_conn|kv_auto|kv_leftright, kt_bool,   KNCF_FIREWALL,NOT_ENUM},
-#endif
+    {"firewall",       kv_conn|kv_auto|kv_leftright|kv_obsolete, kt_bool,   KNCF_FIREWALL,NOT_ENUM},
     {"updown",         kv_conn|kv_auto|kv_leftright, kt_filename, KSCF_UPDOWN,NOT_ENUM},
     {"id",             kv_conn|kv_auto|kv_leftright, kt_idtype, KSCF_ID,NOT_ENUM},
-    {"rsasigkey",      kv_conn|kv_auto|kv_leftright, kt_rsakey, KSCF_RSAKEY1, &kw_rsasigkey_list},
-    {"rsasigkey2",     kv_conn|kv_auto|kv_leftright, kt_rsakey, KSCF_RSAKEY2, &kw_rsasigkey_list},
-#ifdef USE_MANUAL_KEYING
+    {"rsasigkey",      kv_conn|kv_auto|kv_leftright, kt_rsakey, KSCF_RSAKEY1, &kw_rsasigkey_list, PUBKEY_PREEXCHANGED},
+    {"rsasigkey2",     kv_conn|kv_auto|kv_leftright, kt_rsakey, KSCF_RSAKEY2, &kw_rsasigkey_list, PUBKEY_PREEXCHANGED},
     {"spibase",        kv_conn|kv_auto|kv_leftright, kt_number, KNCF_SPIBASE,NOT_ENUM},
-#endif
     {"cert",           kv_conn|kv_auto|kv_leftright, kt_filename, KSCF_CERT,NOT_ENUM},
     {"sendcert",       kv_conn|kv_auto|kv_leftright, kt_enum,   KNCF_SENDCERT, &kw_sendcert_list},
     {"ca",             kv_conn|kv_auto|kv_leftright, kt_string, KSCF_CA,NOT_ENUM},
 
     /* these are conn statements which are not left/right */
     {"auto",           kv_conn|kv_duplicateok, kt_enum,   KBF_AUTO,        &kw_auto_list},
-    {"also",           kv_conn,         kt_appendstring, KSF_ALSO,NOT_ENUM},
-    {"alsoflip",       kv_conn,         kt_string, KSF_ALSOFLIP,NOT_ENUM},
-    {"connaddrfamily", kv_conn,         kt_enum,   KBF_CONNADDRFAMILY,     &kw_connaddrfamily_list},
+    {"also",           kv_conn,         kt_appendstring, KSF_ALSO,    NOT_ENUM},
+    {"alsoflip",       kv_conn,         kt_appendstring, KSF_ALSOFLIP,NOT_ENUM},
+    {"endaddrfamily",  kv_conn,         kt_enum,   KBF_ENDADDRFAMILY,     &kw_connaddrfamily_list},
+    {"connaddrfamily", kv_conn,         kt_enum,   KBF_CONNADDRFAMILY,    &kw_connaddrfamily_list},
+    {"clientaddrfamily", kv_conn,       kt_enum,   KBF_CONNADDRFAMILY,    &kw_connaddrfamily_list},
     {"type",           kv_conn,         kt_enum,   KBF_TYPE,        &kw_type_list},
     {"authby",         kv_conn|kv_auto, kt_enum,   KBF_AUTHBY,     &kw_authby_list},
     {"keyexchange",    kv_conn|kv_auto, kt_enum,   KBF_KEYEXCHANGE, &kw_keyexchange_list},
@@ -409,27 +392,21 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"narrowing",      kv_conn|kv_auto, kt_bool,   KBF_IKEv2_ALLOW_NARROWING, NOT_ENUM},
     {"sareftrack",     kv_conn|kv_auto|kv_processed,kt_enum,KBF_SAREFTRACK,&kw_sareftrack_list},
     {"pfs",            kv_conn|kv_auto, kt_bool,   KBF_PFS,          NOT_ENUM},
-    {"sha2_truncbug",  kv_conn|kv_auto, kt_enum,   KBF_SHA2_TRUNCBUG,          &kw_sha2_truncbug},
+    {"sha2_truncbug",  kv_conn|kv_auto, kt_bool,   KBF_SHA2_TRUNCBUG,NOT_ENUM},
     {"keylife",        kv_conn|kv_auto|kv_alias, kt_time,   KBF_SALIFETIME,NOT_ENUM},
     {"lifetime",       kv_conn|kv_auto|kv_alias, kt_time,   KBF_SALIFETIME,NOT_ENUM},
     {"salifetime",     kv_conn|kv_auto, kt_time,   KBF_SALIFETIME,NOT_ENUM},
-#ifdef HAVE_LABELED_IPSEC
-    {"loopback",       kv_conn|kv_auto, kt_enum, KBF_LOOPBACK, &kw_loopback},
-    {"labeled_ipsec",   kv_conn|kv_auto, kt_enum, KBF_LABELED_IPSEC, &kw_labeled_ipsec},
-    {"policy_label",    kv_conn|kv_auto,         kt_string, KSF_POLICY_LABEL, NOT_ENUM},
-#endif
+    {"loopback",       kv_conn|kv_auto, kt_bool,   KBF_LOOPBACK, NOT_ENUM},
+    {"labeled_ipsec",  kv_conn|kv_auto, kt_bool,   KBF_LABELED_IPSEC, NOT_ENUM},
+    {"policy_label",   kv_conn|kv_auto,         kt_string, KSF_POLICY_LABEL, NOT_ENUM},
 
     /* Cisco interop: remote peer type*/
     {"remote_peer_type", kv_conn|kv_auto, kt_enum, KBF_REMOTEPEERTYPE, &kw_remote_peer_type},
 
     /* Network Manager support*/
-#ifdef HAVE_NM
-    {"nm_configured", kv_conn|kv_auto, kt_enum, KBF_NMCONFIGURED, &kw_nm_configured},
-#endif
+    {"nm_configured", kv_conn|kv_auto, kt_bool,    KBF_NMCONFIGURED, NOT_ENUM},
 
-#ifdef NAT_TRAVERSAL
     {"forceencaps",    kv_conn|kv_auto, kt_bool,   KBF_FORCEENCAP, NOT_ENUM},
-#endif
     {"overlapip",      kv_conn|kv_auto, kt_bool,   KBF_OVERLAPIP, NOT_ENUM},
     {"rekey",          kv_conn|kv_auto, kt_bool,   KBF_REKEY, NOT_ENUM},
     {"rekeymargin",    kv_conn|kv_auto, kt_time,   KBF_REKEYMARGIN,NOT_ENUM},
@@ -453,14 +430,14 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     /* route metric */
     {"metric",         kv_conn|kv_auto, kt_number, KBF_METRIC, NOT_ENUM},
 
-    /* DPD */ 
+    /* DPD */
     {"dpddelay",       kv_conn|kv_auto,kt_number, KBF_DPDDELAY, NOT_ENUM},
     {"dpdtimeout",     kv_conn|kv_auto,kt_number,KBF_DPDTIMEOUT , NOT_ENUM},
     {"dpdaction",      kv_conn|kv_auto,kt_enum, KBF_DPDACTION , &kw_dpdaction_list},
 
     {"mtu",            kv_conn|kv_auto,kt_number, KBF_CONNMTU, NOT_ENUM},
 
-    /* aggr/xauth/modeconfig */ 
+    /* aggr/xauth/modeconfig */
     {"aggrmode",    kv_conn|kv_auto, kt_invertbool,      KBF_AGGRMODE,NOT_ENUM},
     {"xauthserver", kv_conn|kv_auto|kv_leftright, kt_bool, KNCF_XAUTHSERVER,  NOT_ENUM},
     {"xauthclient", kv_conn|kv_auto|kv_leftright, kt_bool, KNCF_XAUTHCLIENT, NOT_ENUM},
@@ -474,12 +451,14 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"modecfgwins1", kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_MODECFGWINS1,NOT_ENUM},
     {"modecfgwins2", kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_MODECFGWINS2,NOT_ENUM},
     /* things for manual keying only */
-#ifdef USE_MANUAL_KEYING
     {"spi",            kv_conn|kv_leftright|kv_manual, kt_number, KNCF_SPI,NOT_ENUM},
     {"espenckey",      kv_conn|kv_leftright|kv_manual, kt_bitstring, KSCF_ESPENCKEY,NOT_ENUM},
     {"espauthkey",     kv_conn|kv_leftright|kv_manual, kt_bitstring, KSCF_ESPAUTHKEY,NOT_ENUM},
-    {"espreplay_window",kv_conn|kv_leftright|kv_manual, kt_number, KNCF_ESPREPLAYWINDOW,NOT_ENUM}, 
-#endif
+    {"espreplay_window",kv_conn|kv_leftright|kv_manual, kt_number, KNCF_ESPREPLAYWINDOW,NOT_ENUM},
+
+    /* some things from libreswan, which we will probably accept */
+    { "plutofork",      kv_config|kv_obsolete, kt_bool,      KBF_PLUTOFORK,  NOT_ENUM },
+
     {NULL, 0, 0, 0, NOT_ENUM}
 };
 
@@ -492,11 +471,11 @@ const int ipsec_conf_keywords_v2_count = sizeof(ipsec_conf_keywords_v2)/sizeof(s
 
 /*
  * look for one of the above tokens, and set the value up right.
- * 
+ *
  * if we don't find it, then strdup() the string and return a string
  *
  */
-   
+
 /* type is really "token" type, which is actually int */
 int parser_find_keyword(const char *s, YYSTYPE *lval)
 {
@@ -528,7 +507,7 @@ int parser_find_keyword(const char *s, YYSTYPE *lval)
 		break;
 	    }
 	}
-	
+
 	k++;
     }
 
@@ -577,7 +556,7 @@ unsigned int parser_enum_list(struct keyword_def *kd, const char *s, bool list)
     char *piece;
     char *scopy;
     int   numfound, kevcount;
-    struct keyword_enum_value *kev;
+    const struct keyword_enum_value *kev;
     unsigned int valresult;
     char complaintbuf[80];
 
@@ -619,7 +598,7 @@ unsigned int parser_enum_list(struct keyword_def *kd, const char *s, bool list)
 		     , "%s: %d: keyword %s, invalid value: %s"
 		     , parser_cur_filename(), parser_cur_lineno()
 		     , kd->keyname, piece);
-	 
+
 	    if(warningsarefatal)
 	    {
 		fprintf(stderr, "ERROR: %s\n", complaintbuf);
@@ -655,17 +634,26 @@ unsigned int parser_enum_list(struct keyword_def *kd, const char *s, bool list)
     free(scopy);
     return valresult;
 }
-	
-unsigned int parser_loose_enum(struct keyword *k, const char *s)
+
+unsigned int parser_loose_enum_arg(struct keyword *k, const char *s, char **rest)
 {
     struct keyword_def *kd = k->keydef;
     int   kevcount;
-    struct keyword_enum_value *kev;
+    const struct keyword_enum_value *kev;
     unsigned int valresult;
 
-    assert(kd->type == kt_loose_enum || kd->type == kt_rsakey);
+    assert(kd->type == kt_loose_enum || kd->type == kt_loose_enumarg || kd->type == kt_rsakey);
     assert(kd->validenum != NULL && kd->validenum->values != NULL);
 
+    if(kd->deliminator != '\0') {
+        char *nl = strchr(s, kd->deliminator);
+        if(nl) {
+            *nl='\0';
+            nl++;
+            if(rest) *rest = nl;
+        }
+    }
+
     for(kevcount = kd->validenum->valuesize, kev = kd->validenum->values;
 	kevcount > 0 && strcasecmp(s, kev->name)!=0;
 	kev++, kevcount--);
@@ -679,8 +667,16 @@ unsigned int parser_loose_enum(struct keyword *k, const char *s)
 	return valresult;
     }
 
+#ifdef KEYWORD_PARSE_DEBUG
+    {
+        char kdtypebuf[KEYWORD_NAME_BUFLEN];
+        fprintf(stderr, "loose enum(%s) for %s is %d\n", s, keyword_name(&kt_values_list, kd->type, kdtypebuf),
+                kd->loose_enum_value);
+    }
+#endif
+    /* else, it's a literal other value */
     k->string = strdup(s);
-    return 255;
+    return kd->loose_enum_value;
 }
 
 
@@ -690,6 +686,6 @@ unsigned int parser_loose_enum(struct keyword *k, const char *s)
  * c-style: pluto
  * End:
  */
-	
 
-    
+
+
