@@ -82,13 +82,13 @@ alg_info_esp_v2tov1aa(enum ikev2_trans_type_integ ti)
 	return AUTH_ALGORITHM_HMAC_MD5;
     case IKEv2_AUTH_HMAC_SHA1_96:
 	return AUTH_ALGORITHM_HMAC_SHA1;
-    case IKEv2_AUTH_HMAC_SHA2_256_128: 
+    case IKEv2_AUTH_HMAC_SHA2_256_128:
 	return AUTH_ALGORITHM_HMAC_SHA2_256;
     case IKEv2_AUTH_HMAC_SHA2_256_128_TRUNCBUG:
 	return AUTH_ALGORITHM_HMAC_SHA2_256_TRUNCBUG;
-    case IKEv2_AUTH_HMAC_SHA2_384_192: 
+    case IKEv2_AUTH_HMAC_SHA2_384_192:
 	return AUTH_ALGORITHM_HMAC_SHA2_256;
-    case IKEv2_AUTH_HMAC_SHA2_512_256: 
+    case IKEv2_AUTH_HMAC_SHA2_512_256:
 	return AUTH_ALGORITHM_HMAC_SHA2_256;
 
     /* invalid or not yet supported */
@@ -142,7 +142,7 @@ alg_enum_search_prefix (enum_names *ed, const char *prefix, const char *str, int
 	int ret;
 	int len=sizeof(buf)-1;	/* reserve space for final \0 */
 
-	for (ptr=buf; *prefix; *ptr++=*prefix++, len--);
+	for (ptr=buf; len&&*prefix; *ptr++=*prefix++, len--);
 
 	while (str_len--&&len--&&*str) *ptr++=toupper(*str++);
 	*ptr=0;
@@ -165,7 +165,7 @@ alg_enum_search_ppfix (enum_names *ed, const char *prefix
 	char *ptr;
 	int ret;
 	int len=sizeof(buf)-1;	/* reserve space for final \0 */
-	for (ptr=buf; *prefix; *ptr++=*prefix++, len--);
+	for (ptr=buf; len&&*prefix; *ptr++=*prefix++, len--);
 	while (str_len--&&len--&&*str) *ptr++=toupper(*str++);
 	while (len--&&*postfix) *ptr++=*postfix++;
 	*ptr=0;
@@ -235,13 +235,13 @@ out:
 	return ret;
 }
 
-void 
+void
 alg_info_free(struct alg_info *alg_info) {
 	pfreeany(alg_info);
 }
 
-/*	
- *	Raw add routine: only checks for no duplicates		
+/*
+ *	Raw add routine: only checks for no duplicates
  */
 static void
 __alg_info_esp_add (struct alg_info_esp *alg_info
@@ -272,7 +272,7 @@ __alg_info_esp_add (struct alg_info_esp *alg_info
 				ealg_id, aalg_id, alg_info->alg_info_cnt));
 }
 
-/*	
+/*
  *	Add ESP alg info _with_ logic (policy):
  */
 static void
@@ -284,7 +284,7 @@ alg_info_esp_add (struct alg_info *alg_info,
 	/*	Policy: default to 3DES */
 	if (ealg_id==0)
 		ealg_id=ESP_3DES;
-	
+
 	if (ealg_id>0) {
 
 	    if(aalg_id > 0 ||
@@ -309,7 +309,7 @@ alg_info_esp_add (struct alg_info *alg_info,
 	}
 }
 
-/*	
+/*
  *	Add AH alg info _with_ logic (policy):
  */
 static void
@@ -341,7 +341,7 @@ static const char *parser_state_esp_names[] = {
 	"ST_INI",
 	"ST_INI_AA",
 	"ST_EA",
-	"ST_EA_END",	
+	"ST_EA_END",
 	"ST_EK",
 	"ST_EK_END",
 	"ST_AA",
@@ -364,10 +364,10 @@ static inline void parser_set_state(struct parser_context *p_ctx, enum parser_st
 		p_ctx->old_state=p_ctx->state;
 		p_ctx->state=state;
 	}
-	
+
 }
 
-static int 
+static int
 parser_machine(struct parser_context *p_ctx)
 {
 	int ch=p_ctx->ch;
@@ -383,7 +383,7 @@ parser_machine(struct parser_context *p_ctx)
 	    case ST_EA:
 	    case ST_EK:
 	    case ST_AA:
-	    case ST_AK: 
+	    case ST_AK:
 	    case ST_MODP:
 	    case ST_FLAG_STRICT:
 		{
@@ -484,7 +484,7 @@ parser_machine(struct parser_context *p_ctx)
                 *(p_ctx->aalg_str++)=0;
                 parser_set_state(p_ctx, ST_AK_END);
                 break;
-            }	    
+            }
 	    if (isalnum(ch) || ch=='_') {
 		*(p_ctx->aalg_str++)=ch;
 		break;
@@ -539,7 +539,7 @@ parser_machine(struct parser_context *p_ctx)
 	    }
 	    p_ctx->err="Flags character(s) must be at end of whole string";
 	    goto err;
-	    
+
 	    /* XXX */
 	case ST_END:
 	case ST_EOF:
@@ -554,7 +554,7 @@ parser_machine(struct parser_context *p_ctx)
 	return ST_ERR;
 }
 
-/*	
+/*
  *	Must be called for each "new" char, with new
  *	character in ctx.ch
  */
@@ -570,13 +570,13 @@ parser_init_esp(struct parser_context *p_ctx)
     p_ctx->ealg_permit = TRUE;
     p_ctx->aalg_permit = TRUE;
     p_ctx->state=ST_INI;
-    
+
     p_ctx->ealg_getbyname=ealg_getbyname_esp;
     p_ctx->aalg_getbyname=aalg_getbyname_esp;
 
 }
 
-/*	
+/*
  *	Must be called for each "new" char, with new
  *	character in ctx.ch
  */
@@ -592,7 +592,7 @@ parser_init_ah(struct parser_context *p_ctx)
     p_ctx->aalg_permit = TRUE;
     p_ctx->modp_str=p_ctx->modp_buf;
     p_ctx->state=ST_INI_AA;
-    
+
     p_ctx->ealg_getbyname=NULL;
     p_ctx->aalg_getbyname=aalg_getbyname_esp;
 
@@ -623,6 +623,26 @@ parser_alg_info_add(struct parser_context *p_ctx
 		p_ctx->err="enc_alg not found";
 		goto out;
 	    }
+
+	    /* AES_GCM_128, AES_GCM_192, AES_GCM_256 */
+	    if(ealg_id == ESP_AES_GCM_8
+		|| ealg_id == ESP_AES_GCM_12
+		|| ealg_id == ESP_AES_GCM_16) {
+
+			/* AES-GCM length key length + 4 bytes (32 bits) */
+			if( p_ctx->eklen != 128
+				&& p_ctx->eklen != 192
+				&& p_ctx->eklen != 256 ) {
+				p_ctx->err="wrong encryption key length with AES-GCM";
+				goto out;
+			}
+			else {
+				/* increase key length by 4 bytes, RFC 4106 */
+				p_ctx->eklen = p_ctx->eklen + 4 *  BITS_PER_BYTE;
+			}
+
+	    }
+
 	    DBG(DBG_CRYPT, DBG_log("parser_alg_info_add() "
 				   "ealg_getbyname(\"%s\")=%d",
 				   p_ctx->ealg_buf,
@@ -711,18 +731,6 @@ alg_info_parse_str (struct alg_info *alg_info
 
 	    case ST_END:
 	    case ST_EOF:
-		/*
-		 * If we detect sha2_256 with key size 96, we know they really meant
-		 * to configure sha2_256_trunc 256 bit with a hash truncation of 96
-		 */
-#warning paul check and redo me
-#if 0
-		if( !strncmp( ctx.aalg_buf, sha2_256, 8) && (ctx.eklen==0) && (ctx.aklen==96)) {
-		   DBG(DBG_CRYPT,DBG_log(" converting sha2_256-96 to sha2_256_trunc-256"));
-		   strncpy(ctx.aalg_buf, "sha2_256_trunc", sizeof("sha2_256_trunc"));
-		   ctx.aklen = 256;
-		}
-#endif
 
 		DBG(DBG_CRYPT, DBG_log("alg_info_parse_str() "
 				       "ealg_buf=%s aalg_buf=%s "
@@ -755,7 +763,7 @@ alg_info_parse_str (struct alg_info *alg_info
 			 ctx.err,
 			 (int)(ptr-alg_str-1), alg_str ,
 			 parser_state_name_esp(ctx.old_state) );
-		
+
 		goto err;
 	    default:
 		if (!ctx.ch) break;
@@ -777,9 +785,9 @@ alg_info_discover_pfsgroup_hack(struct alg_info_esp *aie
     char *pfs_name;
     static char err_buf[256];
     int ret;
-    
+
     pfs_name=index(esp_buf, ';');
-    
+
     if(pfs_name) {
 	*pfs_name='\0';
 	pfs_name++;
@@ -847,7 +855,7 @@ alg_info_esp_create_from_str (const char *alg_str
 	    alg_info_esp=NULL;
 	}
     return alg_info_esp;
-    
+
 }
 
 struct alg_info_esp *
@@ -892,7 +900,7 @@ alg_info_ah_create_from_str (const char *alg_str
  * 	several connections instances,
  * 	handle free() with ref_cnts
  */
-void 
+void
 alg_info_addref(struct alg_info *alg_info)
 {
     if (alg_info != NULL) {
@@ -933,54 +941,43 @@ alg_info_snprint(char *buf, int buflen
 		 , bool permitike)
 {
     char *ptr=buf;
-    int np=0;
     struct esp_info *esp_info;
     struct ike_info *ike_info;
 
+    passert(buflen > 0);
+
     int cnt;
     ptr=buf;
     switch(alg_info->alg_info_protoid) {
-    case PROTO_IPSEC_ESP: 
+    case PROTO_IPSEC_ESP:
 	{
 	    struct alg_info_esp *alg_info_esp=(struct alg_info_esp *)alg_info;
 	    ALG_INFO_ESP_FOREACH(alg_info_esp, esp_info, cnt) {
-		np=snprintf(ptr, buflen, "%s(%d)_%03d-%s(%d)_%03d"
+		snprintf(ptr, buflen, "%s(%d)_%03d-%s(%d)_%03d"
 			    , enum_name(&esp_transformid_names, esp_info->esp_ealg_id)+sizeof("ESP")
 			    , esp_info->esp_ealg_id
 			    , (int)esp_info->esp_ealg_keylen
 			    , enum_name(&auth_alg_names, esp_info->esp_aalg_id) + (esp_info->esp_aalg_id ? sizeof("AUTH_ALGORITHM_HMAC") : sizeof("AUTH_ALGORITHM"))
 			    , esp_info->esp_aalg_id
 			    , (int)esp_info->esp_aalg_keylen);
-		if(np < buflen) {
-			ptr+=np;
-			buflen-=np;
-		} else {
-			ptr+=buflen;
-			buflen=0;
-               }
+		size_t np = strlen(ptr);
+		ptr += np;
+		buflen -= np;
 		if ( cnt > 0) {
-			np=snprintf(ptr, buflen, ", ");
-			if(np < buflen) {
-				ptr+=np;
-				buflen-=np;
-			} else {
-				ptr+=buflen;
-				buflen=0;
-			}
+			snprintf(ptr, buflen, ", ");
+			np = strlen(ptr);
+			ptr += np;
+			buflen -= np;
 		}
 		if(buflen <= 0) goto out;
 	    }
 	    if (alg_info_esp->esp_pfsgroup) {
-		np=snprintf(ptr, buflen, "; pfsgroup=%s(%d)"
+		snprintf(ptr, buflen, "; pfsgroup=%s(%d)"
 			, enum_name(&oakley_group_names, alg_info_esp->esp_pfsgroup)+ sizeof("OAKLEY_GROUP")
 		        , alg_info_esp->esp_pfsgroup);
-		if(np < buflen) {
-			ptr+=np;
-			buflen-=np;
-		} else {
-			ptr+=buflen;
-			buflen=0;
-		}
+		size_t np = strlen(ptr);
+		ptr += np;
+		buflen -= np;
 		if(buflen <= 0) goto out;
 	    }
 	    break;
@@ -990,40 +987,28 @@ alg_info_snprint(char *buf, int buflen
         {
 	    struct alg_info_esp *alg_info_esp=(struct alg_info_esp *)alg_info;
 	    ALG_INFO_ESP_FOREACH(alg_info_esp, esp_info, cnt) {
-		np=snprintf(ptr, buflen, "%s(%d)_%03d"
+		snprintf(ptr, buflen, "%s(%d)_%03d"
 			    , enum_name(&auth_alg_names, esp_info->esp_aalg_id)+sizeof("AUTH_ALGORITHM_HMAC")
 			    , esp_info->esp_aalg_id
 			    , (int)esp_info->esp_aalg_keylen);
-		if(np < buflen) {
-			ptr+=np;
-			buflen-=np;
-		} else {
-			ptr+=buflen;
-			buflen=0;
-               }
+		size_t np = strlen(ptr);
+		ptr += np;
+		buflen -= np;
 		if ( cnt > 0) {
-			np=snprintf(ptr, buflen, ", ");
-			if(np < buflen) {
-				ptr+=np;
-				buflen-=np;
-			} else {
-				ptr+=buflen;
-				buflen=0;
-			}
+			snprintf(ptr, buflen, ", ");
+			np = strlen(ptr);
+			ptr += np;
+			buflen -= np;
 		}
 		if(buflen <= 0) goto out;
 	    }
 	    if (alg_info_esp->esp_pfsgroup) {
-		np=snprintf(ptr, buflen, "; pfsgroup=%s(%d)"
+		snprintf(ptr, buflen, "; pfsgroup=%s(%d)"
 			, enum_name(&oakley_group_names, alg_info_esp->esp_pfsgroup)+ sizeof("OAKLEY_GROUP")
-		        , alg_info_esp->esp_pfsgroup);
-		if(np < buflen) {
-			ptr+=np;
-			buflen-=np;
-		} else {
-			ptr+=buflen;
-			buflen=0;
-		}
+			, alg_info_esp->esp_pfsgroup);
+		size_t np = strlen(ptr);
+		ptr += np;
+		buflen -= np;
 		if(buflen <= 0) goto out;
 	    }
 	    break;
@@ -1032,7 +1017,7 @@ alg_info_snprint(char *buf, int buflen
     case PROTO_ISAKMP:
 	if(permitike) {
 	    ALG_INFO_IKE_FOREACH((struct alg_info_ike *)alg_info, ike_info, cnt) {
-		np=snprintf(ptr, buflen, "%s(%d)_%03d-%s(%d)_%03d-%s(%d)"
+		snprintf(ptr, buflen, "%s(%d)_%03d-%s(%d)_%03d-%s(%d)"
 			    , enum_name(&oakley_enc_names, ike_info->ike_ealg)+sizeof("OAKLEY")
 			    , ike_info->ike_ealg
 			    , (int)ike_info->ike_eklen
@@ -1041,22 +1026,14 @@ alg_info_snprint(char *buf, int buflen
 			    , (int)ike_info->ike_hklen
 			    , enum_name(&oakley_group_names, ike_info->ike_modp)+ sizeof("OAKLEY_GROUP")
 			    , ike_info->ike_modp);
-		if(np < buflen) {
-			ptr+=np;
-			buflen-=np;
-		} else {
-			ptr+=buflen;
-			buflen=0;
-               }
+		size_t np = strlen(ptr);
+		ptr += np;
+		buflen -= np;
                 if ( cnt > 0) {
-                        np=snprintf(ptr, buflen, ", ");
-			if(np < buflen) {
-				ptr+=np;
-				buflen-=np;
-			} else {
-				ptr+=buflen;
-				buflen=0;
-                        }
+                        snprintf(ptr, buflen, ", ");
+			np = strlen(ptr);
+			ptr += np;
+			buflen -= np;
                 }
 		if(buflen <= 0) goto out;
 	    }
@@ -1065,33 +1042,25 @@ alg_info_snprint(char *buf, int buflen
 	/* FALLTHROUGH */
 
     default:
-	np=snprintf(buf, buflen, "INVALID protoid=%d\n",
-		    alg_info->alg_info_protoid);
-	if(np < buflen) {
-		ptr+=np;
-		buflen-=np;
-	} else {
-		ptr+=buflen;
-		buflen=0;
-        }
+	snprintf(buf, buflen, "INVALID protoid=%d\n",
+		 alg_info->alg_info_protoid);
+	size_t np = strlen(ptr);
+	ptr += np;
+	buflen -= np;
 	goto out;
     }
     if(buflen > 0){
-	np=snprintf(ptr, buflen, "; flags=%s",
+	snprintf(ptr, buflen, "; flags=%s",
 		alg_info->alg_info_flags&ALG_INFO_F_STRICT?
 		"strict":"-strict");
-    	if(np < buflen) {
-		ptr+=np;
-		buflen-=np;
-    	} else {
-		ptr+=buflen;
-		buflen=0;
-    	}
+	size_t np = strlen(ptr);
+	ptr += np;
+	buflen -= np;
     }
 
  out:
     passert(buflen >= 0);
-    
+
     return ptr-buf;
 }
 
