@@ -167,9 +167,18 @@ const char *const payload_name_ikev2_main[] = {
     NULL    /* termination for bitnamesof() */
 };
 
+const char *const payload_name_nat_d[] = {
+    "ISAKMP_NEXT_NAT-D",
+    "ISAKMP_NEXT_NAT-OA",
+    NULL
+};
+
+static enum_names payload_names_nat_d =
+{ ISAKMP_NEXT_NATD_DRAFTS, ISAKMP_NEXT_NATOA_DRAFTS, payload_name_nat_d, NULL };
+
 static enum_names payload_names_ikev2_main =
 { ISAKMP_NEXT_v2SA, ISAKMP_NEXT_v2EAP, payload_name_ikev2_main,
-  NULL };
+  &payload_names_nat_d };
 
 const char *const payload_name_ikev2[] = {
     "ISAKMP_NEXT_v2NONE",            /* 33 */
@@ -1451,6 +1460,7 @@ const char *const natt_type_bitnames[] = {
  * Values for right= and left=
  */
 struct keyword_enum_value kw_host_values[]={
+    { "%unset",         KH_NOTSET },
     { "%defaultroute",  KH_DEFAULTROUTE },
     { "%any",           KH_ANY },
     { "%",              KH_IFACE },
@@ -1459,7 +1469,9 @@ struct keyword_enum_value kw_host_values[]={
     { "%opportunisticgroup", KH_OPPOGROUP },
     { "%oppogroup",     KH_OPPOGROUP },
     { "%group",         KH_GROUP },
-    { "%hostname",      KH_IPHOSTNAME },  /* makes no sense on input */
+    { "%address",       KH_IPADDR },
+    { "%dns",           KH_IPHOSTNAME },
+    { "%hostname",      KH_IPHOSTNAME },  /* alias for above */
 };
 
 struct keyword_enum_values kw_host_list=
@@ -1480,17 +1492,21 @@ enum_name(enum_names *ed, unsigned long val)
 }
 
 /* look up an enum in a starter friendly way */
-const char *keyword_name(struct keyword_enum_values *kevs, unsigned int value)
+const char *keyword_name(const struct keyword_enum_values *kevs
+                         , unsigned int value
+                         , char namebuf[KEYWORD_NAME_BUFLEN])
 {
     int kevcount;
-    struct keyword_enum_value *kev;
+    const struct keyword_enum_value *kev;
 
     for(kevcount = kevs->valuesize, kev = kevs->values;
         kevcount > 0 && kev->value != value;
         kev++, kevcount--);
 
-    passert(kevcount != 0);  /* for now */
-
+    if(kevcount == 0) {
+        snprintf(namebuf, 256, "value:%u", value);
+        return namebuf;
+    }
     return kev->name;
 }
 
