@@ -1,12 +1,12 @@
 /*
  * more minor utilities for mask length calculations for IKEv2
  * Copyright (C) 2007 Michael Richardson <mcr@xelerance.com>
- * 
+ *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.  See <http://www.fsf.org/copyleft/lgpl.txt>.
- * 
+ *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
@@ -25,7 +25,7 @@
  */
 int ikev2_highorder_zerobits(ip_address b)
 {
-    unsigned const char *bp;
+    unsigned char *bp;
     int i, j;
     u_int32_t mask;
     size_t n;
@@ -52,8 +52,8 @@ int ikev2_highorder_zerobits(ip_address b)
 
 int ikev2_calc_iprangediff(ip_address low, ip_address high)
 {
-    unsigned const char *hp;
-    unsigned const char *lp, *t;
+    unsigned char *hp;
+    unsigned char *lp, *t;
     unsigned char *dp;
     ip_address diff;
     size_t n;
@@ -85,7 +85,7 @@ int ikev2_calc_iprangediff(ip_address low, ip_address high)
 	/* need to swap! */
 	t=hp; hp=lp; lp=t;
     }
-    
+
     for(i=n-1; i>=0; i--) {
 	int val=hp[i]-lp[i]-carry;
 	if(val < 0) {
@@ -237,4 +237,4 @@ regress()
  * c-style: pluto
  * End:
  */
- 
+
