@@ -774,34 +774,43 @@ err_t osw_process_rsa_keyfile(struct secret **psecrets
 #ifndef HAVE_LIBNSS
     rsa_privkey_t *key = NULL;
 #endif
+    prompt_pass_t backuppass;
 
-    memset(filename,'\0', BUF_LEN);
-    memset(pass->secret,'\0', sizeof(pass->secret));
+    zero(&backuppass);    zero(filename);
+    if(pass == NULL) {
+        pass = &backuppass;
+    }
+
+    memset(pass->secret, '\0', PROMPT_PASS_LEN);
 
     /* we expect the filename of a PKCS#1 private key file */
 
-    if (*flp->tok == '"' || *flp->tok == '\'')  /* quoted filename */
+    /* quoted filename */
+    if (*flp->tok == '"' || *flp->tok == '\'')  {
 	memcpy(filename, flp->tok+1, flp->cur - flp->tok - 2);
-    else
+        filename[flp->cur - flp->tok - 2]='\0';
+    } else {
     	memcpy(filename, flp->tok, flp->cur - flp->tok);
+        filename[flp->cur - flp->tok]='\0';
+    }
 
     if (shift())
     {
 	/* we expect an appended passphrase or passphrase prompt*/
 	if (tokeqword("%prompt"))
 	{
-	    if (pass->fd == NULL_FD)
+	    if (pass && pass->fd == NULL_FD)
 		return "enter a passphrase using ipsec auto --rereadsecrets";
 	}
 	else if (*flp->tok == '"' || *flp->tok == '\'') /* quoted passphrase */
 	{
-	    memcpy(pass->secret, flp->tok+1, flp->cur - flp->tok - 2);
-	    pass->prompt=NULL;
+            memcpy(pass->secret, flp->tok+1, flp->cur - flp->tok - 2);
+            pass->prompt=NULL;
 	}
 	else
 	{
-	    memcpy(pass->secret, flp->tok, flp->cur - flp->tok);
-	    pass->prompt=NULL;
+            memcpy(pass->secret, flp->tok, flp->cur - flp->tok);
+            pass->prompt=NULL;
 	}
 
 	if (shift())
@@ -1623,7 +1632,7 @@ install_public_key(struct pubkey *pk, struct pubkey_list **head)
 	pk->issuer.ptr = clone_bytes(pk->issuer.ptr, pk->issuer.len, "issuer dn");
 
     /* store the time the public key was installed */
-    time(&pk->installed_time);
+    pk->installed_time = now();
 
     /* install new key at front */
     p->key = reference_key(pk);
