@@ -56,7 +56,7 @@ static const char *const kern_interface_name[] = {
   "mastklips",
   "bsdkame"
 };
-enum_names kern_interface_names = 
+enum_names kern_interface_names =
   { NO_KERNEL, USE_BSDKAME, kern_interface_name, NULL };
 
 /* DPD actions */
@@ -87,14 +87,20 @@ static const char *const timer_event_name[] = {
 	"EVENT_CRYPTO_FAILED",
 	"EVENT_PENDING_PHASE2",
 	"EVENT_v2_RETRANSMIT",
-	"EVENT_PENDING_DDNS"
+	"EVENT_PENDING_DDNS",
+        "EVENT_SA_DELETE"
     };
 
 enum_names timer_event_names =
-    { EVENT_NULL, EVENT_PENDING_DDNS, timer_event_name, NULL };
+    { EVENT_NULL, EVENT_SA_DELETE, timer_event_name, NULL };
 
 /* State of exchanges */
 static const char *const state_name[] = {
+    "STATE_UNDEFINED",
+    "STATE_DELETING",
+    "OPPO_ACQUIRE",
+    "OPPO_GW_DISCOVERED",
+
 	"STATE_MAIN_R0",
 	"STATE_MAIN_I1",
 	"STATE_MAIN_R1",
@@ -109,7 +115,7 @@ static const char *const state_name[] = {
 	"STATE_AGGR_R1",
 	"STATE_AGGR_I2",
 	"STATE_AGGR_R2",
-	
+
 	"STATE_QUICK_R0",
 	"STATE_QUICK_I1",
 	"STATE_QUICK_R1",
@@ -139,12 +145,19 @@ static const char *const state_name[] = {
 	"STATE_PARENT_I3",
 	"STATE_PARENT_R1",
 	"STATE_PARENT_R2",
+
+    "STATE_CHILD_C1_KEYED",
+    "STATE_CHILD_C1_REKEY",
+
+        "STATE_IKESA_DEL",
+        "STATE_CHILDSA_DEL",
+
 	"STATE_IKEv2_ROOF"
-	
+
     };
 
 enum_names state_names =
-    { STATE_MAIN_R0, STATE_IKEv2_ROOF-1, state_name, NULL };
+    { STATE_UNDEFINED, STATE_IKEv2_ROOF-1, state_name, NULL };
 
 /* story for state */
 
@@ -187,11 +200,18 @@ const char *const state_story[] = {
 	"invalid state - IKE roof",
 	"invalid state - IKEv2 base",
 	"sent v2I1, expected v2R1",             /* STATE_PARENT_I1 */
-	"sent v2I2, expected v2R2",
-	"PARENT SA established",
-	"received v2I1, sent v2R1",
-	"received v2I2, PARENT SA established",
-	"invalid state - IKEv2 roof"
+	"sent v2I2, expected v2R2",             /* STATE_PARENT_I2 */
+	"PARENT SA established",                /* STATE_PARENT_I3 */
+	"received v2I1, sent v2R1",             /* STATE_PARENT_R1 */
+	"received v2I2, PARENT SA established", /* STATE_PARENT_R2 */
+
+        "CHILD SA established",                 /* STATE_CHILD_C1_KEYED */
+        "CHILD SA being rekeyed",               /* STATE_CHILD_C1_REKEY */
+
+        "PARENT SA scheduled for deletion",     /* STATE_IKESA_DEL */
+        "CHILD SA scheduled for deletion",      /* STATE_CHILDSA_DEL */
+
+	"invalid state - IKEv2 roof (fix state_stories?)"
     };
 
 enum_names state_stories =
@@ -214,10 +234,10 @@ enum_names pluto_cryptoop_names =
     { pcr_build_kenonce, pcr_compute_dh_v2, pluto_cryptoop_strings, NULL};
 
 
-/* pluto crypto importance */
+/* pluto crypto importance: enum crypto_importance in pluto_constants.h */
 static const char *const pluto_cryptoimportance_strings[] = {
 	"import:not set",
-  "import:respond to stranger",	
+  "import:respond to stranger",
   "import:respond to friend",
   "import:ongoing calculation",
   "import:local rekey",
