@@ -42,8 +42,8 @@
 
 /**
  * Pack a string to a whack messages
- * 
- * @param wp 
+ *
+ * @param wp
  * @param p a string
  * @return bool True if operation was successful
  */
@@ -69,7 +69,7 @@ pack_str(struct whackpacker *wp, char **p)
 
 /**
  * Unpack a whack message into a string
- * 
+ *
  * @param wp Whack Message
  * @param p a string into which you want the message to be placed into
  * @return bool True if operation successful
@@ -99,7 +99,7 @@ unpack_str(struct whackpacker *wp, char **p)
 
 /**
  * Pack a message to be sent to whack
- * 
+ *
  * @param wp The whack message
  * @return err_t
  */
@@ -138,7 +138,8 @@ err_t pack_whack_msg (struct whackpacker *wp)
 	|| !pack_str(wp, &wp->msg->string1)                /* string 24 */
 	|| !pack_str(wp, &wp->msg->string2)                /* string 25 */
 	|| !pack_str(wp, &wp->msg->string3)                /* string 26 */
-	|| !pack_str(wp, &wp->msg->dnshostname) /* string 27 ? */
+	|| !pack_str(wp, &wp->msg->string4)                /* string 27: was dnshostname*/
+	|| !pack_str(wp, &wp->msg->policy_label) /* string 28 */
 	|| wp->str_roof - wp->str_next < (ptrdiff_t)wp->msg->keyval.len)    /* chunk (sort of string 28) */
     {
 	ugh = "too many bytes of strings to fit in message to pluto";
@@ -158,7 +159,7 @@ err_t pack_whack_msg (struct whackpacker *wp)
 
 /**
  * Unpack a message whack received
- * 
+ *
  * @param wp The whack message
  * @return err_t
  */
@@ -169,8 +170,10 @@ err_t unpack_whack_msg (struct whackpacker *wp)
     if (wp->str_next > wp->str_roof)
     {
 	ugh = builddiag("ignoring truncated message from whack: got %d bytes; expected %u"
-			, (int) wp->n, (unsigned) sizeof(wp->msg));
+			, (int) wp->n, (unsigned) sizeof(*wp->msg));
+        return ugh;
     }
+
     if (!unpack_str(wp, &wp->msg->name)	          /* string 1 */
 	|| !unpack_str(wp, &wp->msg->left.id)     /* string 2 */
 	|| !unpack_str(wp, &wp->msg->left.cert)   /* string 3 */
@@ -197,7 +200,8 @@ err_t unpack_whack_msg (struct whackpacker *wp)
 	|| !unpack_str(wp, &wp->msg->string1)                /* string 24 */
 	|| !unpack_str(wp, &wp->msg->string2)                /* string 25 */
 	|| !unpack_str(wp, &wp->msg->string3)                /* string 26 */
-	|| !unpack_str(wp, &wp->msg->dnshostname)  /* string 27 ? */
+	|| !unpack_str(wp, &wp->msg->string4)                /* string 27 was dnshostname*/
+	|| !unpack_str(wp, &wp->msg->policy_label) /* string 28 */
 	|| wp->str_roof - wp->str_next != (ptrdiff_t)wp->msg->keyval.len)	/* check chunk */
     {
 	ugh = "message from whack contains bad string";
@@ -231,9 +235,9 @@ whack_get_value(char *buf, size_t bufsize)
     while(try > 0 && len==0)
     {
 	fprintf(stderr, "Enter username:   ");
-	
+
 	memset(buf, 0, bufsize);
-	
+
 	if(fgets(buf, bufsize, stdin) != buf) {
 	    if(errno == 0) {
 		fprintf(stderr, "Can not read password from standard in\n");
@@ -243,7 +247,7 @@ whack_get_value(char *buf, size_t bufsize)
 		exit(RC_WHACK_PROBLEM);
 	    }
 	}
-	
+
 	/* send the value to pluto, including \0, but fgets adds \n */
 	len = strlen(buf);
 	if(len == 0)
@@ -276,7 +280,7 @@ whack_get_secret(char *buf, size_t bufsize)
     strncpy(buf, secret, bufsize);
 
     len = strlen(buf) + 1;
-    
+
     return len;
 }
 
