@@ -477,6 +477,29 @@
 # define DEFINE_RWLOCK(x) rwlock_t x = RW_LOCK_UNLOCKED
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,15,10)
+# define IP_SELECT_IDENT_NEW
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(3,15,0) && LINUX_VERSION_CODE >= KERNEL_VERSION(3,14,17)
+# define IP_SELECT_IDENT_NEW
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(3,14,0) && LINUX_VERSION_CODE >= KERNEL_VERSION(3,13,11)
+# define IP_SELECT_IDENT_NEW
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(3,13,0) && LINUX_VERSION_CODE >= KERNEL_VERSION(3,12,0)
+# define IP_SELECT_IDENT_NEW
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(3,11,0) && LINUX_VERSION_CODE >= KERNEL_VERSION(3,10,53)
+# define IP_SELECT_IDENT_NEW
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(3,5,0) && LINUX_VERSION_CODE >= KERNEL_VERSION(3,4,103)
+# define IP_SELECT_IDENT_NEW
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(3,3,0) && LINUX_VERSION_CODE >= KERNEL_VERSION(3,2,0)
+# define IP_SELECT_IDENT_NEW
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,14,0)
+#define NEIGH_PARAMS_IS_ARRAY
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,18,0)
+#define ALLOC_NETDEV4
+#endif
 
 #endif /* _OPENSWAN_KVERSIONS_H */
 
