@@ -2,18 +2,16 @@
 /*
  * policy interface file between pluto and applications
  * Copyright (C) 2003              Michael Richardson <mcr@freeswan.org>
- * 
+ *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Library General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.  See <http://www.fsf.org/copyleft/lgpl.txt>.
- * 
+ *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
  * License for more details.
- *
- * RCSID $Id: ipsec_policy.h,v 1.8 2005/07/26 01:12:38 mcr Exp $
  */
 #define	_IPSEC_POLICY_H 	/* seen it, no need to see it again */
 
@@ -42,7 +40,7 @@ enum ipsec_policy_command {
 
 struct ipsec_policy_msg_head {
   u_int32_t ipm_version;
-  u_int32_t ipm_msg_len;  
+  u_int32_t ipm_msg_len;
   u_int32_t ipm_msg_type;
   u_int32_t ipm_msg_seq;
 };
@@ -83,8 +81,13 @@ enum ipsec_authentication_algo {
   AH_SHA2_384=6,
   AH_SHA2_512=7,
   AH_RIPEMD=8,
-  AH__AES_XCBC_MAC=9,
-  AH_RSA=10
+  AH_AES_XCBC_MAC=9,
+  AH_RSA=10,
+  AH_AES_128_GMAC=11, /* RFC4543 [Errata1821] */
+  AH_AES_192_GMAC=12, /* RFC4543 [Errata1821] */
+  AH_AES_256_GMAC=13, /* RFC4543 [Errata1821] */
+  AH_NULL=251, /* comes from kame? */
+  AH_SHA2_256_TRUNC=252, /* our own stolen value */
 };
 
 /* IPsec ESP transform values
@@ -116,8 +119,15 @@ enum ipsec_cipher_algo {
   ESP_AES_GCM_16=20,
   ESP_SEED_CBC=21,
   ESP_CAMELLIA=22,
-  ESP_NULL_AUTH_AES_GMAC=23 /* [RFC4543][Errata1821] */
+  ESP_NULL_AUTH_AES_GMAC=23, /* [RFC4543][Errata1821] */
   /* 249-255 reserved for private use */
+  ESP_MARS=249,
+  ESP_RC6=250,
+  ESP_KAME_NULL=251, /* kame? */
+  ESP_SERPENT=252,
+  ESP_TWOFISH=253,
+  ESP_ID254=254,
+  ESP_ID255=255,
 };
 
 /* IPCOMP transform values
@@ -136,7 +146,7 @@ enum ipsec_comp_algo {
  * RFC 2407 The Internet IP security Domain of Interpretation for
  * ISAKMP 4.6.2.1
  *
- * Also for RFC4306. 
+ * Also for RFC4306.
  *
  * enum ident_names;
  */
@@ -175,7 +185,7 @@ enum ipsec_cert_type {
   CERT_ARL=			8,
   CERT_SPKI=			9,
   CERT_X509_ATTRIBUTE=		10,
-  CERT_RAW_RSA=                 11, /* raw RSA from config file */ 
+  CERT_RAW_RSA=                 11, /* raw RSA from config file */
 };
 
 /* a SIG record in ASCII */
@@ -214,7 +224,7 @@ struct ipsec_policy_cmd_query {
   /* Answer section */
   enum ipsec_privacy_quality     strength;
   enum ipsec_bandwidth_quality   bandwidth;
-  enum ipsec_authentication_algo auth_detail;  
+  enum ipsec_authentication_algo auth_detail;
   enum ipsec_cipher_algo         esp_detail;
   enum ipsec_comp_algo           comp_detail;
 
