@@ -1,6 +1,7 @@
 /*
  * addresses to text
  * Copyright (C) 2000  Henry Spencer.
+ * Copyright (C) 2012 Paul Wouters <paul@libreswan.org>
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Library General Public License as published by
@@ -72,18 +73,23 @@ int format;			/* character */
 char *dst;			/* need not be valid if dstlen is 0 */
 size_t dstlen;
 {
-	const unsigned char *b;
+        unsigned char *b;
 	size_t n;
 	char buf[1+ADDRTOT_BUF+1];	/* :address: */
 	char *p;
-	int t = addrtypeof(src);
-#	define	TF(t, f)	(((t)<<8) | (f))
+	int t;
+#define	TF(t, f)	(((t)<<8) | (f))
 
+        if(!src) {
+                strncat(dst, "<none>", dstlen -1);
+                return sizeof("<none>");
+        }
+
+	t = addrtypeof(src);
 	n = addrbytesptr(src, &b);
 	if (n == 0) {
-	bad:
 	  dst[0]='\0';
-	  strncat(dst, "<invalid>", dstlen);
+	  strncat(dst, "<invalid>", dstlen -1); /* we hope possible truncation does not cause problems */
 	  return sizeof("<invalid>");
 	}
 
@@ -107,7 +113,9 @@ size_t dstlen;
 		n = reverse6(b, n, buf, &p);
 		break;
 	default:		/* including (AF_INET, 'R') */
-	        goto bad;
+		dst[0]='\0';
+		strncat(dst, "<invalid>", dstlen - 1); /* we hope possible truncation does not cause problems */
+		return sizeof("<invalid>");
 	}
 
 	if (dstlen > 0) {
@@ -132,15 +140,15 @@ size_t dstlen;
 	size_t n;
 	char buf[1+ADDRTOT_BUF+1];	/* :address: */
 	char *p;
+
 #	define	TF(t, f)	(((t)<<8) | (f))
 
 	switch (t) {
 	case AF_INET: n = IP4BYTES; break;
 	case AF_INET6: n = IP6BYTES; break;
 	default:
-	bad:
 	  dst[0]='\0';
-	  strncat(dst, "<invalid>", dstlen);
+	  strncat(dst, "<invalid>", dstlen - 1); /* we hope possible truncation does not cause problems */
 	  return sizeof("<invalid>");
 	}
 
@@ -164,7 +172,9 @@ size_t dstlen;
 		n = reverse6(src, n, buf, &p);
 		break;
 	default:		/* including (AF_INET, 'R') */
-		goto bad;
+	  	dst[0]='\0';
+	  	strncat(dst, "<invalid>", dstlen - 1); /* we hope possible truncation does not cause problems */
+	  	return sizeof("<invalid>");
 	}
 
 	if (dstlen > 0) {
@@ -190,6 +200,7 @@ size_t dstlen;
 		struct sockaddr_in sin;
 		struct sockaddr_in6 sin6;
 	} *sinp = (const union SINSIN6 *) src;
+
 	switch (sinp->sin.sin_family) {
 	case AF_INET:
 		return inet_addrtot(AF_INET,&sinp->sin.sin_addr,format,dst,dstlen);
@@ -197,7 +208,7 @@ size_t dstlen;
 		return inet_addrtot(AF_INET6,&sinp->sin6.sin6_addr,format,dst,dstlen);
 	default:
 		dst[0]='\0';
-		strncat(dst, "<invalid>", dstlen);
+		strncat(dst, "<invalid>", dstlen - 1); /* we hope possible truncation does not cause problems */
 		return sizeof("<invalid>");
 	}
 }
