@@ -4,6 +4,7 @@
  * Copyright (C) 1998-2003   Richard Guy Briggs.
  * Copyright (C) 2004-2007   Michael Richardson <mcr@xelerance.com>
  * Copyright (C) 2007-2008   Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2012  Paul Wouters  <paul@libreswan.org>
  *
  * OCF/receive state machine written by
  * David McCullough <dmccullough@cyberguard.com>
@@ -34,11 +35,7 @@
 
 #include "openswan/ipsec_param.h"
 
-#ifdef MALLOC_SLAB
-# include <linux/slab.h> /* kmalloc() */
-#else /* MALLOC_SLAB */
-# include <linux/malloc.h> /* kmalloc() */
-#endif /* MALLOC_SLAB */
+#include <linux/slab.h> /* kmalloc() */
 #include <linux/errno.h>  /* error codes */
 #include <linux/types.h>  /* size_t */
 #include <linux/interrupt.h> /* mark_bh */
@@ -55,16 +52,10 @@
 #include <linux/skbuff.h>
 #include <openswan.h>
 
-#ifdef SPINLOCK
-# ifdef SPINLOCK_23
-#  include <linux/spinlock.h> /* *lock* */
-#  ifdef NEED_SPINLOCK_TYPES
-#   include <linux/spinlock_types.h> 
-#  endif
-# else /* SPINLOCK_23 */
-#  include <asm/spinlock.h> /* *lock* */
-# endif /* SPINLOCK_23 */
-#endif /* SPINLOCK */
+#include <linux/spinlock.h> /* *lock* */
+#ifdef NEED_SPINLOCK_TYPES
+# include <linux/spinlock_types.h>
+#endif
 
 #include <net/ip.h>
 
@@ -337,11 +328,7 @@ struct sk_buff *ipsec_rcv_unclone(struct sk_buff *skb,
 		}
 		skb_push(skb, irs->hard_header_len);
 		if
-#ifdef SKB_COW_NEW
 		  (skb_cow(skb, skb_headroom(skb)) != 0)
-#else /* SKB_COW_NEW */
-		  ((skb = skb_cow(skb, skb_headroom(skb))) == NULL)
-#endif /* SKB_COW_NEW */
 		{
 			return NULL;
 		}
@@ -392,16 +379,16 @@ struct sk_buff *ipsec_rcv_natt_decap(struct sk_buff *skb
 		struct udphdr *udp = (struct udphdr *)((__u32 *)ip+ip->ihl);
 		__u8 *udpdata = (__u8 *)udp + sizeof(struct udphdr);
 		__u32 *udpdata32 = (__u32 *)udpdata;
-		
+
 		irs->natt_sport = ntohs(udp->source);
 		irs->natt_dport = ntohs(udp->dest);
-	  
+
 		KLIPS_PRINT(debug_rcv,
 			    "klips_debug:ipsec_rcv_natt_decap: "
 			    "suspected ESPinUDP packet (NAT-Traversal) [%d].\n",
 			    tp->esp_in_udp);
 		KLIPS_IP_PRINT(debug_rcv, ip);
-	  
+
 		if (udpdata < skb->tail) {
 			unsigned int len = skb->tail - udpdata;
 			if ((len==1) && (udpdata[0]==0xff)) {
@@ -416,7 +403,7 @@ struct sk_buff *ipsec_rcv_natt_decap(struct sk_buff *skb
 				  (len > (2*sizeof(__u32) + sizeof(struct esphdr))) &&
 				  (udpdata32[0]==0) && (udpdata32[1]==0) ) {
 				/* ESP Packet with Non-IKE header */
-				KLIPS_PRINT(debug_rcv, 
+				KLIPS_PRINT(debug_rcv,
 					    "klips_debug:ipsec_rcv_natt_decap: "
 					    "ESPinUDP pkt with Non-IKE - spi=0x%x\n",
 					    ntohl(udpdata32[2]));
@@ -429,7 +416,7 @@ struct sk_buff *ipsec_rcv_natt_decap(struct sk_buff *skb
 				/* ESP Packet without Non-ESP header */
 				irs->natt_type = ESPINUDP_WITH_NON_ESP;
 				irs->natt_len = sizeof(struct udphdr);
-				KLIPS_PRINT(debug_rcv, 
+				KLIPS_PRINT(debug_rcv,
 					    "klips_debug:ipsec_rcv_natt_decap: "
 					    "ESPinUDP pkt without Non-ESP - spi=0x%x\n",
 					    ntohl(udpdata32[0]));
@@ -462,7 +449,7 @@ void ipsec_rcv_setoutif(struct ipsec_rcv_state *irs)
 				    irs->ipsp->ips_out->name);
 		}
 		skb->dev = irs->ipsp->ips_out;
-		
+
 #ifdef USE_NETDEV_OPS
 		if(skb->dev && skb->dev->netdev_ops->ndo_get_stats) {
 			struct net_device_stats *stats = skb->dev->netdev_ops->ndo_get_stats(skb->dev);
@@ -474,7 +461,7 @@ void ipsec_rcv_setoutif(struct ipsec_rcv_state *irs)
 			irs->stats = stats;
 		}
 #endif
-	} 
+	}
 }
 
 static enum ipsec_rcv_value
@@ -555,7 +542,7 @@ ipsec_rcv_decap_ipip(struct ipsec_rcv_state *irs)
 				    "klips_debug:ipsec_rcv_decap_ipip: "
 				    "SA:%s, src=%s(%s) does match expected %s.\n",
 				    irs->sa_len ? irs->sa : " (error)",
-				    irs->ipsaddr_txt, 
+				    irs->ipsaddr_txt,
 				    saddr1, saddr2);
 			if(irs->stats) {
 				irs->stats->rx_dropped++;
@@ -563,7 +550,7 @@ ipsec_rcv_decap_ipip(struct ipsec_rcv_state *irs)
 			goto rcvleave;
 		}
 	}
-	
+
 	ipsec_rcv_setoutif(irs);
 
 	/* added to support AT&T heartbeats to SIG/GIG */
@@ -577,16 +564,16 @@ ipsec_rcv_decap_ipip(struct ipsec_rcv_state *irs)
 		 */
 		ipsp->ips_life.ipl_bytes.ipl_count += skb->len;
 		ipsp->ips_life.ipl_bytes.ipl_last   = skb->len;
-		
+
 		if(!ipsp->ips_life.ipl_usetime.ipl_count) {
 			ipsp->ips_life.ipl_usetime.ipl_count = jiffies / HZ;
 		}
 		ipsp->ips_life.ipl_usetime.ipl_last = jiffies / HZ;
 		ipsp->ips_life.ipl_packets.ipl_count += 1;
-		
+
 		/* new L3 header is where L4 payload was */
 		skb_set_network_header(skb, ipsec_skb_offset(skb, skb_transport_header(skb)));
-		
+
 		/* now setup new L4 payload location */
 		ipp = (struct iphdr *)skb_network_header(skb);
 #ifdef CONFIG_KLIPS_IPV6
@@ -612,26 +599,26 @@ ipsec_rcv_decap_ipip(struct ipsec_rcv_state *irs)
 			       "tried to skb_pull iphlen=%d, %d available.  This should never happen, please report(3).\n",
 			       irs->iphlen,
 			       (int)(skb->len));
-			
+
 			goto rcvleave;
 		}
-		
+
 		/*
 		 * we need to pull up by size of the new IP header,
 		 * + options, but also by any UDP/ESP encap there might
 		 * have been, and this deals with all cases.
 		 */
 		skb_pull(skb, (skb_transport_header(skb) - skb_network_header(skb)));
-		
+
 		/* remove any saved options that we might have,
 		 * since we have a new IP header.
 		 */
 		memset(&(IPCB(skb)->opt), 0, sizeof(struct ip_options));
-		
+
 #if 0
 		KLIPS_PRINT(debug_rcv, "csum: %d\n", ip_fast_csum((u8 *)ipp, ipp->ihl));
 #endif
-		
+
 		/* re-do any strings for debugging */
 		irs->iph = (void*)ipp;
 		if (debug_rcv)
@@ -647,7 +634,7 @@ ipsec_rcv_decap_ipip(struct ipsec_rcv_state *irs)
 			    "IPIP tunnel stripped.\n");
 		KLIPS_IP_PRINT(debug_rcv & DB_RX_PKTRX, ipp);
 	}
-	
+
 	if(sysctl_ipsec_inbound_policy_check) {
 	   /*
 	     Note: "xor" (^) logically replaces "not equal"
@@ -701,7 +688,7 @@ ipsec_rcv_decap_ipip(struct ipsec_rcv_state *irs)
 	if (failed_inbound_check)
 	{
 		char sflow_txt[SUBNETTOA_BUF], dflow_txt[SUBNETTOA_BUF];
-		
+
 #ifdef CONFIG_KLIPS_IPV6
 		if (ipsp->ips_flow_s.u.v4.sin_family == AF_INET6) {
 			subnet6toa(&ipsp->ips_flow_s.u.v6.sin6_addr,
@@ -833,8 +820,7 @@ ipsec_rcv_init(struct ipsec_rcv_state *irs)
 		return IPSEC_RCV_REALLYBAD;
 	}
 
-#if IP_FRAGMENT_LINEARIZE
-	/* In Linux 2.4.4, we may have to reassemble fragments. They are
+	/* In Linux >2.4.4, we may have to reassemble fragments. They are
 	   not assembled automatically to save TCP from having to copy
 	   twice.
 	*/
@@ -842,13 +828,12 @@ ipsec_rcv_init(struct ipsec_rcv_state *irs)
 #ifdef HAVE_NEW_SKB_LINEARIZE
 		if (skb_linearize_cow(skb) != 0)
 #else
-		if (skb_linearize(skb, GFP_ATOMIC) != 0) 
+		if (skb_linearize(skb, GFP_ATOMIC) != 0)
 #endif
 		{
 			return IPSEC_RCV_REALLYBAD;
 		}
 	}
-#endif /* IP_FRAGMENT_LINEARIZE */
 
 	irs->iph = (void *)ip_hdr(skb);
 
@@ -924,14 +909,6 @@ ipsec_rcv_init(struct ipsec_rcv_state *irs)
 			skb->dev->name ? skb->dev->name : "NULL");
 	KLIPS_PRINTMORE(debug_rcv, "\n");
 
-#ifndef NET_21
-	if((!protocol) || (protocol->protocol != irs->proto)) {
-		KLIPS_PRINT(debug_rcv & DB_RX_IPSA,
-			    "klips_debug:ipsec_rcv_init: "
-			    "protocol arg is NULL or unequal to the packet contents, this is odd, using value in packet.\n");
-	}
-#endif /* !NET_21 */
-
 	if( (irs->proto != IPPROTO_AH) &&
 #ifdef CONFIG_KLIPS_IPCOMP_disabled_until_we_register_IPCOMP_HANDLER
 	    (irs->proto != IPPROTO_COMP) &&
@@ -957,7 +934,7 @@ ipsec_rcv_init(struct ipsec_rcv_state *irs)
 			if(!netdev_priv(ipsecdevices[i])) continue;
 
 			prvdev = netdev_to_ipsecpriv(ipsecdevices[i]);
-			
+
 			if(prvdev->dev == skb->dev) {
 				ipsecdev = ipsecdevices[i];
 				break;
@@ -1070,7 +1047,7 @@ ipsec_rcv_decap_lookup(struct ipsec_rcv_state *irs)
 		irs->said.dst.u.v4.sin_addr.s_addr = osw_ip4_hdr(irs)->daddr;
 		irs->said.dst.u.v4.sin_family = AF_INET;
 	}
-	
+
 	/* note: rcv_checks set up the said.spi value, if appropriate */
 	if (irs->proto_funcs->rcv_checks)
 		return (*irs->proto_funcs->rcv_checks)(irs, irs->skb);
@@ -1226,7 +1203,7 @@ ipsec_rcv_auth_init(struct ipsec_rcv_state *irs)
                                 return IPSEC_RCV_FAILEDINBOUND;
                         }
                 }
-#endif		 
+#endif
 	}
 
 	if (newipsp != irs->ipsp) {
@@ -1298,7 +1275,7 @@ ipsec_rcv_auth_decap(struct ipsec_rcv_state *irs)
 	   ipsec_lifetime_check(&irs->ipsp->ips_life.ipl_packets, "packets",
 				irs->sa, ipsec_life_countbased, ipsec_incoming,
 				irs->ipsp) == ipsec_life_harddied) {
-		
+
 		/*
 		 * disconnect SA from the hash table, so it can not be
 		 * found again.
@@ -1307,7 +1284,7 @@ ipsec_rcv_auth_decap(struct ipsec_rcv_state *irs)
 		if(irs->stats) {
 			irs->stats->rx_dropped++;
 		}
-		
+
 		KLIPS_PRINT(debug_rcv,
 			    "klips_debug:ipsec_rcv_auth_decap: "
 			    "decap (%d) failed lifetime check\n",
@@ -1399,7 +1376,7 @@ ipsec_rcv_auth_decap(struct ipsec_rcv_state *irs)
 		KLIPS_PRINT(debug_rcv,
 				"klips_debug:ipsec_rcv_auth_decap: "
 				"authalg=%d authlen=%d\n",
-				irs->ipsp->ips_authalg, 
+				irs->ipsp->ips_authalg,
 				irs->authlen);
 	} else
 #endif /* CONFIG_KLIPS_ALG */
@@ -1655,7 +1632,7 @@ ipsec_rcv_decap_cont(struct ipsec_rcv_state *irs)
 		irs->iphlen = osw_ip4_hdr(irs)->ihl<<2;
 		skb_set_transport_header(skb, ipsec_skb_offset(skb, skb_network_header(skb) + irs->iphlen));
 	}
-	
+
 	/* zero any options that there might be */
 	memset(&(IPCB(skb)->opt), 0, sizeof(struct ip_options));
 
@@ -1857,7 +1834,7 @@ ipsec_rcv_cleanup(struct ipsec_rcv_state *irs)
 			return decap_stat;
 		}
 	}
-  
+
 	if(irs->stats) {
 		irs->stats->rx_bytes += skb->len;
 	}
@@ -2066,9 +2043,6 @@ ipsec_rsm(struct ipsec_rcv_state *irs)
 
 int
 ipsec_rcv(struct sk_buff *skb
-#ifndef PROTO_HANDLER_SINGLE_PARM
-	  unsigned short xlen
-#endif /* PROTO_HANDLER_SINGLE_PARM */
 	  )
 {
 	struct ipsec_rcv_state *irs = NULL;
@@ -2103,7 +2077,7 @@ ipsec_rcv(struct sk_buff *skb
   		/* NET_26 NAT-T is handled by seperate function */
   		struct sk_buff *nskb;
 		int udp_decap_ret = 0;
-  
+
 		nskb = ipsec_rcv_natt_decap(skb, irs, &udp_decap_ret);
   		if(nskb == NULL) {
 			/* return with non-zero, because UDP.c code
@@ -2150,7 +2124,7 @@ ipsec_rcv(struct sk_buff *skb
  *
  * skb->iph->tot_len has been byte-swapped, and reduced by the size of
  *              the IP header (and options).
- * 
+ *
  * skb->h.raw has been pulled up the ESP header.
  *
  * skb->iph->protocol = 50 IPPROTO_ESP;
@@ -2304,11 +2278,11 @@ int klips26_rcv_encap(struct sk_buff *skb, __u16 encap_type)
 	case UDP_ENCAP_ESPINUDP:
 	  irs->natt_type = ESPINUDP_WITH_NON_ESP;
 	  break;
-	  
+
 	case UDP_ENCAP_ESPINUDP_NON_IKE:
 	  irs->natt_type = ESPINUDP_WITH_NON_IKE;
 	  break;
-	  
+
 	default:
 	  if(printk_ratelimit()) {
 	    printk(KERN_INFO "KLIPS received unknown UDP-ESP encap type %u\n",
