@@ -1,12 +1,12 @@
 /*
  * A program to read the configuration file and load a single conn
  * Copyright (C) 2005 Michael Richardson <mcr@xelerance.com>
- * 
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
@@ -53,7 +53,7 @@
 #include "ipsecconf/starterwhack.h"
 #include "ipsecconf/keywords.h"
 
-char *progname;
+const char *progname;
 int verbose=0;
 int warningsarefatal = 0;
 
@@ -65,7 +65,7 @@ static const char *usage_string = ""
     "               [--configsetup] \n"
     "               {--checkconfig] \n"
     "               [--defaultroute <addr>] [--defaultroutenexthop <addr>]\n"
-    
+
     "               names\n";
 
 
@@ -90,7 +90,7 @@ static struct option const longopts[] =
 	{"listroute",           no_argument, NULL, 'r'},
 	{"liststart",           no_argument, NULL, 's'},
 	{"varprefix",           required_argument, NULL, 'P'},
-	{ "ctlbase",            required_argument, NULL, 'c' },
+	{"ctlbase" ,            required_argument, NULL, 'c' },
 	{"search",              no_argument, NULL, 'S'},
 	{"rootdir",             required_argument, NULL, 'R'},
 	{"configsetup",         no_argument, NULL, 'T'},
@@ -121,6 +121,7 @@ main(int argc, char *argv[])
     char *defaultnexthop = NULL;
     char *ctlbase = NULL;
     bool resolvip = FALSE;
+    char  ctlbuf[1024];
 
 #if 0
     /* efence settings */
@@ -130,7 +131,7 @@ main(int argc, char *argv[])
     EF_PROTECT_BELOW=1;
     EF_PROTECT_FREE=1;
 #endif
-    
+
 
     progname = argv[0];
     rootdir[0]='\0';
@@ -173,7 +174,11 @@ main(int argc, char *argv[])
 	    break;
 
 	case 'c':
-	    ctlbase = clone_str(optarg, "control base");
+	    if(snprintf(ctlbuf, sizeof(ctlbuf), "%s%s", optarg, CTL_SUFFIX) == -1) {
+		fprintf(stderr, "<ctlbase>" CTL_SUFFIX " must be fit in a 1024 bytes");
+                exit(4);
+            }
+	    ctlbase = clone_str(ctlbuf, "control base");
 	    break;
 
 	case 'A':
@@ -226,7 +231,7 @@ main(int argc, char *argv[])
     {
 	confdir = IPSEC_CONFDIR;
     }
-	
+
     if(!configfile) {
 	configfile = alloc_bytes(strlen(confdir)+sizeof("/ipsec.conf")+2,"conf file");
 
@@ -254,7 +259,7 @@ main(int argc, char *argv[])
 	resolvip=FALSE;
     }
     cfg = confread_load(configfile, &err, resolvip, ctlbase,typeexport);
-    
+
     if(cfg == NULL) {
 	fprintf(stderr, "can not load config '%s': %s\n",
 		configfile, err);
@@ -297,7 +302,7 @@ main(int argc, char *argv[])
 	}
     }
 
-    if(all) 
+    if(all)
     {
 	if(verbose) {
 	    printf("loading all conns:");
@@ -351,7 +356,7 @@ main(int argc, char *argv[])
 
 	printf("%s_confreadstatus=\n", varprefix);
 	printf("%s_confreadnames=\"",varprefix);
-	
+
 	/* find conn names that have value set */
 	for(conn = cfg->conns.tqh_first;
 	    conn != NULL;
@@ -376,6 +381,10 @@ main(int argc, char *argv[])
 	printf("export %sconfreadstatus=''\n", varprefix);
 	for(kd=ipsec_conf_keywords_v2; kd->keyname != NULL; kd++) {
 	    if((kd->validity & kv_config)==0) continue;
+	    if(kd->validity & kv_obsolete) {
+		printf("# obsolete option '%s%s' ignored\n", varprefix, kd->keyname);
+                continue;
+            }
 
 	    switch(kd->type) {
 	    case kt_string:
@@ -402,10 +411,6 @@ main(int argc, char *argv[])
 		printf("'\n");
 		break;
 
-	    case kt_obsolete:
-		printf("# obsolete option '%s%s' ignored\n", varprefix, kd->keyname);
-		break;
-
 	    default:
 		if(cfg->setup.options[kd->field] || cfg->setup.options_set[kd->field]) {
 		    printf("export %s%s='%d'\n",
@@ -449,7 +454,7 @@ main(int argc, char *argv[])
 		    break;
 		}
 	    }
-	    
+
 	    if(conn == NULL) {
 		/* only if we don't find it, do we now look for aliases */
 
@@ -486,7 +491,7 @@ main(int argc, char *argv[])
 	if(verbose) printf("\n");
     }
 
-    confread_free(cfg);    
+    confread_free(cfg);
     exit(exit_status);
 }
 
