@@ -133,7 +133,7 @@
 #ifdef XAUTH_USEPAM
 #include <security/pam_appl.h>
 #endif
-#include "connections.h"	/* needs id.h */
+#include "pluto/connections.h"	/* needs id.h */
 #include "state.h"
 #include "packet.h"
 #include "md5.h"
@@ -146,7 +146,7 @@
 #include "ipsec_doi.h"	/* needs demux.h and state.h */
 #include "timer.h"
 #include "whack.h"	/* requires connections.h */
-#include "server.h"
+#include "pluto/server.h"
 #ifdef XAUTH
 #include "xauth.h"
 #endif
@@ -739,7 +739,7 @@ informational(struct msg_digest *md)
                 DBG(DBG_CONTROLMORE, DBG_log("that has_client: %d", tmp_spd->that.has_client));
                 DBG(DBG_CONTROLMORE, DBG_log("that has_client_wildcard: %d", tmp_spd->that.has_client_wildcard));
                 DBG(DBG_CONTROLMORE, DBG_log("that has_port_wildcard: %d", tmp_spd->that.has_port_wildcard));
-                DBG(DBG_CONTROLMORE, DBG_log("that has_id_wildcards: %d", tmp_spd->that.has_id_wildcards));
+                DBG(DBG_CONTROLMORE, DBG_log("that has_id_wildcards: %d", tmp_spd->that.id.has_wildcards));
 
                 tmp_spd = tmp_spd->next;
                 } while(tmp_spd!=NULL);
@@ -792,7 +792,7 @@ informational(struct msg_digest *md)
                     }
                    );
 
-		tmp_c->host_pair->him.addr = tmp_c->spd.that.host_addr;
+		tmp_c->IPhost_pair->him.addr = tmp_c->spd.that.host_addr;
 
 		/*Initiating connection to the redirected peer*/
 		initiate_connection(tmp_name, tmp_whack_sock, 0, pcim_demand_crypto);
@@ -1390,6 +1390,10 @@ process_v1_packet(struct msg_digest **mdp)
      * (may be suspended due to crypto operation not yet complete)
      */
     md->st = st;
+    md->pst = st;
+    if(st!=NULL && st->st_clonedfrom != 0) {
+        md->pst = state_with_serialno(st->st_clonedfrom);
+    }
     md->from_state = from_state;
     md->smc = smc;
     md->new_iv_set = new_iv_set;
@@ -1530,7 +1534,7 @@ void process_packet_tail(struct msg_digest **mdp)
     {
 	/* packet was not encryped -- should it have been? */
 
-	if (smc->flags & SMF_INPUT_ENCRYPTED)
+	if (smc != NULL && smc->flags & SMF_INPUT_ENCRYPTED)
 	{
 	    loglog(RC_LOG_SERIOUS, "packet rejected: should have been encrypted");
 	    SEND_NOTIFICATION(INVALID_FLAGS);
@@ -1557,7 +1561,7 @@ void process_packet_tail(struct msg_digest **mdp)
 
 	while (np != ISAKMP_NEXT_NONE)
 	{
-	    struct_desc *sd = np < ISAKMP_NEXT_ROOF? payload_descs[np] : NULL;
+	    struct_desc *sd = payload_desc(np);
 
 	    if (pd == &md->digest[PAYLIMIT])
 	    {
@@ -1604,12 +1608,12 @@ void process_packet_tail(struct msg_digest **mdp)
 #ifdef NAT_TRAVERSAL
 		case ISAKMP_NEXT_NATD_DRAFTS:
 		    np = ISAKMP_NEXT_NATD_RFC;  /* NAT-D relocated */
-		    sd = payload_descs[np];
+		    sd = payload_desc(np);
 		    break;
 
 		case ISAKMP_NEXT_NATOA_DRAFTS:
 		    np = ISAKMP_NEXT_NATOA_RFC;  /* NAT-OA relocated */
-		    sd = payload_descs[np];
+		    sd = payload_desc(np);
 		    break;
 #endif
 		default:
@@ -1619,6 +1623,7 @@ void process_packet_tail(struct msg_digest **mdp)
 		    SEND_NOTIFICATION(INVALID_PAYLOAD_TYPE);
 		    return;
 		}
+	    passert(sd != NULL);
 	    }
 
 	    {
@@ -1945,7 +1950,8 @@ complete_v1_state_transition(struct msg_digest **mdp, stf_status result)
      * we can only be in calculating state if state is ignore,
      * or suspended.
      */
-    passert(result == STF_INLINE || result == STF_IGNORE || result == STF_SUSPEND || st->st_calculating==FALSE);
+    passert(result == STF_INLINE || result == STF_IGNORE || result == STF_SUSPEND
+            || st == NULL || (st && st->st_calculating==FALSE));
 
     switch (result)
     {
@@ -1994,11 +2000,11 @@ complete_v1_state_transition(struct msg_digest **mdp, stf_status result)
  	     * actual end of phase 1. With modecfg, negotiation ends with
  	     * STATE_MAIN_I4 already.
  	     */
-	    /*if(st->st_connection->spd.this.xauth_client 
-		&& st->hidden_variables.st_xauth_client_done 
+	    /*if(st->st_connection->spd.this.xauth_client
+		&& st->hidden_variables.st_xauth_client_done
 		&& !st->st_connection->spd.this.modecfg_client
 		&& st->st_state == STATE_XAUTH_I1) {
-		DBG(DBG_CONTROL, DBG_log("As XAUTH is done and modecfg is not configured, 
+		DBG(DBG_CONTROL, DBG_log("As XAUTH is done and modecfg is not configured,
 						so Phase 1 neogtiation finishes successfully"));
 		change_state(st, STATE_MAIN_I4);
 	    }*/
@@ -2299,9 +2305,7 @@ complete_v1_state_transition(struct msg_digest **mdp, stf_status result)
 		change_state(st, STATE_MAIN_R3);    /* ISAKMP is up... */
 	        set_cur_state(st);
 	        quick_outI1(st->st_whack_sock, st, st->st_connection, st->st_connection->policy, 1, SOS_NOBODY
-#ifdef HAVE_LABELED_IPSEC
                                , NULL /* Setting NULL as this is responder and will not have sec ctx from a flow*/
-#endif
 			   );
 		break;
 	    }
@@ -2409,32 +2413,36 @@ complete_v1_state_transition(struct msg_digest **mdp, stf_status result)
 	     * whatever retrying was in place, remains in place.
 	     */
 	    whack_log(RC_NOTIFICATION + md->note
-		, "%s: %s", enum_name(&state_names, st->st_state)
+		, "%s: %s", enum_name(&state_names, from_state)
 		, enum_name(&ipsec_notification_names, md->note));
 
 	    if(md->note > 0) {
 		SEND_NOTIFICATION(md->note);
 	    }
 
-	    DBG(DBG_CONTROL,
-		DBG_log("state transition function for %s failed: %s"
-			, enum_name(&state_names, from_state)
-			, enum_name(&ipsec_notification_names, md->note)));
+            if( st ) {
+                DBG(DBG_CONTROL,
+                    DBG_log("state transition function for %s failed: %s whack_fd: %u"
+                            , enum_name(&state_names, from_state)
+                            , enum_name(&ipsec_notification_names, md->note)
+                            , st->st_whack_sock
+                            ));
 #ifdef HAVE_NM
-           if (st->st_connection->remotepeertype == CISCO
-               && st->st_connection->nmconfigured) {
-                if(!do_command(st->st_connection, &st->st_connection->spd, "disconnectNM", st)) {
-                DBG(DBG_CONTROL, DBG_log("sending disconnect to NM failed, you may need to do it manually"));
+                if (st->st_connection->remotepeertype == CISCO
+                    && st->st_connection->nmconfigured) {
+                    if(!do_command(st->st_connection, &st->st_connection->spd, "disconnectNM", st)) {
+                        DBG(DBG_CONTROL, DBG_log("sending disconnect to NM failed, you may need to do it manually"));
+                    }
                 }
-           }
 #endif
-	    if(st!=NULL && IS_PHASE1_INIT(st->st_state)) {
-		delete_event(st);
-		release_whack(st);
+                if(IS_PHASE1_INIT(st->st_state)) {
+                    delete_event(st);
+                    release_whack(st);
+                }
+                if(IS_QUICK(st->st_state)) {
+                    delete_state(st);
+                }
             }
-	    if(st!=NULL && IS_QUICK(st->st_state)) {
-		delete_state(st);
-	    }
             break;
     }
 
