@@ -41,7 +41,8 @@
 #ifdef XAUTH_USEPAM
 #include <security/pam_appl.h>
 #endif
-#include "connections.h"	/* needs id.h */
+#include "pluto/connections.h"	/* needs id.h */
+#include "hostpair.h"
 #include "keys.h"
 #include "packet.h"
 #include "demux.h"	/* needs packet.h */
@@ -50,7 +51,7 @@
 #include "kernel.h"	/* needs connections.h */
 #include "log.h"
 #include "cookie.h"
-#include "server.h"
+#include "pluto/server.h"
 #include "spdb.h"
 #include "timer.h"
 #include "rnd.h"
@@ -78,7 +79,7 @@
 #ifdef NAT_TRAVERSAL
 #include "nat_traversal.h"
 #endif
-#include "virtual.h"
+#include "pluto/virtual.h"
 #include "dpd.h"
 #include "x509more.h"
 #include "tpm/tpm.h"
@@ -225,10 +226,12 @@ aggr_inI1_outR1_common(struct msg_digest *md
     struct state *st;
     struct payload_digest *const sa_pd = md->chain[ISAKMP_NEXT_SA];
     pb_stream *keyex_pbs = &md->chain[ISAKMP_NEXT_KE]->pbs;
-    struct connection *c = find_host_connection(&md->iface->ip_addr
+    lset_t policy_hint = LEMPTY;
+    struct connection *c = find_host_connection(ANY_MATCH, &md->iface->ip_addr
 						, md->iface->port
+                                                , KH_ANY
 						, &md->sender
-						, md->sender_port, LEMPTY);
+						, md->sender_port, LEMPTY, POLICY_IKEV1_DISABLE, &policy_hint);
 
 
 #if 0
@@ -236,7 +239,7 @@ aggr_inI1_outR1_common(struct msg_digest *md
     if (c == NULL && md->iface->ike_float)
     {
 	c = find_host_connection(&md->iface->addr, NAT_T_IKE_FLOAT_PORT
-				 , &md->sender, md->sender_port, LEMPTY);
+				 , &md->sender, md->sender_port, LEMPTY, POLICY_IKEV1_DISABLE, &policy_hint);
     }
 #endif
 #endif
@@ -246,8 +249,8 @@ aggr_inI1_outR1_common(struct msg_digest *md
 	/* see if a wildcarded connection can be found */
  	pb_stream pre_sa_pbs = sa_pd->pbs;
  	lset_t policy = preparse_isakmp_sa_body(&pre_sa_pbs) | POLICY_AGGRESSIVE;
-	c = find_host_connection(&md->iface->ip_addr, pluto_port
-				 , (ip_address*)NULL, md->sender_port, policy);
+	c = find_host_connection(ANY_MATCH, &md->iface->ip_addr, pluto_port500
+				 , KH_ANY, (ip_address*)NULL, md->sender_port, policy, POLICY_IKEV1_DISABLE, &policy_hint);
 	if (c == NULL || (c->policy & POLICY_AGGRESSIVE) == 0) {
 	    loglog(RC_LOG_SERIOUS, "initial Aggressive Mode message from %s"
 		   " but no (wildcard) connection has been configured%s%s"
@@ -271,6 +274,9 @@ aggr_inI1_outR1_common(struct msg_digest *md
     st->st_localaddr  = md->iface->ip_addr;
     st->st_localport  = md->iface->port;
     st->st_interface  = md->iface;
+    /* IKE version numbers -- used mostly in logging */
+    st->st_ike_maj        = md->maj;
+    st->st_ike_min        = md->min;
     change_state(st, STATE_AGGR_R1);
 
     /* until we have clue who this is, then be conservative about allocating
@@ -404,6 +410,9 @@ aggr_inI1_outR1_tail(struct pluto_crypto_req_cont *pcrc
      */
 
     finish_dh_secretiv(st, r);
+    if(!r->pcr_success) {
+        return STF_FAIL + INVALID_KEY_INFORMATION;
+    }
 
     init_pbs(&reply_stream, reply_buffer, sizeof(reply_buffer), "reply packet");
 
@@ -675,8 +684,11 @@ aggr_inR1_outI2_crypto_continue(struct pluto_crypto_req_cont *pcrc
   st->st_calculating = FALSE;
 
   finish_dh_secretiv(st, r);
-
-  e = aggr_inR1_outI2_tail(md, NULL);
+  if(!r->pcr_success) {
+      e = STF_FAIL + INVALID_KEY_INFORMATION;
+  } else {
+      e = aggr_inR1_outI2_tail(md, NULL);
+  }
 
   if(dh->md != NULL) {
       complete_v1_state_transition(&dh->md, e);
@@ -991,12 +1003,11 @@ stf_status
 aggr_outI1(int whack_sock,
 	   struct connection *c,
 	   struct state *predecessor,
+           so_serial_t  *newstateno,
 	   lset_t policy,
 	   unsigned long try
 	   , enum crypto_importance importance
-#ifdef HAVE_LABELED_IPSEC
            , struct xfrm_user_sec_ctx_ike * uctx
-#endif
 	   )
 {
     struct state *st;
@@ -1004,18 +1015,26 @@ aggr_outI1(int whack_sock,
 
     /* set up new state */
     cur_state = st = new_state();
+    if(newstateno) *newstateno = st->st_serialno;
+
     st->st_connection = c;
 #ifdef HAVE_LABELED_IPSEC
     st->sec_ctx = NULL;
 #endif
     set_state_ike_endpoints(st, c);
 
+    DBG_log("aggr_outI1");
 #ifdef DEBUG
     extra_debugging(c);
 #endif
     st->st_policy = policy & ~POLICY_IPSEC_MASK;
     st->st_whack_sock = whack_sock;
     st->st_try = try;
+
+    /* IKE version numbers -- used mostly in logging */
+    st->st_ike_maj        = IKEv1_MAJOR_VERSION;
+    st->st_ike_min        = IKEv1_MINOR_VERSION;
+
     change_state(st, STATE_AGGR_I1);
 
     get_cookie(TRUE, st->st_icookie, COOKIE_SIZE, &c->spd.that.host_addr);
@@ -1048,9 +1067,7 @@ aggr_outI1(int whack_sock,
     if (HAS_IPSEC_POLICY(policy))
 	add_pending(dup_any(whack_sock), st, c, policy, 1
 	    , predecessor == NULL? SOS_NOBODY : predecessor->st_serialno
-#ifdef HAVE_LABELED_IPSEC
 	     , uctx
-#endif
 		   );
 
     if (predecessor == NULL) {
