@@ -1,12 +1,14 @@
 /* information about connections between hosts and clients
  * Copyright (C) 1998-2010  D. Hugh Redelmeier.
- * Copyright (C) 2007 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2007,2014 Michael Richardson <mcr@xelerance.com>
  * Copyright (C) 2007 Ken Bantoft <ken@xelerance.com>
  * Copyright (C) 2009 Stefan Arentz <stefan@arentz.ca>
  * Copyright (C) 2009-2010 David McCullough <david_mccullough@securecomputing.com>
  * Copyright (C) 2007-2010 Paul Wouters <paul@xelerance.com>
  * Copyright (C) 2010 Avesh Agarwal <avagarwa@redhat.com>
  * Copyright (C) 2010 Tuomo Soini <tis@foobar.fi>
+ * Copyright (C) 2012 Paul Wouters <pwouters@redhat.com>
+ * Copyright (C) 2012 Panagiotis Tamtamis <tamtamis@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -52,7 +54,7 @@
 #ifdef XAUTH_USEPAM
 #include <security/pam_appl.h>
 #endif
-#include "connections.h"	/* needs id.h */
+#include "pluto/connections.h"	/* needs id.h */
 #include "pending.h"
 #include "foodgroups.h"
 #include "packet.h"
@@ -60,7 +62,7 @@
 #include "state.h"
 #include "timer.h"
 #include "ipsec_doi.h"	/* needs demux.h and state.h */
-#include "server.h"
+#include "pluto/server.h"
 #include "kernel.h"	/* needs connections.h */
 #include "log.h"
 #include "keys.h"
@@ -78,92 +80,10 @@
 #include "nat_traversal.h"
 #endif
 
-#include "virtual.h"
+#include "pluto/virtual.h"
 
 #include "hostpair.h"
 
-bool
-orient(struct connection *c)
-{
-    struct spd_route *sr;
-
-    if (!oriented(*c))
-    {
-	struct iface_port *p;
-
-	for (sr = &c->spd; sr; sr = sr->next)
-	{
-	    /* There can be more then 1 spd policy associated - required
-	     * for cisco split networking when remote_peer_type=cisco
-	     */
-	    if(c->remotepeertype == CISCO && sr != &c->spd ) continue;
-
-	    /* Note: this loop does not stop when it finds a match:
-	     * it continues checking to catch any ambiguity.
-	     */
-	    for (p = interfaces; p != NULL; p = p->next)
-	    {
-#ifdef NAT_TRAVERSAL
-		if (p->ike_float) continue;
-#endif
-
-#ifdef HAVE_LABELED_IPSEC
-		if (c->loopback && sameaddr(&sr->this.host_addr, &p->ip_addr)) {
-		DBG(DBG_CONTROLMORE,
-			DBG_log("loopback connections \"%s\" with interface %s!"
-			 , c->name, p->ip_dev->id_rname));
-			c->interface = p;
-			break;
-		}
-#endif
-
-		for (;;)
-		{
-		    /* check if this interface matches this end */
-		    if (sameaddr(&sr->this.host_addr, &p->ip_addr)
-			&& (kern_interface != NO_KERNEL
-			    || sr->this.host_port == pluto_port))
-		    {
-			if (oriented(*c))
-			{
-			    if (c->interface->ip_dev == p->ip_dev)
-				loglog(RC_LOG_SERIOUS
-				       , "both sides of \"%s\" are our interface %s!"
-				       , c->name, p->ip_dev->id_rname);
-			    else
-				loglog(RC_LOG_SERIOUS, "two interfaces match \"%s\" (%s, %s)"
-				       , c->name, c->interface->ip_dev->id_rname, p->ip_dev->id_rname);
-			    terminate_connection(c->name);
-			    c->interface = NULL;	/* withdraw orientation */
-			    return FALSE;
-			}
-			c->interface = p;
-		    }
-
-		    /* done with this interface if it doesn't match that end */
-		    if (!(sameaddr(&sr->that.host_addr, &p->ip_addr)
-			  && (kern_interface!=NO_KERNEL
-			      || sr->that.host_port == pluto_port)))
-			break;
-
-		    /* swap ends and try again.
-		     * It is a little tricky to see that this loop will stop.
-		     * Only continue if the far side matches.
-		     * If both sides match, there is an error-out.
-		     */
-		    {
-			struct end t = sr->this;
-
-			sr->this = sr->that;
-			sr->that = t;
-		    }
-		}
-	    }
-	}
-    }
-    return oriented(*c);
-}
-
 struct initiate_stuff {
     int    whackfd;
     lset_t moredebug;
@@ -197,19 +117,20 @@ initiate_a_connection(struct connection *c
     else if (c->kind != CK_PERMANENT)
     {
 	if (isanyaddr(&c->spd.that.host_addr)) {
-#ifdef DYNAMICDNS
-	    if (c->dnshostname != NULL) {
-		loglog(RC_NOPEERIP, "cannot initiate connection without resolved dynamic peer IP address, will keep retrying");
-		success = 1;
+            if(c->spd.that.host_type == KH_IPHOSTNAME
+               && c->spd.that.host_address_list.address_list == NULL) {
+                loglog(RC_NOPEERIP, "dns resolution for %s not yet complete, still trying"
+                       , c->spd.that.host_addr_name);
+                success = 1;
 		c->policy |= POLICY_UP;
-	    } else
-#endif
+            } else {
 		loglog(RC_NOPEERIP, "cannot initiate connection without knowing peer IP address (kind=%s)"
 		       , enum_show(&connection_kind_names, c->kind));
-	} else
-	    loglog(RC_WILDCARD, "cannot initiate connection with ID wildcards (kind=%s)"
-		   , enum_show(&connection_kind_names, c->kind));
-    }
+            }
+        } else
+            loglog(RC_WILDCARD, "cannot initiate connection with ID wildcards (kind=%s)"
+                   , enum_show(&connection_kind_names, c->kind));
+        }
     else
     {
 	/* We will only request an IPsec SA if policy isn't empty
@@ -234,11 +155,9 @@ initiate_a_connection(struct connection *c
 
 	{
 	    whackfd = dup(whackfd);
-	    ipsecdoi_initiate(whackfd, c, c->policy, 1
+	    ipsecdoi_initiate(whackfd, NULL, c, c->policy, 1
 			      , SOS_NOBODY, importance
-#ifdef HAVE_LABELED_IPSEC
-                                        , NULL
-#endif
+                              , NULL_POLICY
 			     );
 	    success = 1;
 	}
@@ -279,48 +198,93 @@ initiate_connection(const char *name, int whackfd
     close_any(is.whackfd);
 }
 
+bool kick_adns_connection(struct connection *c, err_t ugh)
+{
+    bool kicknow = FALSE;
+    struct state *st = state_with_serialno(c->newest_isakmp_sa);
+
+    if(st != NULL
+       && (IS_ISAKMP_SA_ESTABLISHED(st->st_state) || IS_PARENT_SA_ESTABLISHED(st->st_state))) {
+
+        /* the connection is up, no point in doing it again */
+        DBG(DBG_DNS
+            , DBG_log("  DNS already connected, do not reinit"));
+        return FALSE;
+    }
+
+    if(st == NULL) {
+        st = state_with_serialno(c->prospective_parent_sa);
+
+        /* might need to put some state restrictions on this, or look at how many retries */
+        if(st != NULL && ugh != NULL) {
+            DBG(DBG_DNS
+                , DBG_log("  DNS connection attempt in progress, do not restart"));
+            return FALSE;
+        }
+    }
+
+    /* arrange to rekey the phase 1, if there was one. */
+    /* we should do this if we immediately found a new address in the list.
+     * the DNS lookup will be done again if this was not the result of a failure
+     * of a previous DNS lookup: on failure, we wait for the rescan event
+     */
+    if (c->spd.that.host_type == KH_IPHOSTNAME) {
+        kicknow = kicknow || kick_adns_connection_lookup(c, &c->spd.that, ugh == NULL);
+    }
+    if (!kicknow && c->spd.this.host_type == KH_IPHOSTNAME) {
+        kicknow = kicknow || kick_adns_connection_lookup(c, &c->spd.this, ugh == NULL);
+    }
+
+    if(kicknow && c->policy & POLICY_UP) {
+        struct initiate_stuff is;
+        char targetaddr[ADDRTOT_BUF];
+        is.whackfd   = NULL_FD;
+        is.moredebug = 0;
+        is.importance= pcim_ongoing_crypto;
+
+        DBG(DBG_DNS
+            , DBG_log("  attempting to reinit"));
+
+        set_cur_connection(c);
+        addrtot(&c->spd.that.host_addr, 0, targetaddr, sizeof(targetaddr));
+        loglog(RC_NOPEERIP, "trying to initiate to address %s (name=%s)"
+               , targetaddr
+               , c->spd.that.host_addr_name);
+
+	initiate_a_connection(c, &is);
+    } else {
+        DBG(DBG_DNS
+            , DBG_log("  no additional DNS answers,  not re-attempting"));
+    }
+
+    return kicknow;
+}
+
+
 void
 restart_connections_by_peer(struct connection *c)
 {
     struct connection *d;
 
-    if (c->host_pair == NULL)
+    if (c->IPhost_pair == NULL)
    	   return;
 
-    d = c->host_pair->connections;
-    for (; d != NULL; d = d->hp_next) {
-	   if (
-#ifdef DYNAMICDNS
-	       (c->dnshostname && d->dnshostname && (strcmp(c->dnshostname, d->dnshostname) == 0))
-	   	|| (c->dnshostname == NULL && d->dnshostname == NULL &&
-#endif /* DYNAMICDNS */
-		sameaddr(&d->spd.that.host_addr, &c->spd.that.host_addr)
-#ifdef DYNAMICDNS
-		)
-#endif /* DYNAMICDNS */
-		)
-	       terminate_connection(d->name);
+    d = c->IPhost_pair->connections;
+    for (; d != NULL; d = d->IPhp_next) {
+        if(compare_end_addr_names(&c->spd.that, &d->spd.that)) {
+                terminate_connection(d->name);
+        }
     }
 
-#ifdef DYNAMICDNS
     update_host_pairs(c);
-#endif /* DYNAMICDNS */
 
-    if (c->host_pair == NULL)
+    if (c->IPhost_pair == NULL)
     	   return;
-    d = c->host_pair->connections;
-    for (; d != NULL; d = d->hp_next) {
-    	   if (
-#ifdef DYNAMICDNS
-	       (c->dnshostname && d->dnshostname && (strcmp(c->dnshostname, d->dnshostname) == 0))
-	   	|| (c->dnshostname == NULL && d->dnshostname == NULL &&
-#endif /* DYNAMICDNS */
-		sameaddr(&d->spd.that.host_addr, &c->spd.that.host_addr)
-#ifdef DYNAMICDNS
-		)
-#endif /* DYNAMICDNS */
-		)
-	       initiate_connection(d->name, NULL_FD, 0, pcim_demand_crypto);
+    d = c->IPhost_pair->connections;
+    for (; d != NULL; d = d->IPhp_next) {
+        if (compare_end_addr_names(&c->spd.that, &d->spd.that)) {
+            initiate_connection(d->name, NULL_FD, 0, pcim_demand_crypto);
+        }
     }
 }
 
@@ -480,7 +444,7 @@ cannot_oppo(struct connection *c
 	}
 #endif /* DEBUG */
 
-	if (!route_and_eroute(c, shunt_spd, st))
+	if (!route_and_eroute(c, shunt_spd, shunt_spd, st))
 	{
 	    whack_log(RC_OPPOFAILURE
 		      , "failed to instantiate shunt policy %s for %s to %s"
@@ -490,8 +454,11 @@ cannot_oppo(struct connection *c
 	return;
     }
 
-#ifdef KLIPS /* This should really be 'if opportunistic is supported' - netlink has acquires too */
-    if (b->held)
+    /*
+     * NETKEY default for level param in tmpl is required, so no traffic will
+     * transmitted until an SA is fully up
+     */
+    if (b->held && kern_interface != USE_NETKEY)
     {
 	int failure_shunt = b->failure_shunt;
 
@@ -510,14 +477,11 @@ cannot_oppo(struct connection *c
 	    , b->transport_proto
 	    , ughmsg);
     }
-#endif
 }
 
 static int initiate_ondemand_body(struct find_oppo_bundle *b
     , struct adns_continuation *ac, err_t ac_ugh
-#ifdef HAVE_LABELED_IPSEC
     , struct xfrm_user_sec_ctx_ike *uctx
-#endif
     );	/* forward */
 
 int
@@ -526,9 +490,7 @@ initiate_ondemand(const ip_address *our_client
 , int transport_proto
 , bool held
 , int whackfd
-#ifdef HAVE_LABELED_IPSEC
-, struct xfrm_user_sec_ctx_ike *uctx
-#endif
+, struct xfrm_user_sec_ctx_ike *uctx UNUSED
 , err_t why)
 {
     struct find_oppo_bundle b;
@@ -544,10 +506,7 @@ initiate_ondemand(const ip_address *our_client
     b.whackfd = whackfd;
     b.step = fos_start;
     return initiate_ondemand_body(&b, NULL, NULL
-#ifdef HAVE_LABELED_IPSEC
-				 , uctx
-#endif
-				 );
+				 , uctx);
 }
 
 static void
@@ -563,7 +522,6 @@ continue_oppo(struct adns_continuation *acr, err_t ugh)
      */
     whack_log_fd = whackfd;
 
-#ifdef KLIPS
     /* Discover and record whether %hold has gone away.
      * This could have happened while we were awaiting DNS.
      * We must check BEFORE any call to cannot_oppo.
@@ -571,7 +529,6 @@ continue_oppo(struct adns_continuation *acr, err_t ugh)
     if (was_held)
 	cr->b.held = has_bare_hold(&cr->b.our_client, &cr->b.peer_client
 	    , cr->b.transport_proto);
-#endif
 
 #ifdef DEBUG
     /* if we're going to ignore the error, at least note it in debugging log */
@@ -617,10 +574,7 @@ continue_oppo(struct adns_continuation *acr, err_t ugh)
     else
     {
 	(void)initiate_ondemand_body(&cr->b, &cr->ac, ugh
-#ifdef HAVE_LABELED_IPSEC
-				     , NULL
-#endif
-				    );
+				     , NULL_POLICY);
 	whackfd = NULL_FD;	/* was handed off */
     }
 
@@ -745,12 +699,9 @@ check_txt_recs(enum myid_state try_state
 /* return true if we did something */
 static int
 initiate_ondemand_body(struct find_oppo_bundle *b
-, struct adns_continuation *ac
-, err_t ac_ugh
-#ifdef HAVE_LABELED_IPSEC
-    , struct xfrm_user_sec_ctx_ike *uctx
-#endif
-)
+                       , struct adns_continuation *ac
+                       , err_t ac_ugh
+                       , struct xfrm_user_sec_ctx_ike *uctx)
 {
     struct connection *c;
     struct spd_route *sr;
@@ -763,7 +714,7 @@ initiate_ondemand_body(struct find_oppo_bundle *b
     int work = 0;
 
     /* on klips/mast assume we will do something */
-    work = (kern_interface == USE_KLIPS || kern_interface == USE_MASTKLIPS);
+    work = (kern_interface == USE_KLIPS || kern_interface == USE_MASTKLIPS || kern_interface == USE_NETKEY);
 
     /* What connection shall we use?
      * First try for one that explicitly handles the clients.
@@ -776,19 +727,17 @@ initiate_ondemand_body(struct find_oppo_bundle *b
 
 
 #ifdef HAVE_LABELED_IPSEC
-    char sec_ctx_value[256];
+    char sec_ctx_value[MAX_SECCTX_LEN];
     memset(sec_ctx_value, 0, sizeof(sec_ctx_value));
     if(uctx != NULL) {
-    memcpy(sec_ctx_value, uctx->sec_ctx_value, uctx->ctx_len);
+	memcpy(sec_ctx_value, uctx->sec_ctx_value, uctx->ctx_len);
     }
-    snprintf(demandbuf, 256, "initiate on demand from %s:%d to %s:%d proto=%d state: %s because: %s with security context %s"
-             , ours, ourport, his, hisport, b->transport_proto
-             , oppo_step_name[b->step], b->want, sec_ctx_value);
-#else
+    DBG(DBG_CONTROLMORE, DBG_log("received security label string: %s", sec_ctx_value));
+#endif
+
     snprintf(demandbuf, 256, "initiate on demand from %s:%d to %s:%d proto=%d state: %s because: %s"
 	     , ours, ourport, his, hisport, b->transport_proto
 	     , oppo_step_name[b->step], b->want);
-#endif
 
     if(DBGP(DBG_OPPOINFO)) {
 	openswan_log("%s", demandbuf);
@@ -854,21 +803,16 @@ initiate_ondemand_body(struct find_oppo_bundle *b
 	/* otherwise, there is some kind of static conn that can handle
 	 * this connection, so we initiate it */
 
-#ifdef KLIPS
-	if (b->held)
+	if (b->held && kern_interface != USE_NETKEY)
 	{
 	    /* what should we do on failure? */
 	    (void) assign_hold(c, sr, b->transport_proto, &b->our_client, &b->peer_client);
 	}
-#endif
 
 	if(!loggedit) { openswan_log("%s", demandbuf); loggedit=TRUE; }
-	ipsecdoi_initiate(b->whackfd, c, c->policy, 1
+	ipsecdoi_initiate(b->whackfd, NULL, c, c->policy, 1
 			  , SOS_NOBODY, pcim_local_crypto
-#ifdef HAVE_LABELED_IPSEC
-			  , uctx
-#endif
-			);
+			  , uctx);
 	b->whackfd = NULL_FD;	/* protect from close */
     }
     else
@@ -1253,7 +1197,6 @@ initiate_ondemand_body(struct find_oppo_bundle *b
 			  " between %s and %s with %s as peer"
 			, ocb, pcb, pb);
 
-#ifdef KLIPS
 		    if (b->held)
 		    {
 			/* Replace HOLD with PASS.
@@ -1266,7 +1209,6 @@ initiate_ondemand_body(struct find_oppo_bundle *b
 			    , TRUE, b->transport_proto
 			    , "no suitable connection");
 		    }
-#endif
 		}
 		else
 		{
@@ -1275,7 +1217,6 @@ initiate_ondemand_body(struct find_oppo_bundle *b
 		    passert(c->gw_info != NULL);
 		    passert(HAS_IPSEC_POLICY(c->policy));
 		    passert(LHAS(LELEM(RT_UNROUTED) | LELEM(RT_ROUTED_PROSPECTIVE), c->spd.routing));
-#ifdef KLIPS
 		    if (b->held)
 		    {
 			/* what should we do on failure? */
@@ -1283,18 +1224,15 @@ initiate_ondemand_body(struct find_oppo_bundle *b
 					   , b->transport_proto
 					   , &b->our_client, &b->peer_client);
 		    }
-#endif
 		    c->gw_info->key->last_tried_time = now();
 		    DBG(DBG_OPPO|DBG_CONTROL,
 			DBG_log("initiate on demand from %s:%d to %s:%d proto=%d state: %s because: %s"
 				, ours, ourport, his, hisport, b->transport_proto
 				, oppo_step_name[b->step], b->want));
 
-		    ipsecdoi_initiate(b->whackfd, c, c->policy, 1
+		    ipsecdoi_initiate(b->whackfd, NULL, c, c->policy, 1
 				      , SOS_NOBODY, pcim_local_crypto
-#ifdef HAVE_LABELED_IPSEC
 					, NULL /*shall we pass uctx for opportunistic connections?*/
-#endif
 				     );
 		    b->whackfd = NULL_FD;	/* protect from close */
 		}
@@ -1589,12 +1527,11 @@ ISAKMP_SA_established(struct connection *c, so_serial_t serial)
 	    && same_id(&c->spd.that.id, &d->spd.that.id)
 	    && (!sameaddr(&c->spd.that.host_addr, &d->spd.that.host_addr)
 		|| (c->spd.that.host_port != d->spd.that.host_port))
-#ifdef DYNAMICDNS
-	    && !(c->dnshostname && d->dnshostname && (strcmp(c->dnshostname, d->dnshostname) == 0))
-#endif /* DYNAMICDNS */
+	    && !(c->spd.that.host_type == KH_IPHOSTNAME
+                 && d->spd.that.host_type == KH_IPHOSTNAME
+                 && (strcasecmp(c->spd.that.host_addr_name, d->spd.that.host_addr_name) == 0))
 	       )
-
-	    {
+                {
 		release_connection(d, FALSE);
 	    }
 	    d = next;
@@ -1637,65 +1574,53 @@ shunt_owner(const ip_subnet *ours, const ip_subnet *his)
 
 static void connection_check_ddns1(struct connection *c)
 {
-    struct connection *d;
-    ip_address new_addr;
-    /* const struct af_info afi; */
-    const char *e;
+    struct connection *d, *dslow;
+    bool dslowflip;
 
     if (NEVER_NEGOTIATE(c->policy))
 	return;
 
-    if (c->dnshostname == NULL)
-	return;
+    if (c->spd.this.host_type != KH_IPHOSTNAME
+        && c->spd.that.host_type != KH_IPHOSTNAME)
+        return;
 
-    if (!isanyaddr(&c->spd.that.host_addr)) {
+    if (c->newest_isakmp_sa != SOS_NOBODY) {
 	DBG(DBG_CONTROLMORE,
-	    DBG_log("pending ddns: connection \"%s\" has address",
+	    DBG_log("pending ddns: connection \"%s\" already up",
 	    c->name));
 	return;
     }
 
     if (c->spd.that.has_client_wildcard || c->spd.that.has_port_wildcard
 	    || ((c->policy & POLICY_SHUNT_MASK) == 0 &&
-	    c->spd.that.has_id_wildcards)) {
+	    c->spd.that.id.has_wildcards)) {
 	DBG(DBG_CONTROL,
 	    DBG_log("pending ddns: connection \"%s\" with wildcard not started",
 	    c->name));
 	return;
     }
 
-
-    e = ttoaddr(c->dnshostname, 0, 0, &new_addr);
-    if (e != NULL) {
+    if (c->spd.this.host_type != KH_IPHOSTNAME
+         && isanyaddr(&c->spd.this.host_addr)) {
 	DBG(DBG_CONTROL,
-	    DBG_log("pending ddns: connection \"%s\" lookup of \"%s\" failed: %s",
-	    c->name, c->dnshostname, e));
+	    DBG_log("pending ddns: connection \"%s\" still no this address for \"%s\"",
+	    c->name, c->spd.this.host_addr_name));
 	return;
     }
 
-    if (isanyaddr(&new_addr)) {
+    if (c->spd.that.host_type != KH_IPHOSTNAME
+         && isanyaddr(&c->spd.that.host_addr)) {
 	DBG(DBG_CONTROL,
-	    DBG_log("pending ddns: connection \"%s\" still no address for \"%s\"",
-	    c->name, c->dnshostname));
+	    DBG_log("pending ddns: connection \"%s\" still no that address for \"%s\"",
+	    c->name, c->spd.that.host_addr_name));
 	return;
     }
 
-    /* I think this is ok now we check everything above ? */
-    c->kind = CK_PERMANENT;
-    c->spd.that.host_addr = new_addr;
-
     /* a small bit of code from default_end to fixup the end point */
     /* default nexthop to other side */
     if (isanyaddr(&c->spd.this.host_nexthop))
 	c->spd.this.host_nexthop = c->spd.that.host_addr;
 
-    /* default client to subnet containing only self
-     * XXX This may mean that the client's address family doesn't match
-     * tunnel_addr_family.
-     */
-    if (!c->spd.that.has_client)
-	addrtosubnet(&c->spd.that.host_addr, &c->spd.that.client);
-
     /*
      * reduce the work we do by updating all connections waiting for this
      * lookup
@@ -1704,18 +1629,26 @@ static void connection_check_ddns1(struct connection *c)
     initiate_connection(c->name, NULL_FD, 0, pcim_demand_crypto);
 
     /* no host pairs,  no more to do */
-    if (c->host_pair == NULL)
+    if (c->IPhost_pair == NULL)
 	return;
 
-    d = c->host_pair->connections;
-    for (; d != NULL; d = d->hp_next) {
+    d = c->IPhost_pair->connections;
+    dslow = c->IPhost_pair->connections;
+    dslowflip = TRUE;
+    for (; d != NULL; d = d->IPhp_next) {
+
+        /* protect: against corrupted lists */
+        if(dslowflip) {
+            dslow = dslow->IPhp_next;
+        }
+        dslowflip = !dslowflip;
+        passert(d != dslow);
+
+
 	/* just in case we see ourselves */
 	if (c == d)
 	    continue;
-	if ((c->dnshostname && d->dnshostname &&
-		(strcmp(c->dnshostname, d->dnshostname) == 0))
-		|| (c->dnshostname == NULL && d->dnshostname == NULL &&
-		sameaddr(&d->spd.that.host_addr, &c->spd.that.host_addr)))
+	if (compare_end_addr_names(&c->spd.that, &d->spd.that))
 	    initiate_connection(d->name, NULL_FD, 0, pcim_demand_crypto);
     }
 }
@@ -1740,7 +1673,6 @@ void connection_check_ddns(void)
 }
 #endif /* DYNAMICDNS */
 
-
 #define PENDING_PHASE2_INTERVAL (60*2) /*time between scans of pending phase2*/
 
 /*
@@ -1770,49 +1702,50 @@ void connection_check_phase2(void)
 	}
 
 	DBG(DBG_CONTROL,
-	    DBG_log("pending review: connection \"%s\" checked", c->name));
+	    DBG_log("pending review: connection \"%s\" being checked", c->name));
+
+	if(c->newest_isakmp_sa == SOS_NOBODY
+           || pending_check_timeout(c)) {
 
-	if(pending_check_timeout(c)) {
 	    struct state *p1st;
+            bool kicknow = kick_adns_connection(c, NULL);
+
 	    openswan_log("pending Quick Mode with %s \"%s\" took too long -- replacing phase 1"
 			 , ip_str(&c->spd.that.host_addr)
 			 , c->name);
 
-	    p1st = find_phase1_state(c, ISAKMP_SA_ESTABLISHED_STATES|PHASE1_INITIATOR_STATES);
+            if(kicknow) {
+                /*
+                 * look for a phase 1 to kill, but not point in doing that until we
+                 * actually have something new to try.
+                 */
+                p1st = find_phase1_state(c, ISAKMP_SA_ESTABLISHED_STATES|PHASE1_INITIATOR_STATES);
+                if(p1st) {
+                    delete_event(p1st);
+                    event_schedule(EVENT_SA_REPLACE, 0, p1st);
+                }
+                else {
+                    /* start a new connection. Something wanted it up, and we have new info */
+                    struct initiate_stuff is;
 
-	    if(p1st) {
-		/* arrange to rekey the phase 1, if there was one. */
-#ifdef DYNAMICDNS
-	    if (c->dnshostname != NULL)
-		restart_connections_by_peer(c);
-	    else
-	    {
-#endif /* DYNAMICDNS */
-		delete_event(p1st);
-		event_schedule(EVENT_SA_REPLACE, 0, p1st);
-#ifdef DYNAMICDNS
-	    }
-#endif /* DYNAMICDNS */
+                    is.whackfd   = NULL_FD;
+                    is.moredebug = 0;
+                    is.importance= pcim_local_crypto;
 
-	    } else {
-		/* start a new connection. Something wanted it up */
-		struct initiate_stuff is;
+                    initiate_a_connection(c, &is);
+                }
+            }
+	} else {
+            DBG(DBG_CONTROL,
+                DBG_log("pending review: connection \"%s\" not time for review", c->name));
+        }
 
-		is.whackfd   = NULL_FD;
-		is.moredebug = 0;
-		is.importance= pcim_local_crypto;
-
-		initiate_a_connection(c, &is);
-	    }
-	}
     }
 }
 
 void init_connections(void)
 {
-#ifdef DYNAMICDNS
     event_schedule(EVENT_PENDING_DDNS, PENDING_DDNS_INTERVAL, NULL);
-#endif
     event_schedule(EVENT_PENDING_PHASE2, PENDING_PHASE2_INTERVAL, NULL);
 }
 
