@@ -17,6 +17,9 @@
 
 #include <net/if.h>
 
+/* global variables */
+extern u_int16_t pluto_port500;	        /* Pluto's port (usually 500) */
+extern u_int16_t pluto_port4500;	/* Pluto's NAT port (usually 4500) */
 extern bool can_do_IPcomp;  /* can system actually perform IPCOMP? */
 
 /*
@@ -77,6 +80,8 @@ struct kernel_sa {
 	const ip_subnet *src_client;
 	const ip_subnet *dst_client;
 
+        bool inbound;
+        bool add_selector;
 	ipsec_spi_t spi;
 	unsigned proto;
 	unsigned int transport_proto;
@@ -154,16 +159,14 @@ struct kernel_ops {
 		       time_t use_lifetime,
 		       enum pluto_sadb_operations op,
 		       const char *text_said
-#ifdef HAVE_LABELED_IPSEC
-		      , char *policy_label
-#endif
+                       , char *policy_label
 		       );
     bool (*shunt_eroute)(struct connection *c
-			 , struct spd_route *sr
+			 , const struct spd_route *sr
 			 , enum routing_t rt_kind
 			 , enum pluto_sadb_operations op
 			 , const char *opname);
-    bool (*sag_eroute)(struct state *st, struct spd_route *sr
+    bool (*sag_eroute)(struct state *st, const struct spd_route *sr
 		       , enum pluto_sadb_operations op, const char *opname);
     bool (*eroute_idle)(struct state *st, time_t idle_max);
     void (*remove_orphaned_holds)(int transportproto
@@ -183,7 +186,7 @@ struct kernel_ops {
 			   ipsec_spi_t max,
 			   const char *text_said);
     bool (*docommand)(struct connection *c
-		      , struct spd_route *sr
+		      , const struct spd_route *sr
 		      , const char *verb
 		      , struct state *st);
     void (*process_ifaces)(struct raw_iface *rifaces);
@@ -208,7 +211,7 @@ extern struct raw_iface *find_raw_ifaces6(void);
 
 /* helper for invoking call outs */
 extern int fmt_common_shell_out(char *buf, int blen, struct connection *c
-				, struct spd_route *sr, struct state *st);
+				, const struct spd_route *sr, struct state *st);
 
 #ifdef KLIPS_MAST
 /* KLIPS/mast/pfkey things */
@@ -216,28 +219,28 @@ extern bool pfkey_plumb_mast_device(int mast_dev);
 #endif
 
 /* many bits reach in to use this, but maybe shouldn't */
-extern bool do_command(struct connection *c, struct spd_route *sr, const char *verb, struct state *st);
+extern bool do_command(struct connection *c, const struct spd_route *sr, const char *verb, struct state *st);
 
 #if defined(linux)
-extern bool do_command_linux(struct connection *c, struct spd_route *sr
+extern bool do_command_linux(struct connection *c, const struct spd_route *sr
 			     , const char *verb, struct state *st);
 extern bool invoke_command(const char *verb, const char *verb_suffix, char *cmd);
 #endif
 
 #if defined(__FreeBSD__)
-extern bool do_command_freebsd(struct connection *c, struct spd_route *sr
+extern bool do_command_freebsd(struct connection *c, const struct spd_route *sr
 			       , const char *verb, struct state *st);
 extern bool invoke_command(const char *verb, const char *verb_suffix, char *cmd);
 #endif
 
 #if defined(macintosh) || (defined(__MACH__) && defined(__APPLE__))
-extern bool do_command_darwin(struct connection *c, struct spd_route *sr
+extern bool do_command_darwin(struct connection *c, const struct spd_route *sr
 			       , const char *verb, struct state *st);
 extern bool invoke_command(const char *verb, const char *verb_suffix, char *cmd);
 #endif
 
 #if defined(__CYGWIN32__)
-extern bool do_command_cygwin(struct connection *c, struct spd_route *sr
+extern bool do_command_cygwin(struct connection *c, const struct spd_route *sr
 			      , const char *verb, struct state *st);
 #endif
 
@@ -307,15 +310,13 @@ struct bare_shunt **bare_shunt_ptr(const ip_subnet *ours
 # define EM_MAXRELSPIS 4	/* AH ESP IPCOMP IPIP */
 #endif
 
-#ifdef HAVE_LABELED_IPSEC
-struct xfrm_user_sec_ctx_ike; /* forward declaration of tag */
-#endif
+#define USER_SEC_CTX_NULL NULL  /* makes it easier to read/comprehend */
+struct xfrm_user_sec_ctx_ike;   /* forward declaration of tag */
+
 extern void record_and_initiate_opportunistic(const ip_subnet *
                                               , const ip_subnet *
                                               , int transport_proto
-#ifdef HAVE_LABELED_IPSEC
                                               , struct xfrm_user_sec_ctx_ike *
-#endif
                                               , const char *why);
 extern void init_kernel(void);
 
@@ -350,28 +351,27 @@ extern ipsec_spi_t get_ipsec_spi(ipsec_spi_t avoid
 				 , bool tunnel_mode);
 extern ipsec_spi_t get_my_cpi(struct spd_route *sr, bool tunnel_mode);
 
-extern bool install_inbound_ipsec_sa(struct state *st);
-extern bool install_ipsec_sa(struct state *st, bool inbound_also);
+extern bool install_inbound_ipsec_sa(struct state *parent_st, struct state *st);
+extern bool install_ipsec_sa(struct state *parent_st, struct state *st, bool inbound_also);
 extern void delete_ipsec_sa(struct state *st, bool inbound_only);
 extern bool route_and_eroute(struct connection *c
-			     , struct spd_route *sr
+			     , const struct spd_route *sr
+			     , struct spd_route *orig_sr
 			     , struct state *st);
 
 extern bool was_eroute_idle(struct state *st, time_t idle_max);
 extern bool get_sa_info(struct state *st, bool inbound, time_t *ago);
 
 #ifdef NAT_TRAVERSAL
-extern bool update_ipsec_sa(struct state *st);
+extern bool update_ipsec_sa(struct state *parent_st, struct state *st);
 #endif
 
-extern bool eroute_connection(struct spd_route *sr
+extern bool eroute_connection(struct state *st, const struct spd_route *sr
 			      , ipsec_spi_t spi, unsigned int proto
 			      , enum eroute_type esatype
 			      , const struct pfkey_proto_info *proto_info
 			      , unsigned int op, const char *opname
-#ifdef HAVE_LABELED_IPSEC
 			      , char *policy_label
-#endif
 			      );
 
 static inline bool
@@ -395,6 +395,9 @@ extern bool kernel_overlap_supported(void);
 extern const char *kernel_if_name(void);
 extern void show_kernel_interface(void);
 
+extern void saref_init(void);
+
+
 #define _KERNEL_H_
 #endif /* _KERNEL_H_ */
 
