@@ -39,7 +39,7 @@
 
 #include "defs.h"
 #include "id.h"
-#include "connections.h"
+#include "pluto/connections.h"
 #include "state.h"
 #include "kernel.h"
 #include "kernel_pfkey.h"
@@ -145,7 +145,7 @@ bsdkame_process_raw_ifaces(struct raw_iface *rifaces)
 		if (q == NULL)
 		{
 		    /* matches nothing -- create a new entry */
-		    int fd = create_socket(ifp, ifp->name, pluto_port);
+		    int fd = create_socket(ifp, ifp->name, pluto_port500);
 
 		    if (fd < 0)
 			break;
@@ -169,9 +169,10 @@ bsdkame_process_raw_ifaces(struct raw_iface *rifaces)
 
 		    q->ip_addr = ifp->addr;
 		    q->fd = fd;
+                    init_iface_port(q);
 		    q->next = interfaces;
 		    q->change = IFN_ADD;
-		    q->port = pluto_port;
+		    q->port = pluto_port500;
 		    q->ike_float = FALSE;
 
 		    interfaces = q;
@@ -205,6 +206,7 @@ bsdkame_process_raw_ifaces(struct raw_iface *rifaces)
 			q->port = NAT_T_IKE_FLOAT_PORT;
 			q->fd = fd;
 			q->next = interfaces;
+                        init_iface_port(q);
 			q->change = IFN_ADD;
 			q->ike_float = TRUE;
 			interfaces = q;
@@ -477,9 +479,7 @@ bsdkame_raw_eroute(const ip_address *this_host
 		   , time_t use_lifetime UNUSED
 		   , enum pluto_sadb_operations op
 		   , const char *text_said UNUSED
-#ifdef HAVE_LABELED_IPSEC
 		   , char *policy_label UNUSED
-#endif
 		   )
 {
     const struct sockaddr *saddr = (const struct sockaddr *)&this_client->addr;
@@ -629,7 +629,7 @@ bsdkame_raw_eroute(const ip_address *this_host
  */
 static bool
 bsdkame_shunt_eroute(struct connection *c
-		     , struct spd_route *sr
+		     , const struct spd_route *sr
 		     , enum routing_t rt_kind
 		     , enum pluto_sadb_operations op
 		     , const char *opname)
@@ -895,7 +895,7 @@ bsdkame_shunt_eroute(struct connection *c
  */
 static bool
 bsdkame_sag_eroute(struct state *st
-		   , struct spd_route *sr
+		   , const struct spd_route *sr
 		   , unsigned op UNUSED
 		   , const char *opname UNUSED)
 {
@@ -925,9 +925,7 @@ bsdkame_sag_eroute(struct state *st
 			      , 0    /* use lifetime unused */
 			      , op
 			      , NULL /* said unused */
-#ifdef HAVE_LABELED_IPSEC
-			      , NULL /*unused*/
-#endif
+			      , NULL_POLICY /*unused*/
 			      );
 }
 
