@@ -38,7 +38,7 @@
 
 #include "defs.h"
 #include "id.h"
-#include "connections.h"
+#include "pluto/connections.h"
 #include "state.h"
 #include "kernel.h"
 #include "kernel_pfkey.h"
@@ -300,7 +300,7 @@ mast_process_raw_ifaces(struct raw_iface *rifaces)
 
 		vname = clone_str(useful_mast_name
 				  , "virtual device name mast");
-		fd = create_socket(ifp, vname, pluto_port);
+		fd = create_socket(ifp, vname, pluto_port500);
 
 		if (fd < 0)
 		    break;
@@ -320,8 +320,9 @@ mast_process_raw_ifaces(struct raw_iface *rifaces)
 		id->id_count++;
 
 		q->ip_addr = ifp->addr;
+                init_iface_port(q);
 		q->change = IFN_ADD;
-		q->port = pluto_port;
+		q->port = pluto_port500;
 		q->ike_float = FALSE;
 
 #ifdef NAT_TRAVERSAL
@@ -368,6 +369,7 @@ mast_process_raw_ifaces(struct raw_iface *rifaces)
 		    setportof(htons(NAT_T_IKE_FLOAT_PORT), &q->ip_addr);
 		    q->port = NAT_T_IKE_FLOAT_PORT;
 		    q->fd = fd;
+                    init_iface_port(q);
 		    q->change = IFN_ADD;
 		    q->ike_float = TRUE;
 
@@ -399,7 +401,7 @@ mast_process_raw_ifaces(struct raw_iface *rifaces)
 }
 
 static bool
-mast_do_command(struct connection *c, struct spd_route *sr
+mast_do_command(struct connection *c, const struct spd_route *sr
 		, const char *verb, struct state *st)
 {
     char cmd[2048];     /* arbitrary limit on shell command length */
@@ -477,9 +479,7 @@ mast_raw_eroute(const ip_address *this_host UNUSED
                , time_t use_lifetime UNUSED
                , enum pluto_sadb_operations op UNUSED
                , const char *text_said UNUSED
-#ifdef HAVE_LABELED_IPSEC
 	       , char *policy_label UNUSED
-#endif
 	       )
 {
     /* actually, we did all the work with iptables in _updown */
@@ -487,9 +487,7 @@ mast_raw_eroute(const ip_address *this_host UNUSED
     return pfkey_raw_eroute(this_host, this_client, that_host, that_client,
 		    spi, proto, transport_proto, satype,
 		    proto_info, use_lifetime, op, text_said
-#ifdef HAVE_LABELED_IPSEC
 		    , policy_label
-#endif
 		    );
 }
 
@@ -502,7 +500,7 @@ mast_raw_eroute(const ip_address *this_host UNUSED
  */
 static bool
 mast_shunt_eroute(struct connection *c UNUSED
-		   , struct spd_route *sr UNUSED
+		   , const struct spd_route *sr UNUSED
 		   , enum routing_t rt_kind UNUSED
 		   , enum pluto_sadb_operations op UNUSED
 		  , const char *opname UNUSED)
@@ -518,7 +516,7 @@ mast_shunt_eroute(struct connection *c UNUSED
  * @return TRUE if add was successful, FALSE otherwise
  */
 static bool
-mast_sag_eroute_replace(struct state *st, struct spd_route *sr)
+mast_sag_eroute_replace(struct state *st, const struct spd_route *sr)
 {
 	struct connection *c = st->st_connection;
 	struct state *old_st;
@@ -552,7 +550,7 @@ mast_sag_eroute_replace(struct state *st, struct spd_route *sr)
 
 /* install or remove eroute for SA Group */
 static bool
-mast_sag_eroute(struct state *st, struct spd_route *sr
+mast_sag_eroute(struct state *st, const struct spd_route *sr
 		, enum pluto_sadb_operations op, const char *opname UNUSED)
 {
     bool ok;
