@@ -48,7 +48,7 @@
 
 #include "defs.h"
 #include "id.h"
-#include "connections.h"
+#include "pluto/connections.h"
 #include "state.h"
 #include "kernel.h"
 #include "kernel_pfkey.h"
@@ -302,7 +302,7 @@ pfkey_get(pfkey_buf *buf)
 		, (unsigned) IPSEC_PFKEYv2_ALIGN);
 	}
 	else if (!(buf->msg.sadb_msg_pid == (unsigned)pid
-	/*	for now, unsolicited messages can be: 
+	/*	for now, unsolicited messages can be:
 	 *	K_SADB_ACQUIRE, K_SADB_REGISTER, K_SADB_X_NAT_T_NEW_MAPPING
 	 */
 	|| (buf->msg.sadb_msg_pid == 0 && buf->msg.sadb_msg_type == SADB_ACQUIRE)
@@ -453,10 +453,8 @@ process_pfkey_acquire(pfkey_buf *buf, struct sadb_ext *extensions[K_SADB_EXT_MAX
 	&& !(ugh = addrtypeof(src) == addrtypeof(dst)? NULL : "conflicting address types")
 	&& !(ugh = addrtosubnet(src, &ours))
 	&& !(ugh = addrtosubnet(dst, &his)))
-      record_and_initiate_opportunistic(&ours, &his, 0, 
-#ifdef HAVE_LABELED_IPSEC
-						NULL,
-#endif
+      record_and_initiate_opportunistic(&ours, &his, 0,
+						NULL_POLICY,
 						"%acquire-pfkey");
 
     if (ugh != NULL)
@@ -541,9 +539,7 @@ void pfkey_dequeue(void)
       record_and_initiate_opportunistic(&orphaned_holds->ours
 					, &orphaned_holds->his
 					, orphaned_holds->transport_proto
-#ifdef HAVE_LABELED_IPSEC
-                                        , NULL
-#endif
+                                        , NULL_POLICY
 					, "%hold found-pfkey");
 
     if (limit <= 0) {
@@ -685,7 +681,7 @@ finish_pfkey_msg(struct sadb_ext *extensions[K_SADB_EXT_MAX + 1]
 		    loglog(RC_LOG_SERIOUS, "requested algorithm is not available in the kernel");
 		    success=FALSE;
 		    /* fall through to get error message */
-		    
+
 		  default:
 		  logerr:
 		    openswan_log_errno_routine(e1, "pfkey write() of %s message %u"
@@ -791,7 +787,7 @@ pfkey_register_proto(unsigned int sadb_register
 }
 
 #ifdef KLIPS
-void 
+void
 klips_register_proto(unsigned satype, const char *satypename)
 {
     return pfkey_register_proto(K_SADB_REGISTER, satype, satypename);
@@ -877,9 +873,7 @@ pfkey_raw_eroute(const ip_address *this_host
 		 , time_t use_lifetime UNUSED
 		 , enum pluto_sadb_operations op
 		 , const char *text_said
-#ifdef HAVE_LABELED_IPSEC
 		 , char *policy_label UNUSED
-#endif
 		 )
 {
     struct sadb_ext *extensions[K_SADB_EXT_MAX + 1];
@@ -1040,14 +1034,14 @@ bool pfkey_add_sa(struct kernel_sa *sa, bool replace)
 	    if(!success) return FALSE;
     }
 #endif
-	
+
     if(sa->outif != -1) {
 	    success = pfkey_outif_build(&extensions[K_SADB_X_EXT_PLUMBIF],sa->outif);
 	    success = pfkey_build(success, "pfkey_outif_build", sa->text_said, extensions);
-	    
+
 	    if(!success) return FALSE;
     }
- 
+
     if(sa->enckeylen != 0) {
 	success = pfkey_build(pfkey_key_build(&extensions[K_SADB_EXT_KEY_ENCRYPT]
 					      , K_SADB_EXT_KEY_ENCRYPT
@@ -1057,7 +1051,7 @@ bool pfkey_add_sa(struct kernel_sa *sa, bool replace)
 			      , sa->text_said, extensions);
 	if(!success) return FALSE;
     }
-	
+
 
 #ifdef NAT_TRAVERSAL
     if(sa->natt_type != 0) {
@@ -1081,7 +1075,7 @@ bool pfkey_add_sa(struct kernel_sa *sa, bool replace)
 	      , DBG_log("setting natt_sport to %d\n", sa->natt_sport));
 	  if(!success) return FALSE;
 	}
-	
+
 	if(sa->natt_dport != 0) {
 	  success = pfkey_build(pfkey_x_nat_t_port_build(
 				  &extensions[K_SADB_X_EXT_NAT_T_DPORT]
@@ -1093,7 +1087,7 @@ bool pfkey_add_sa(struct kernel_sa *sa, bool replace)
 	      , DBG_log("setting natt_dport to %d\n", sa->natt_dport));
 	  if(!success) return FALSE;
 	}
-	
+
 
 	if(sa->natt_type!=0 && !isanyaddr(sa->natt_oa)) {
 	  success = pfkeyext_address(K_SADB_X_EXT_NAT_T_OA, sa->natt_oa
@@ -1107,7 +1101,7 @@ bool pfkey_add_sa(struct kernel_sa *sa, bool replace)
 #endif
 
     success = finish_pfkey_msg(extensions, "Add SA", sa->text_said, &pfb);
-    
+
     if(success) {
 	    /* extract the saref extension */
 	    struct sadb_ext *replies[K_SADB_EXT_MAX + 1];
@@ -1117,10 +1111,10 @@ bool pfkey_add_sa(struct kernel_sa *sa, bool replace)
 	    if (error) {
 		plog("success on unparsable message - cannot happen");
 	    }
-#ifdef KLIPS_MAST	    
+#ifdef KLIPS_MAST
 	    if(replies[K_SADB_X_EXT_SAREF]) {
 		    struct sadb_x_saref *sar = (struct sadb_x_saref *)replies[K_SADB_X_EXT_SAREF];
-		    
+
 		    sa->ref = sar->sadb_x_saref_me;
 		    sa->refhim = sar->sadb_x_saref_him;
 	    }
@@ -1221,7 +1215,7 @@ pfkey_close(void)
  */
 bool
 pfkey_shunt_eroute(struct connection *c
-		   , struct spd_route *sr
+		   , const struct spd_route *sr
 		   , enum routing_t rt_kind
 		   , enum pluto_sadb_operations op, const char *opname)
 {
@@ -1250,7 +1244,7 @@ pfkey_shunt_eroute(struct connection *c
         case ERO_DELETE:
             /* delete remains delete */
             break;
-	    
+
 	case ERO_ADD_INBOUND:
 	    break;
 
@@ -1276,7 +1270,7 @@ pfkey_shunt_eroute(struct connection *c
             opname = "replace eclipsed";
             eclipse_count--;
             break;
-	    
+
         case ERO_DELETE:
             /* delete unnecessary: we don't actually have an eroute */
             eclipse_count--;
@@ -1305,11 +1299,11 @@ pfkey_shunt_eroute(struct connection *c
 
 #if 0
     {
-	bool ok; 
+	bool ok;
 	enum pluto_sadb_operations inop;
-	
+
 	inop = op + ERO_ADD_INBOUND-ERO_ADD;
-	
+
 	ok = pfkey_raw_eroute(&c->spd.that.host_addr, &c->spd.that.client
 			      , &c->spd.this.host_addr, &c->spd.this.client
 			      , htonl(spi)
@@ -1327,11 +1321,11 @@ pfkey_shunt_eroute(struct connection *c
       const ip_address *peer = &sr->that.host_addr;
       char buf2[256];
       const struct af_info *fam = aftoinfo(addrtypeof(peer));
-      
+
       if(fam == NULL) {
 	      fam=aftoinfo(AF_INET);
       }
-      
+
       snprintf(buf2, sizeof(buf2)
 	       , "eroute_connection %s", opname);
 
@@ -1343,22 +1337,23 @@ pfkey_shunt_eroute(struct connection *c
 			      , sr->this.protocol
 			      , ET_INT
 			      , null_proto_info, 0, op, buf2
-#ifdef HAVE_LABELED_IPSEC
 			      , c->policy_label
-#endif
 			      );
     }
 }
 
+#define PROTO_COUNT 4
+
 /* install or remove eroute for SA Group */
 bool
-pfkey_sag_eroute(struct state *st, struct spd_route *sr
+pfkey_sag_eroute(struct state *st, const struct spd_route *sr
 		 , unsigned op, const char *opname)
 {
     unsigned int inner_proto;
     enum eroute_type inner_esatype;
     ipsec_spi_t inner_spi;
-    struct pfkey_proto_info proto_info[4];
+#define PROTO_INFO_COUNT 4
+    struct pfkey_proto_info proto_info[PROTO_INFO_COUNT];
     int i;
     bool tunnel;
 
@@ -1427,19 +1422,17 @@ pfkey_sag_eroute(struct state *st, struct spd_route *sr
         inner_esatype = ET_IPIP;
 
         proto_info[i].encapsulation = ENCAPSULATION_MODE_TUNNEL;
-        for (j = i + 1; proto_info[j].proto; j++)
+        for (j = i + 1; j < PROTO_INFO_COUNT && proto_info[j].proto; j++)
         {
             proto_info[j].encapsulation = ENCAPSULATION_MODE_TRANSPORT;
         }
     }
 
-    return eroute_connection(sr
+    return eroute_connection(NULL_STATE, sr
 			     , inner_spi, inner_proto
 			     , inner_esatype, proto_info + i
 			     , op, opname
-#ifdef HAVE_LABELED_IPSEC
-			     , NULL
-#endif
+			     , NULL_POLICY
 			     );
 }
 
@@ -1766,7 +1759,7 @@ bool pfkey_was_eroute_idle(struct state *st, time_t idle_max)
         int ret = TRUE;
 
         passert(st != NULL);
- 
+
         f = fopen(procname, "r");
         if(f == NULL) { /** Can't open the file, perhaps were are on 26sec? */
                 ret = TRUE;
@@ -1780,7 +1773,7 @@ bool pfkey_was_eroute_idle(struct state *st, time_t idle_max)
                 ipsec_spi_t spi = 0;
                 static const char idle[] = "idle=";
                 time_t idle_time; /* idle time we read from /proc */
-        
+
                 dst = st->st_connection->spd.this.host_addr; /* inbound SA */
                 if(st->st_ah.present) {
                         proto = SA_AH;
@@ -1788,27 +1781,27 @@ bool pfkey_was_eroute_idle(struct state *st, time_t idle_max)
                 }
                 if(st->st_esp.present) {
                         proto = SA_ESP;
-                        spi = st->st_esp.our_spi;  
+                        spi = st->st_esp.our_spi;
                 }
-        
+
                 if(proto == 0 && spi == 0) {
                         ret = TRUE;
 
                         break;
                 }
-                 
+
                 initsaid(&dst, spi, proto, &said);
                 satot(&said, 'x', text_said, SATOT_BUF);
-                        
+
                 line = fgets(buf, sizeof(buf), f);
                 if(line == NULL) { /* Reached end of list */
                         ret = TRUE;
                         break;
                 }
-                 
+
                 if(strncmp(line, text_said, strlen(text_said)) == 0) {
                         /* we found a match, now try to find idle= */
-                        char *p = strstr(line, idle);   
+                        char *p = strstr(line, idle);
                         if(p == NULL) { /* SAs which haven't been used yet
                                          don't have it */
                                 ret = TRUE; /* it didn't have traffic */
@@ -1841,7 +1834,7 @@ bool pfkey_was_eroute_idle(struct state *st, time_t idle_max)
                         }
 
                 }
-                                
+
            }
            fclose(f);
 	}
@@ -1855,7 +1848,7 @@ void pfkey_set_debug(int cur_debug
 #ifdef DEBUG
     pfkey_lib_debug = (cur_debug&DBG_PFKEY ?
 		       PF_KEY_DEBUG_PARSE_MAX : PF_KEY_DEBUG_PARSE_NONE);
-    
+
     pfkey_debug_func = debug_func;
     pfkey_error_func = error_func;
 #endif
@@ -1912,8 +1905,8 @@ pfkey_plumb_mast_device(int mast_dev)
 		return FALSE;
 	}
 
-	
-	
+
+
 	return TRUE;
 }
 #endif /* KLIPS_MAST */
