@@ -3,7 +3,7 @@
  * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  * Copyright (C) 2005-2007 Michael Richardson
  * Copyright (C) 2006-2010 Bart Trojanowski
- * Copyright (C) 2008-2010 Paul Wouters
+ * Copyright (C) 2008-2012 Paul Wouters
  * Copyright (C) 2008-2010 David McCullough.
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -40,23 +40,23 @@
 
 #include "defs.h"
 #include "log.h"
-#include "server.h"
-#include "state.h"
+#include "pluto/server.h"
 #include "id.h"
+#include "state.h"
 #include "x509.h"
 #include "pgp.h"
 #include "certs.h"
 #ifdef XAUTH_USEPAM
 #include <security/pam_appl.h>
 #endif
-#include "connections.h"	/* needs id.h */
+#include "pluto/connections.h"	/* needs id.h */
 #include "kernel.h"	/* needs connections.h */
 #include "whack.h"	/* needs connections.h */
 #include "timer.h"
 #include "kernel_alg.h"
 #include "ike_alg.h"
 #include "plutoalg.h"
-#include "virtual.h" /* for show_virtual_private */
+#include "pluto/virtual.h" /* for show_virtual_private */
 
 #ifndef NO_DB_OPS_STATS
 #define NO_DB_CONTEXT
@@ -71,7 +71,9 @@ bool
     log_to_stderr = TRUE,	/* should log go to stderr? */
     log_to_syslog = TRUE,	/* should log go to syslog? */
     log_to_perpeer= FALSE,	/* should log go to per-IP file? */
-    log_did_something=TRUE;     /* set if we wrote something recently */
+    log_did_something=TRUE,     /* set if we wrote something recently */
+    log_with_timestamp= FALSE; /* some people want timestamps, but we
+				   don't want those in our test output */
 
 
 bool
@@ -374,6 +376,19 @@ open_peerlog(struct connection *c)
     perpeer_count++;
 }
 
+char *oswtimestr(void)
+{
+    static char datebuf[32];
+    time_t n;
+    struct tm *t;
+
+    time(&n);
+    t = localtime(&n);
+
+    strftime(datebuf, sizeof(datebuf), "%Y-%m-%d %T", t);
+    return datebuf;
+}
+
 /* log a line to cur_connection's log */
 static void
 peerlog(const char *prefix, const char *m)
@@ -392,15 +407,7 @@ peerlog(const char *prefix, const char *m)
     /* despite our attempts above, we may not be able to open the file. */
     if (cur_connection->log_file != NULL)
     {
-	char datebuf[32];
-	time_t n;
-	struct tm *t;
-
-	time(&n);
-	t = localtime(&n);
-
-	strftime(datebuf, sizeof(datebuf), "%Y-%m-%d %T", t);
-	fprintf(cur_connection->log_file, "%s %s%s\n", datebuf, prefix, m);
+	fprintf(cur_connection->log_file, "%s %s%s\n", oswtimestr(), prefix, m);
 
 	/* now move it to the front of the list */
 	CIRCLEQ_REMOVE(&perpeer_list, cur_connection, log_link);
@@ -421,8 +428,19 @@ openswan_log(const char *message, ...)
 
     log_did_something=TRUE;
 
-    if (log_to_stderr)
-	fprintf(stderr, "%s\n", m);
+    if (log_to_stderr) {
+	if (log_with_timestamp) {
+		struct tm *timeinfo;
+		char fmt[32];
+		time_t rtime;
+		time(&rtime);
+		timeinfo = localtime (&rtime);
+		strftime (fmt,sizeof(fmt),"%b %e %T",timeinfo);
+		fprintf(stderr, "%s: %s\n", fmt, m);
+	} else {
+		fprintf(stderr, "%s\n", m);
+	}
+    }
     if (log_to_syslog)
 	syslog(LOG_WARNING, "%s", m);
     if (log_to_perpeer)
@@ -445,8 +463,19 @@ loglog(int mess_no, const char *message, ...)
 
     log_did_something=TRUE;
 
-    if (log_to_stderr)
-	fprintf(stderr, "%s\n", m);
+    if (log_to_stderr) {
+	if (log_with_timestamp) {
+		struct tm *timeinfo;
+		char fmt[32];
+		time_t rtime;
+		time(&rtime);
+		timeinfo = localtime (&rtime);
+		strftime (fmt,sizeof(fmt),"%b %e %T",timeinfo);
+		fprintf(stderr, "%s: %s\n", fmt, m);
+	} else {
+		fprintf(stderr, "%s\n", m);
+	}
+    }
     if (log_to_syslog)
 	syslog(LOG_WARNING, "%s", m);
     if (log_to_perpeer)
@@ -701,8 +730,19 @@ DBG_log(const char *message, ...)
     /* then sanitize anything else that is left. */
     (void)sanitize_string(m, sizeof(m));
 
-    if (log_to_stderr)
-	fprintf(stderr, "%c %s\n", debug_prefix, m);
+    if (log_to_stderr) {
+	if (log_with_timestamp) {
+		struct tm *timeinfo;
+		char fmt[32];
+		time_t rtime;
+		time(&rtime);
+		timeinfo = localtime (&rtime);
+		strftime (fmt,sizeof(fmt),"%b %e %T",timeinfo);
+		fprintf(stderr, "%c %s: %s\n", debug_prefix, fmt, m);
+	} else {
+		fprintf(stderr, "%c %s\n", debug_prefix, m);
+	}
+    }
     if (log_to_syslog)
 	syslog(LOG_DEBUG, "%c %s", debug_prefix, m);
     if (log_to_perpeer) {
@@ -806,7 +846,7 @@ show_status(void)
     db_ops_show_status();
     whack_log(RC_COMMENT, BLANK_FORMAT);	/* spacer */
 #endif
-    show_connections_status();
+    show_connections_status(whack_log);
     whack_log(RC_COMMENT, BLANK_FORMAT);	/* spacer */
     show_states_status();
 #ifdef KLIPS
@@ -909,7 +949,7 @@ connection_state(struct state *st, void *data)
 		return;
 
 	if (st->st_connection != lc->conn) {
-		if (lc->conn->host_pair != st->st_connection->host_pair ||
+		if (lc->conn->IPhost_pair != st->st_connection->IPhost_pair ||
 			!same_peer_ids(lc->conn, st->st_connection, NULL))
 		    return;
 		/* phase1 is shared with another connnection */
