@@ -40,7 +40,7 @@
 #ifdef XAUTH_USEPAM
 #include <security/pam_appl.h>
 #endif
-#include "connections.h"	/* needs id.h */
+#include "pluto/connections.h"	/* needs id.h */
 #include "pending.h"
 #include "log.h"
 #include "state.h"
@@ -62,9 +62,7 @@ struct pending {
     unsigned long try;
     so_serial_t   replacing;
     time_t        pend_time;
-#ifdef HAVE_LABELED_IPSEC
     struct xfrm_user_sec_ctx_ike * uctx;
-#endif
 
     struct pending *next;
 };
@@ -77,9 +75,7 @@ add_pending(int whack_sock
 , lset_t policy
 , unsigned long try
 , so_serial_t replacing
-#ifdef HAVE_LABELED_IPSEC
-, struct xfrm_user_sec_ctx_ike * uctx
-#endif
+, struct xfrm_user_sec_ctx_ike * uctx UNUSED
 )
 {
     struct pending *p, **pp;
@@ -215,9 +211,7 @@ unpend(struct state *st)
 	    p->pend_time = time(NULL);
 	    (void) quick_outI1(p->whack_sock, st, p->connection, p->policy
 			       , p->try, p->replacing
-#ifdef HAVE_LABELED_IPSEC
 			       , p->uctx
-#endif
 			      );
 	    p->whack_sock = NULL_FD;	/* ownership transferred */
 	    p->connection = NULL;	/* ownership transferred */
@@ -274,7 +268,7 @@ bool pending_check_timeout(struct connection *c)
 		    , (unsigned long)p->pend_time
 		    , (unsigned long)c->dpd_timeout
 		    , (unsigned long)n));
-		    
+
 	if(c->dpd_timeout > 0) {
 	    if((p->pend_time + c->dpd_timeout*3) <= n) {
 		DBG(DBG_DPD, DBG_log("connection \"%s\" stuck, restarting", c->name));
@@ -300,7 +294,7 @@ update_pending(struct state *os, struct state *ns)
 	 p = p->next) {
 	if (p->isakmp_sa == os)
 	    p->isakmp_sa = ns;
-    }	    
+    }
 }
 
 /* a Main Mode negotiation has failed; discard any pending */
@@ -309,7 +303,7 @@ flush_pending_by_state(struct state *st)
 {
     struct pending **pp
 	, *p;
-    
+
     pp = host_pair_first_pending(st->st_connection);
     if(pp == NULL) return;
 
@@ -321,7 +315,7 @@ flush_pending_by_state(struct state *st)
 	     */
 	    delete_pending(pp);
 	}
-	else 
+	else
 	    pp = &p->next;
     }
 }
@@ -332,7 +326,7 @@ flush_pending_by_connection(struct connection *c)
 {
     struct pending **pp
 	, *p;
-    
+
     pp = host_pair_first_pending(c);
     if(pp == NULL) return;
 
@@ -354,7 +348,7 @@ show_pending_phase2(const struct connection *c, const struct state *st)
 {
     struct pending **pp
 	, *p;
-    
+
     pp = host_pair_first_pending(c);
     if(pp == NULL) return;
 
@@ -379,7 +373,7 @@ bool in_pending_use(struct connection *c)
 {
     /* see if it is being used by a pending */
     struct pending **pp, *p;
-    
+
     pp = host_pair_first_pending(c);
     if(pp == NULL) return FALSE;
 
