@@ -1,4 +1,4 @@
-/* 
+/*
  * Cryptographic helper process.
  * Copyright (C) 2004-2007 Michael C. Richardson <mcr@xelerance.com>
  * Copyright (C) 2008 David McCullough <david_mccullough@securecomputing.com>
@@ -48,7 +48,7 @@ struct pcr_kenonce {
 
   /* inputs */
   u_int16_t oakley_group;
-  
+
   /* outputs */
   wire_chunk_t secret;
   wire_chunk_t gi;
@@ -65,9 +65,9 @@ struct pcr_skeyid_q {
 
   /* inputs */
   u_int16_t     oakley_group;
-  oakley_auth_t auth;	            
+  oakley_auth_t auth;
   oakley_hash_t integ_hash;
-  oakley_hash_t prf_hash;               
+  oakley_hash_t prf_hash;
   enum phase1_role init;
   size_t        keysize;     /* of encryptor */
   wire_chunk_t gi;
@@ -80,7 +80,7 @@ struct pcr_skeyid_q {
   wire_chunk_t secret;
 #ifdef HAVE_LIBNSS
   /* u_int16_t encrypt_algo; */
-  struct encrypt_desc *encrypter;
+  const struct encrypt_desc *encrypter;
   wire_chunk_t   pubk;
 #endif
 };
@@ -95,7 +95,7 @@ struct pcr_skeyid_r {
   wire_chunk_t skeyid_d;        /* output */
   wire_chunk_t skeyid_a;        /* output */
   wire_chunk_t skeyid_e;        /* output */
-  wire_chunk_t new_iv;          
+  wire_chunk_t new_iv;
   wire_chunk_t enc_key;
 };
 
@@ -134,6 +134,7 @@ struct pluto_crypto_req {
   pcr_req_id                 pcr_id;
   enum crypto_importance     pcr_pcim;
   int                        pcr_slot;
+  int                        pcr_success;
   union {
       struct pcr_kenonce      kn;
       struct pcr_skeyid_q     dhq;
@@ -164,7 +165,7 @@ struct pluto_crypto_req_cont {
 #endif
 };
 
-  
+
 
 #define PCR_REQ_SIZE sizeof(struct pluto_crypto_req)+10
 
@@ -204,7 +205,7 @@ extern void calc_nonce(struct pluto_crypto_req *r);
 extern void compute_dh_shared(struct state *st, const chunk_t g
 			      , const struct oakley_group_desc *group);
 
-/* no longer exists? 
+/* no longer exists?
 extern stf_status perform_dh(struct pluto_crypto_req_cont *cn, struct state *st);
 */
 
@@ -222,7 +223,7 @@ extern void finish_dh_secretiv(struct state *st,
 extern stf_status start_dh_secret(struct pluto_crypto_req_cont *cn
 				  , struct state *st
 				  , enum crypto_importance importance
-				  , enum phase1_role init      
+				  , enum phase1_role init
 				  , u_int16_t oakley_group_p);
 
 extern void finish_dh_secret(struct state *st,
@@ -241,6 +242,7 @@ extern void calc_dh_iv(struct pluto_crypto_req *r);
 extern void calc_dh(struct pluto_crypto_req *r);
 extern void calc_dh_v2(struct pluto_crypto_req *r);
 
+extern int unpack_v2KE(struct state *st, struct pluto_crypto_req *r, chunk_t *g);
 extern void unpack_KE(struct state *st
 		      , struct pluto_crypto_req *r
 		      , chunk_t *g);
@@ -309,4 +311,4 @@ static inline void pcr_init(struct pluto_crypto_req *r
  * c-style: pluto
  * End:
  */
- 
+
