@@ -3,6 +3,7 @@
  * Originally by: JuanJo Ciarlante <jjo-ipsec@mendoza.gov.ar>
  * Reworked into openswan 2.x by Michael Richardson <mcr@xelerance.com>
  * (C)opyright 2012 Paul Wouters <pwouters@redhat.com>
+ * (C)opyright 2012 Paul Wouters <paul@libreswan.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -28,7 +29,7 @@
 #include "oswalloc.h"
 #include "defs.h"
 #include "id.h"
-#include "connections.h"
+#include "pluto/connections.h"
 #include "state.h"
 #include "kernel_alg.h"
 #include "alg_info.h"
@@ -186,8 +187,8 @@ in_loop:
  * print which ESP algorithm has actually been selected, based upon which
  * ones are actually loaded.
  */
-int
-alg_info_snprint_esp(char *buf, int buflen, struct alg_info_esp *alg_info)
+static void
+alg_info_snprint_esp(char *buf, size_t buflen, struct alg_info_esp *alg_info)
 {
 	char *ptr=buf;
 	int ret;
@@ -196,9 +197,11 @@ alg_info_snprint_esp(char *buf, int buflen, struct alg_info_esp *alg_info)
 	int eklen, aklen;
 	const char *sep="";
 
-	ptr=buf;
+	passert(buflen >= sizeof("none"));
 
-	buf[0]=0; strncat(buf, "none", buflen);
+	ptr=buf;
+	buf[0]=0;
+	strncat(buf, "none", buflen - 1);
 
 	ALG_INFO_ESP_FOREACH(alg_info, esp_info, cnt) {
 	    if (kernel_alg_esp_enc_ok(esp_info->esp_ealg_id, 0, NULL)) {
@@ -224,21 +227,23 @@ alg_info_snprint_esp(char *buf, int buflen, struct alg_info_esp *alg_info)
 			 , esp_info->esp_ealg_id, eklen
 			 , enum_name(&auth_alg_names, esp_info->esp_aalg_id) + (esp_info->esp_aalg_id ? sizeof("AUTH_ALGORITHM_HMAC") : sizeof("AUTH_ALGORITHM"))
 			 , esp_info->esp_aalg_id, aklen);
+
+	    if ( ret < 0 || (size_t)ret >= buflen) {
+		DBG_log("alg_info_snprint_esp: buffer too short for snprintf");
+		break;
+	    }
 	    ptr+=ret;
 	    buflen-=ret;
-	    if (buflen<0) break;
-
 	    sep = ", ";
 	}
-	return ptr-buf;
 }
 
 /*
  * print which AH algorithm has actually been selected, based upon which
  * ones are actually loaded.
  */
-int
-alg_info_snprint_ah(char *buf, int buflen, struct alg_info_esp *alg_info)
+static void
+alg_info_snprint_ah(char *buf, size_t buflen, struct alg_info_esp *alg_info)
 {
 	char *ptr=buf;
 	int ret;
@@ -247,9 +252,11 @@ alg_info_snprint_ah(char *buf, int buflen, struct alg_info_esp *alg_info)
 	int aklen;
 	const char *sep="";
 
+	passert(buflen >= sizeof("none"));
 	ptr=buf;
 
-	buf[0]=0; strncat(buf, "none", buflen);
+	buf[0]=0;
+	strncat(buf, "none", buflen - 1);
 
 	ALG_INFO_ESP_FOREACH(alg_info, esp_info, cnt) {
 
@@ -266,23 +273,27 @@ alg_info_snprint_ah(char *buf, int buflen, struct alg_info_esp *alg_info)
 			 , sep
 			 , enum_name(&auth_alg_names, esp_info->esp_aalg_id)+sizeof("AUTH_ALGORITHM_HMAC")
 			 , esp_info->esp_aalg_id, aklen);
+
+	    if ( ret < 0 || (size_t)ret >= buflen) {
+		DBG_log("alg_info_snprint_ah: buffer too short for snprintf");
+		break;
+	    }
 	    ptr+=ret;
 	    buflen-=ret;
-	    if (buflen<0) break;
-
 	    sep = ", ";
 	}
-	return ptr-buf;
 }
 
-int
-alg_info_snprint_phase2(char *buf, int buflen, struct alg_info_esp *alg_info)
+void
+alg_info_snprint_phase2(char *buf, size_t buflen, struct alg_info_esp *alg_info)
 {
     switch(alg_info->alg_info_protoid) {
     case PROTO_IPSEC_ESP:
-	return alg_info_snprint_esp(buf, buflen, alg_info);
+	alg_info_snprint_esp(buf, buflen, alg_info);
+	return;
     case PROTO_IPSEC_AH:
-	return alg_info_snprint_ah(buf, buflen, alg_info);
+	alg_info_snprint_ah(buf, buflen, alg_info);
+	return;
     default:
 	bad_case(alg_info->alg_info_protoid);
     }
@@ -304,8 +315,8 @@ char *alg_info_snprint_ike1(struct ike_info *ike_info
     return buf;
 }
 
-int
-alg_info_snprint_ike(char *buf, int buflen, struct alg_info_ike *alg_info)
+void
+alg_info_snprint_ike(char *buf, size_t buflen, struct alg_info_ike *alg_info)
 {
 	char *ptr=buf;
 	int ret;
@@ -316,6 +327,7 @@ alg_info_snprint_ike(char *buf, int buflen, struct alg_info_ike *alg_info)
 	struct encrypt_desc *enc_desc;
 	struct hash_desc *hash_desc;
 
+
 	ALG_INFO_IKE_FOREACH(alg_info, ike_info, cnt) {
 	    if (ike_alg_enc_present(ike_info->ike_ealg)
 		&& (ike_alg_hash_present(ike_info->ike_halg))
@@ -340,14 +352,14 @@ alg_info_snprint_ike(char *buf, int buflen, struct alg_info_ike *alg_info)
 			     , ike_info->ike_halg, aklen
 			     , enum_name(&oakley_group_names, ike_info->ike_modp)+sizeof("OAKLEY_GROUP")
 			     , ike_info->ike_modp);
+		if ( ret < 0 || (size_t)ret >= buflen) {
+		   DBG_log("alg_info_snprint_ike: buffer too short for snprintf");
+		   break;
+		}
 		ptr+=ret;
 		buflen-=ret;
-		if (buflen<0) break;
-
-		sep = ", ";
 	    }
 	}
-	return ptr-buf;
 }
 
 /*
@@ -463,6 +475,7 @@ kernel_alg_db_add(struct db_context *db_ctx
 	    /*	open new transformation */
 	    db_trans_add(db_ctx, ealg_i);
 
+            /* XXX-MCR todo: needs to handle IKEV2 now as well  #3949  */
 	    /* add ESP auth attr (if present) */
 	    if (esp_info->esp_aalg_id != AUTH_ALGORITHM_NONE) {
 		db_attr_add_values(db_ctx,
