@@ -31,7 +31,7 @@
 #include "constants.h"
 #include "defs.h"
 #include "id.h"
-#include "connections.h"	/* needs id.h */
+#include "pluto/connections.h"	/* needs id.h */
 #include "foodgroups.h"
 #include "whack.h"	/* needs connections.h */
 #include "packet.h"
@@ -44,7 +44,7 @@
 #include "keys.h"
 #include "adns.h"	/* needs <resolv.h> */
 #include "dnskey.h"	/* needs keys.h and adns.h */
-#include "server.h"
+#include "pluto/server.h"
 
 #include "openswan/ipsec_policy.h"
 #include "rcv_info.h"
