@@ -1,4 +1,3 @@
-
 /* whack communicating routines
  * Copyright (C) 1997 Angelos D. Keromytis.
  * Copyright (C) 1998-2001  D. Hugh Redelmeier.
@@ -49,7 +48,7 @@
 #ifdef XAUTH_USEPAM
 #include <security/pam_appl.h>
 #endif
-#include "connections.h"	/* needs id.h */
+#include "pluto/connections.h"	/* needs id.h */
 #include "foodgroups.h"
 #include "whack.h"	/* needs connections.h */
 #include "packet.h"
@@ -58,15 +57,17 @@
 #include "ipsec_doi.h"	/* needs demux.h and state.h */
 #include "kernel.h"	/* needs connections.h */
 #include "rcv_whack.h"
+#include "pluto/whackfile.h"
 #include "log.h"
 #include "keys.h"
 #include "secrets.h"
 #include "adns.h"	/* needs <resolv.h> */
 #include "dnskey.h"	/* needs keys.h and adns.h */
-#include "server.h"
+#include "pluto/server.h"
 #include "fetch.h"
 #include "ocsp.h"
 #include "timer.h"
+#include "hostpair.h"
 
 #include "kernel_alg.h"
 #include "ike_alg.h"
@@ -129,89 +130,6 @@ key_add_merge(struct key_add_common *oc, const struct id *keyid)
     }
 }
 
-static char whackrecordname[PATH_MAX];
-static FILE *whackrecordfile=NULL;
-
-/*
- * writes out 64-bit time, even though we actually
- * only have 32-bit time here. Assumes that time will
- * be written out in big-endian format, with MSB word
- * being first.
- *
- */
-static bool writewhackrecord(char *buf, int buflen)
-{
-    u_int32_t header[3];
-    time_t n;
-
-    /* round up buffer length */
-    int abuflen = (buflen + 3) & ~0x3;
-
-    /* bail if we aren't writing anything */
-    if(whackrecordfile == NULL) return TRUE;
-
-    header[0]=buflen + sizeof(u_int32_t)*3;
-    header[1]=0;
-    time(&n);
-    header[2]=n;
-
-    /* DBG_log("buflen: %u abuflen: %u\n", header[0], abuflen); */
-    
-    if(fwrite(header, sizeof(u_int32_t)*3, 1, whackrecordfile) < 1) {
-	DBG_log("writewhackrecord: fwrite error when writing header");
-    }
-
-    if(fwrite(buf, abuflen, 1, whackrecordfile) < 1) {
-	DBG_log("writewhackrecord: fwrite error when writing buf");
-    }
-    
-    return TRUE;
-}
-
-
-/*
- * we write out an empty record with the right WHACK magic.
- * this should permit a later mechanism to figure out the
- * endianess of the file, since we will get records from
- * other systems for analysis eventually.
- */
-static bool openwhackrecordfile(char *file)
-{
-    char when[256];
-    char FQDN[HOST_NAME_MAX + 1];
-    u_int32_t magic;
-    struct tm tm1, *tm;
-    time_t n;
-    
-    strcpy(FQDN, "unknown host");
-    gethostname(FQDN, sizeof(FQDN));
-
-    strncpy(whackrecordname, file, sizeof(whackrecordname));
-    whackrecordfile = fopen(whackrecordname, "w");
-    if(whackrecordfile==NULL) {
-	openswan_log("Failed to open whack record file: '%s'\n"
-		     , whackrecordname);
-	return FALSE;
-    }
-
-    time(&n);
-    tm = localtime_r(&n, &tm1);
-    strftime(when, sizeof(when), "%F %T", tm);
-    
-    fprintf(whackrecordfile, "#!-pluto-whack-file- recorded on %s on %s\n",
-	    FQDN, when);
-
-    magic = WHACK_BASIC_MAGIC;
-    writewhackrecord((char *)&magic, 4);
-    
-    DBG(DBG_CONTROL
-	, DBG_log("started recording whack messages to %s\n"
-		  , whackrecordname));
-    return TRUE;
-}
-    
-
-
 static void
 key_add_continue(struct adns_continuation *ac, err_t ugh)
 {
@@ -320,6 +238,25 @@ key_add_request(const struct whack_message *msg)
     }
 }
 
+/*
+ * handle a whack --listen: may also be called on SIGHUP eventually,
+ *               or when routing socket is added.
+ */
+void whack_listen(void) {
+    fflush(stderr);
+    fflush(stdout);
+    close_peerlog();    /* close any open per-peer logs */
+    openswan_log("listening for IKE messages");
+    listening = TRUE;
+    daily_log_reset();
+    reset_adns_restart_count();
+    set_myFQDN();
+    find_ifaces();
+    load_preshared_secrets(NULL_FD);
+    load_groups();
+    check_orientations();
+}
+
 /*
  * handle a whack message.
  */
@@ -347,7 +284,7 @@ void whack_process(int whackfd, struct whack_message msg)
 	    else if (!msg.whack_connection)
 	    {
 		struct connection *c = con_by_name(msg.name, TRUE);
-		
+
 		if (c != NULL)
 		{
 		    c->extra_debugging = msg.debugging;
@@ -363,30 +300,17 @@ void whack_process(int whackfd, struct whack_message msg)
 	case WHACK_SETDUMPDIR:
 	    /* XXX */
 	    break;
-	    
+
 	case WHACK_STARTWHACKRECORD:
 	    /* close old filename */
-	    if(whackrecordfile) {
-		DBG(DBG_CONTROL
-		    , DBG_log("stopped recording whack messages to %s\n"
-			      , whackrecordname));
-		fclose(whackrecordfile);
-	    }
-	    whackrecordfile=NULL;
+            close_whackrecordfile();
 
 	    openwhackrecordfile(msg.string1);
-
 	    /* do not do any other processing for these */
 	    goto done;
-	    
+
 	case WHACK_STOPWHACKRECORD:
-	    if(whackrecordfile) {
-		DBG(DBG_CONTROL
-		    , DBG_log("stopped recording whack messages to %s\n"
-			      , whackrecordname));
-		fclose(whackrecordfile);
-	    }
-	    whackrecordfile=NULL;
+            close_whackrecordfile();
 	    /* do not do any other processing for these */
 	    goto done;
 	}
@@ -409,7 +333,7 @@ void whack_process(int whackfd, struct whack_message msg)
 	if (st == NULL)
 	{
 	    loglog(RC_UNKNOWN_NAME, "no state #%lu to delete"
-		, msg.whack_deletestateno);
+                   , (long unsigned int)msg.whack_deletestateno);
 	}
 	else
 	{
@@ -426,17 +350,7 @@ void whack_process(int whackfd, struct whack_message msg)
     /* process "listen" before any operation that could require it */
     if (msg.whack_listen)
     {
-	fflush(stderr);
-	fflush(stdout);
-	close_peerlog();    /* close any open per-peer logs */
-	openswan_log("listening for IKE messages");
-	listening = TRUE;
-	daily_log_reset();
-	reset_adns_restart_count();
-	set_myFQDN();
-	find_ifaces();
-	load_preshared_secrets(NULL_FD);
-	load_groups();
+        whack_listen();
     }
     if (msg.whack_unlisten)
     {
@@ -484,14 +398,14 @@ void whack_process(int whackfd, struct whack_message msg)
 	load_crls();
     }
 
-    if (msg.tpmeval) 
+    if (msg.tpmeval)
     {
 #ifdef TPM
 	passert(msg.tpmeval != NULL);
 	tpm_eval(msg.tpmeval);
 #else
 	openswan_log("Pluto not built with TAPROOM");
-#endif	
+#endif
     }
 
 #ifdef HAVE_OCSP
@@ -555,6 +469,11 @@ void whack_process(int whackfd, struct whack_message msg)
     }
 #endif
 
+    if (msg.whack_list & LIST_HOSTPAIRS)
+    {
+	hostpair_list();
+    }
+
     if (msg.whack_list & LIST_EVENTS)
     {
 	timer_list();
@@ -634,9 +553,7 @@ void whack_process(int whackfd, struct whack_message msg)
 	    (void)initiate_ondemand(&msg.oppo_my_client, &msg.oppo_peer_client, 0
 		, FALSE
 		, msg.whack_async? NULL_FD : dup_any(whackfd)
-#ifdef HAVE_LABELED_IPSEC 
-		, NULL 
-#endif
+		, NULL
 		, "whack");
     }
 
@@ -658,7 +575,7 @@ done:
 }
 
 /*
- * Handle a whack request. 
+ * Handle a whack request.
  */
 void
 whack_handle(int whackctlfd)
@@ -701,7 +618,7 @@ whack_handle(int whackctlfd)
     {
 	err_t ugh = NULL;
         struct whackpacker wp;
-        
+
         wp.msg = &msg;
         wp.n   = n;
         wp.str_next = msg.string;
@@ -728,8 +645,8 @@ whack_handle(int whackctlfd)
 	    }
 	    else
 	    {
-		ugh = builddiag("ignoring message from whack with bad magic %d; should be %d; Mismatched versions of userland tools and KLIPS code."
-		    , msg.magic, WHACK_MAGIC);
+		ugh = builddiag("ignoring message from whack with bad magic %08x; should be %08x; Mismatched versions of userland tools and PLUTO code."
+                                , msg.magic, WHACK_MAGIC);
 	    }
 	}
         else if ((ugh = unpack_whack_msg(&wp)) != NULL)
@@ -743,7 +660,7 @@ whack_handle(int whackctlfd)
 
 	if (ugh != NULL)
 	{
-	    if (*ugh != '\0') 
+	    if (*ugh != '\0')
 		loglog(RC_BADWHACKMESSAGE, "%s", ugh);
 	    whack_log_fd = NULL_FD;
 	    close(whackfd);
@@ -753,7 +670,7 @@ whack_handle(int whackctlfd)
 
     /* dump record if necessary */
     writewhackrecord((char *)&msg_saved, n);
-    
+
     whack_process(whackfd, msg);
 }
 
@@ -778,19 +695,19 @@ bool whack_prompt_for(int whackfd
 	      , prompt1, prompt2);
 
     whack_log_fd = savewfd;
-    
+
     n = read(whackfd, ansbuf, ansbuf_len);
-    
+
     if(n == -1) {
 	whack_log(RC_LOG_SERIOUS, "read(whackfd) failed: %s", strerror(errno));
 	return FALSE;
     }
-    
+
     if(strlen(ansbuf) == 0) {
 	whack_log(RC_LOG_SERIOUS, "no %s entered, aborted", prompt2);
 	return FALSE;
     }
-	
+
     return TRUE;
 }
 
