@@ -58,18 +58,18 @@
 #include "socketwrapper.h"
 #include "constants.h"
 #include "defs.h"
-#include "state.h"
 #include "id.h"
+#include "state.h"
 #include "x509.h"
 #include "pgp.h"
 #include "certs.h"
 #ifdef XAUTH_USEPAM
 #include <security/pam_appl.h>
 #endif
-#include "connections.h"	/* needs id.h */
+#include "pluto/connections.h"	/* needs id.h */
 #include "kernel.h"  /* for no_klips; needs connections.h */
 #include "log.h"
-#include "server.h"
+#include "pluto/server.h"
 #include "timer.h"
 #include "packet.h"
 #include "demux.h"  /* needs packet.h */
@@ -440,7 +440,7 @@ create_socket(struct raw_iface *ifp, const char *v_name, int port)
 	close(fd);
 	return -1;
     }
-    setportof(htons(pluto_port), &ifp->addr);
+    setportof(htons(pluto_port500), &ifp->addr);
 
 #if defined(HAVE_UDPFROMTO)
     /* we are going to use udpfromto.c, so initialize it */
@@ -483,10 +483,12 @@ show_ifaces_status(void)
     struct iface_port *p;
 
     for (p = interfaces; p != NULL; p = p->next)
-	whack_log(RC_COMMENT, "interface %s/%s %s"
-	    , p->ip_dev->id_vname, p->ip_dev->id_rname, ip_str(&p->ip_addr));
+	whack_log(RC_COMMENT, "interface %s/%s %s (%s)"
+                  , p->ip_dev->id_vname, p->ip_dev->id_rname
+                  , p->addrname, p->socktypename);
 }
 
+
 void
 show_debug_status(void)
 {
