@@ -41,7 +41,7 @@
 #ifdef XAUTH_USEPAM
 #include <security/pam_appl.h>
 #endif
-#include "connections.h"	/* needs id.h */
+#include "pluto/connections.h"	/* needs id.h */
 #include "state.h"
 #include "kernel.h"	/* needs connections.h */
 #include "log.h"
@@ -66,12 +66,6 @@
 # include <keyhi.h>
 #endif
 
-/*
- * Global variables: had to go somewhere, might as well be this file.
- */
-
-u_int16_t pluto_port = IKE_UDP_PORT;	/* Pluto's port */
-
 /*
  * This file has the functions that handle the
  * state hash table and the Message ID list.
@@ -156,12 +150,14 @@ generate_msgid(struct state *isakmp_sa)
 
 /* state table functions */
 
+#ifndef STATE_TABLE_SIZE
 #define STATE_TABLE_SIZE 32
+#endif
 
 static struct state *statetable[STATE_TABLE_SIZE];
 
 static struct state **
-state_hash(const u_char *icookie, const u_char *rcookie)
+state_hash(const u_char *icookie, const u_char *rcookie, unsigned *state_bucket)
 {
     u_int i = 0, j;
 
@@ -177,6 +173,9 @@ state_hash(const u_char *icookie, const u_char *rcookie)
     i = i % STATE_TABLE_SIZE;
 
     DBG(DBG_CONTROL, DBG_log("state hash entry %d", i));
+    if(state_bucket) {
+        *state_bucket = i;
+    }
 
     return &statetable[i];
 }
@@ -247,13 +246,14 @@ state_with_serialno(so_serial_t sn)
 void
 insert_state(struct state *st)
 {
-    struct state **p = state_hash(st->st_icookie, st->st_rcookie);
+    unsigned int bucket;
+    struct state **p = state_hash(st->st_icookie, st->st_rcookie, &bucket);
 
     passert(st->st_hashchain_prev == NULL && st->st_hashchain_next == NULL);
 
     DBG(DBG_CONTROL
-	, DBG_log("inserting state object #%lu"
-		  , st->st_serialno));
+	, DBG_log("inserting state object #%lu bucket: %u"
+		  , st->st_serialno, bucket));
 
     if (*p != NULL)
     {
@@ -271,7 +271,7 @@ insert_state(struct state *st)
     if (st->st_event == NULL)
 	event_schedule(EVENT_SO_DISCARD, 0, st);
 
-	refresh_state(st);
+    refresh_state(st);
 }
 
 /*
@@ -281,14 +281,15 @@ insert_state(struct state *st)
 void
 rehash_state(struct state *st)
 {
+    unsigned bucket = 0;
     /* unlink from forward chain */
     struct state **p = st->st_hashchain_prev == NULL
-	? state_hash(st->st_icookie, zero_cookie)
+	? state_hash(st->st_icookie, zero_cookie, &bucket)
 	: &st->st_hashchain_prev->st_hashchain_next;
 
     DBG(DBG_CONTROL
-	, DBG_log("rehashing state object #%lu"
-			     , st->st_serialno));
+	, DBG_log("rehashing state object #%lu from bucket %u"
+                  , st->st_serialno, bucket));
 
     /* unlink from forward chain */
     passert(*p == st);
@@ -316,9 +317,9 @@ unhash_state(struct state *st)
     struct state **p;
 
     if(st->st_hashchain_prev == NULL) {
-	p = state_hash(st->st_icookie, st->st_rcookie);
+	p = state_hash(st->st_icookie, st->st_rcookie, NULL);
 	if(*p != st) {
-	    p = state_hash(st->st_icookie, zero_cookie);
+	    p = state_hash(st->st_icookie, zero_cookie, NULL);
 	}
     } else {
 	p = &st->st_hashchain_prev->st_hashchain_next;
@@ -347,17 +348,93 @@ release_whack(struct state *st)
     close_any(st->st_whack_sock);
 }
 
-/* freeing allocated traffic selectors */
-static void delete_ts(struct traffic_selector *ts) {
-	struct traffic_selector *tmp;
+/* here we are just freeing RAM */
+void free_state(struct state *st)
+{
+    delete_event(st);	/* delete any pending timer event */
 
-	/*first one is not malloced so skiping that*/
-	ts = ts-> next;
-	while(ts!=NULL) {
-		tmp = ts->next;
-		pfreeany(ts);
-		ts = tmp;
+    {
+	struct msgid_list *p = st->st_used_msgids;
+
+	while (p != NULL)
+	{
+	    struct msgid_list *q = p;
+	    p = p->next;
+	    pfree(q);
 	}
+    }
+
+    unreference_key(&st->st_peer_pubkey);
+
+    free_sa(st->st_sadb);
+    st->st_sadb=NULL;
+
+    if (st->st_sec_in_use) {
+#ifdef HAVE_LIBNSS
+	SECKEYPrivateKey *privk;
+	SECKEYPublicKey   *pubk;
+	memcpy(&pubk,st->pubk.ptr,st->pubk.len);
+	SECKEY_DestroyPublicKey(pubk);
+	freeanychunk(st->pubk);
+	memcpy(&privk,st->st_sec_chunk.ptr,st->st_sec_chunk.len);
+	SECKEY_DestroyPrivateKey(privk);
+#else
+	mpz_clear(&(st->st_sec));
+#endif
+	pfreeany(st->st_sec_chunk.ptr);
+    }
+
+    freeanychunk(st->st_firstpacket_me);
+    freeanychunk(st->st_firstpacket_him);
+    freeanychunk(st->st_tpacket);
+    freeanychunk(st->st_rpacket);
+    freeanychunk(st->st_p1isa);
+    freeanychunk(st->st_gi);
+    freeanychunk(st->st_gr);
+    freeanychunk(st->st_shared);
+    freeanychunk(st->st_ni);
+    freeanychunk(st->st_nr);
+#ifdef HAVE_LIBNSS
+    free_osw_nss_symkey(st->st_skeyid);
+    free_osw_nss_symkey(st->st_skey_d);
+    free_osw_nss_symkey(st->st_skey_ai);
+    free_osw_nss_symkey(st->st_skey_ar);
+    free_osw_nss_symkey(st->st_skey_ei);
+    free_osw_nss_symkey(st->st_skey_er);
+    free_osw_nss_symkey(st->st_skey_pi);
+    free_osw_nss_symkey(st->st_skey_pr);
+    free_osw_nss_symkey(st->st_enc_key);
+
+    if(st->st_ah.our_keymat!=NULL)
+    memset(st->st_ah.our_keymat, 0, st->st_ah.keymat_len);
+
+    if(st->st_ah.peer_keymat!=NULL)
+    memset(st->st_ah.peer_keymat, 0, st->st_ah.keymat_len);
+
+    if(st->st_esp.our_keymat!=NULL)
+    memset(st->st_esp.our_keymat, 0, st->st_esp.keymat_len);
+
+    if(st->st_esp.peer_keymat!=NULL)
+    memset(st->st_esp.peer_keymat, 0, st->st_esp.keymat_len);
+#endif
+    freeanychunk(st->st_skeyid);
+    freeanychunk(st->st_skey_d);
+    freeanychunk(st->st_skey_ai);
+    freeanychunk(st->st_skey_ar);
+    freeanychunk(st->st_skey_ei);
+    freeanychunk(st->st_skey_er);
+    freeanychunk(st->st_skey_pi);
+    freeanychunk(st->st_skey_pr);
+    freeanychunk(st->st_enc_key);
+    pfreeany(st->st_ah.our_keymat);
+    pfreeany(st->st_ah.peer_keymat);
+    pfreeany(st->st_esp.our_keymat);
+    pfreeany(st->st_esp.peer_keymat);
+    freeanychunk(st->st_xauth_password);
+#ifdef HAVE_LABELED_IPSEC
+    pfreeany(st->sec_ctx);
+#endif
+    pfree(st);
 }
 
 /* delete a state object */
@@ -367,60 +444,58 @@ delete_state(struct state *st)
     struct connection *const c = st->st_connection;
     struct state *old_cur_state = cur_state == st? NULL : cur_state;
 
-    if(st->st_ikev2)
-    {
-    /* child sa*/
-    if(st->st_clonedfrom != 0)
-    {
-	DBG(DBG_CONTROL, DBG_log("received request to delete child state"));
-	if(st->st_state == STATE_CHILDSA_DEL) {
+    openswan_log("deleting state #%lu (%s)",
+                 st->st_serialno,
+                 enum_show(&state_names, st->st_state));
+
+    /*
+     * for most IKEv2 things, we may have further things to do after marking the state deleted,
+     * so we do not actually free it here at all, but back in the main loop when all the work is done.
+     */
+    if(st->st_ikev2) {
+        /* child sa*/
+        if(st->st_clonedfrom != 0) {
+            DBG(DBG_CONTROL, DBG_log("received request to delete child state"));
+            if(st->st_state == STATE_CHILDSA_DEL) {
 		DBG(DBG_CONTROL, DBG_log("now deleting the child state"));
-	}
-	else
-	{
-		/* Only send request if child sa is established
+
+            } else {
+                /* Only send request if child sa is established
 		 * otherwise continue with deletion
 		 */
-		if(IS_CHILD_SA_ESTABLISHED(st))
-		{
-		DBG(DBG_CONTROL, DBG_log("sending Child SA delete equest"));
-		//change_state(st, STATE_CHILDSA_DEL);
-		send_delete(st);
-		change_state(st, STATE_CHILDSA_DEL);
-		/* actual deletion when we receive peer response*/
-		goto delete_state_end;
+		if(IS_CHILD_SA_ESTABLISHED(st)) {
+                    DBG(DBG_CONTROL, DBG_log("sending Child SA delete equest"));
+                    send_delete(st);
+                    change_state(st, STATE_CHILDSA_DEL);
+
+                    /* actual deletion when we receive peer response*/
+                    return;
 		}
-	}
-    }
-    else
-    {
-	DBG(DBG_CONTROL, DBG_log("received request to delete IKE parent state"));
-	/* parent sa */
-	if(st->st_state == STATE_IKESA_DEL)
-	{
-		DBG(DBG_CONTROL, DBG_log("now deleting the IKE (or parent) state"));
-	}
-	else
-	{
+            }
+
+        } else {
+            DBG(DBG_CONTROL, DBG_log("considering request to delete IKE parent state"));
+            /* parent sa */
+            if(st->st_state == STATE_IKESA_DEL) {
+                DBG(DBG_CONTROL, DBG_log("now deleting the IKE (or parent) state"));
+
+            } else {
 		/* Another check to verify if a secured
 		 * INFORMATIONAL exchange can be sent or not
 		 */
 		if(st->st_skey_ei.ptr && st->st_skey_ai.ptr
-			&& st->st_skey_er.ptr && st->st_skey_ar.ptr)
-		{
-		DBG(DBG_CONTROL, DBG_log("sending IKE SA delete request"));
-		//change_state(st, STATE_IKESA_DEL);
-		send_delete(st);
-		change_state(st, STATE_IKESA_DEL);
-		/* actual deletion when we receive peer response*/
-                goto delete_state_end;
+                   && st->st_skey_er.ptr && st->st_skey_ar.ptr) {
+                    DBG(DBG_CONTROL, DBG_log("sending IKE SA delete request"));
+                    send_delete(st);
+                    change_state(st, STATE_IKESA_DEL);
+                    event_schedule(EVENT_SA_DELETE, 300, st);
+
+                    /* actual deletion when we receive peer response*/
+                    return;
 		}
-	}
-
+            }
+        }
     }
-    }
-
-    DBG(DBG_CONTROL, DBG_log("deleting state #%lu", st->st_serialno));
 
     /* If DPD is enabled on this state object, clear any pending events */
     if(st->st_dpd_event != NULL)
@@ -487,97 +562,7 @@ delete_state(struct state *st)
 
     release_whack(st);
 
-    /* from here on we are just freeing RAM */
-
-    {
-	struct msgid_list *p = st->st_used_msgids;
-
-	while (p != NULL)
-	{
-	    struct msgid_list *q = p;
-	    p = p->next;
-	    pfree(q);
-	}
-    }
-
-    unreference_key(&st->st_peer_pubkey);
-
-    free_sa(st->st_sadb);
-    st->st_sadb=NULL;
-
-    if (st->st_sec_in_use) {
-#ifdef HAVE_LIBNSS
-	SECKEYPrivateKey *privk;
-	SECKEYPublicKey   *pubk;
-	memcpy(&pubk,st->pubk.ptr,st->pubk.len);
-	SECKEY_DestroyPublicKey(pubk);
-	freeanychunk(st->pubk);
-	memcpy(&privk,st->st_sec_chunk.ptr,st->st_sec_chunk.len);
-	SECKEY_DestroyPrivateKey(privk);
-#else
-	mpz_clear(&(st->st_sec));
-#endif
-	pfreeany(st->st_sec_chunk.ptr);
-    }
-
-    /*free selectors if any */
-    delete_ts(&st->st_ts_this);
-    delete_ts(&st->st_ts_that);
-
-    freeanychunk(st->st_firstpacket_me);
-    freeanychunk(st->st_firstpacket_him);
-    freeanychunk(st->st_tpacket);
-    freeanychunk(st->st_rpacket);
-    freeanychunk(st->st_p1isa);
-    freeanychunk(st->st_gi);
-    freeanychunk(st->st_gr);
-    freeanychunk(st->st_shared);
-    freeanychunk(st->st_ni);
-    freeanychunk(st->st_nr);
-#ifdef HAVE_LIBNSS
-    free_osw_nss_symkey(st->st_skeyid);
-    free_osw_nss_symkey(st->st_skey_d);
-    free_osw_nss_symkey(st->st_skey_ai);
-    free_osw_nss_symkey(st->st_skey_ar);
-    free_osw_nss_symkey(st->st_skey_ei);
-    free_osw_nss_symkey(st->st_skey_er);
-    free_osw_nss_symkey(st->st_skey_pi);
-    free_osw_nss_symkey(st->st_skey_pr);
-    free_osw_nss_symkey(st->st_enc_key);
-
-    if(st->st_ah.our_keymat!=NULL)
-    memset(st->st_ah.our_keymat, 0, st->st_ah.keymat_len);
-
-    if(st->st_ah.peer_keymat!=NULL)
-    memset(st->st_ah.peer_keymat, 0, st->st_ah.keymat_len);
-
-    if(st->st_esp.our_keymat!=NULL)
-    memset(st->st_esp.our_keymat, 0, st->st_esp.keymat_len);
-
-    if(st->st_esp.peer_keymat!=NULL)
-    memset(st->st_esp.peer_keymat, 0, st->st_esp.keymat_len);
-#endif
-    freeanychunk(st->st_skeyid);
-    freeanychunk(st->st_skey_d);
-    freeanychunk(st->st_skey_ai);
-    freeanychunk(st->st_skey_ar);
-    freeanychunk(st->st_skey_ei);
-    freeanychunk(st->st_skey_er);
-    freeanychunk(st->st_skey_pi);
-    freeanychunk(st->st_skey_pr);
-    freeanychunk(st->st_enc_key);
-    pfreeany(st->st_ah.our_keymat);
-    pfreeany(st->st_ah.peer_keymat);
-    pfreeany(st->st_esp.our_keymat);
-    pfreeany(st->st_esp.peer_keymat);
-    freeanychunk(st->st_xauth_password);
-#ifdef HAVE_LABELED_IPSEC
-    pfreeany(st->sec_ctx);
-#endif
-    pfree(st);
-
-delete_state_end:;
-
+    change_state(st, STATE_CHILDSA_DEL);
 }
 
 /*
@@ -621,6 +606,12 @@ foreach_states_by_connection_func(struct connection *c
     {
 	int i;
 
+        if(pass == 0) {
+            DBG(DBG_CONTROL, DBG_log("pass 0: considering CHILD SAs to delete"));
+        } else {
+            DBG(DBG_CONTROL, DBG_log("pass 1: considering PARENT SAs to delete"));
+        }
+
 	/* For each hash chain... */
 	for (i = 0; i < STATE_TABLE_SIZE; i++)
 	{
@@ -633,8 +624,13 @@ foreach_states_by_connection_func(struct connection *c
 
 		st = st->st_hashchain_next;	/* before this is deleted */
 
-		/* on pass 2, ignore phase2 states */
- 		if(pass == 1 && IS_ISAKMP_SA_ESTABLISHED(this->st_state)) {
+		/* on pass 0, ignore phase1 states */
+ 		if(pass == 0 && IS_ISAKMP_SA_ESTABLISHED(this->st_state)) {
+		    continue;
+		}
+
+		/* on pass 1, ignore phase2 states */
+ 		if(pass == 1 && IS_CHILD_SA(this)) {
 		    continue;
 		}
 
@@ -647,7 +643,7 @@ foreach_states_by_connection_func(struct connection *c
 		    lset_t old_cur_debugging = cur_debugging;
 #endif
 
-    set_cur_state(this);
+                    set_cur_state(this);
 		    (*successfunc)(this, c, arg);
 
 		    cur_state = old_cur_state;
@@ -664,13 +660,7 @@ static void delete_state_function(struct state *this
 				  , struct connection *c UNUSED
 				  , void *arg UNUSED)
 {
-    openswan_log("deleting state (%s)"
-		 , enum_show(&state_names, this->st_state));
-
     if(this->st_event != NULL) delete_event(this);
-    if(this->st_ikev2) {
-	this->st_state = this->st_clonedfrom ? STATE_CHILDSA_DEL : STATE_IKESA_DEL;
-    }
     delete_state(this);
 }
 
@@ -733,7 +723,6 @@ delete_states_by_connection(struct connection *c, bool relations)
 {
     so_serial_t parent_sa = c->newest_isakmp_sa;
     enum connection_kind ck = c->kind;
-    struct spd_route *sr;
 
     /* save this connection's isakmp SA,
      * since it will get set to later SOS_NOBODY */
@@ -750,23 +739,6 @@ delete_states_by_connection(struct connection *c, bool relations)
 					  , &parent_sa);
     }
 
-    /*
-     * Seems to dump here because 1 of the states is NULL.  Removing the Assert
-     * makes things work.  We should fix this eventually.
-     *
-     *  passert(c->newest_ipsec_sa == SOS_NOBODY
-     *  && c->newest_isakmp_sa == SOS_NOBODY);
-     *
-     */
-
-    sr = &c->spd;
-    while (sr != NULL)
-    {
-	passert(sr->eroute_owner == SOS_NOBODY);
-	passert(sr->routing != RT_ROUTED_TUNNEL);
-	sr = sr->next;
-    }
-
     if (ck == CK_INSTANCE)
     {
 	c->kind = ck;
@@ -954,6 +926,8 @@ duplicate_state(struct state *st)
     nst->st_clonedfrom = st->st_serialno;
     nst->st_import     = st->st_import;
     nst->st_ikev2      = st->st_ikev2;
+    nst->st_ike_maj    = st->st_ike_maj;
+    nst->st_ike_min    = st->st_ike_min;
     nst->st_event      = NULL;
 
 #   define clone_chunk(ch, name) \
@@ -1022,7 +996,7 @@ find_state_ikev1(const u_char *icookie
 		 , const ip_address *peer UNUSED
 		 , msgid_t /*network order*/ msgid)
 {
-    struct state *st = *state_hash(icookie, rcookie);
+    struct state *st = *state_hash(icookie, rcookie, NULL);
 
     while (st != (struct state *) NULL)
     {
@@ -1060,7 +1034,7 @@ find_state_ikev1_loopback(const u_char *icookie
                  , msgid_t /*network order*/ msgid
 		 , struct msg_digest *md)
 {
-    struct state *st = *state_hash(icookie, rcookie);
+    struct state *st = *state_hash(icookie, rcookie, NULL);
 
     while (st != (struct state *) NULL)
     {
@@ -1099,7 +1073,8 @@ struct state *
 find_state_ikev2_parent(const u_char *icookie
 			, const u_char *rcookie)
 {
-    struct state *st = *state_hash(icookie, rcookie);
+    unsigned bucket;
+    struct state *st = *state_hash(icookie, rcookie, &bucket);
 
     while (st != (struct state *) NULL)
     {
@@ -1120,9 +1095,9 @@ find_state_ikev2_parent(const u_char *icookie
 	if (st == NULL)
 	    DBG_log("v2 state object not found");
 	else
-	    DBG_log("v2 state object #%lu found, in %s"
-		, st->st_serialno
-		, enum_show(&state_names, st->st_state)));
+	    DBG_log("v2 state object #%lu (%s) found, in %s"
+                    , st->st_serialno, st->st_connection->name
+                    , enum_show(&state_names, st->st_state)));
 
     return st;
 }
@@ -1134,7 +1109,7 @@ find_state_ikev2_parent(const u_char *icookie
 struct state *
 find_state_ikev2_parent_init(const u_char *icookie)
 {
-    struct state *st = *state_hash(icookie, zero_cookie);
+    struct state *st = *state_hash(icookie, zero_cookie, NULL);
 
     while (st != (struct state *) NULL)
     {
@@ -1169,7 +1144,7 @@ find_state_ikev2_child(const u_char *icookie
 		       , const u_char *rcookie
 		       , msgid_t msgid)
 {
-    struct state *st = *state_hash(icookie, rcookie);
+    struct state *st = *state_hash(icookie, rcookie, NULL);
 
     while (st != (struct state *) NULL)
     {
@@ -1207,7 +1182,7 @@ find_state_ikev2_child_to_delete(const u_char *icookie
 		       , u_int8_t protoid
 		       , ipsec_spi_t spi)
 {
-    struct state *st = *state_hash(icookie, rcookie);
+    struct state *st = *state_hash(icookie, rcookie, NULL);
 
     while (st != (struct state *) NULL)
     {
@@ -1250,7 +1225,7 @@ find_info_state(const u_char *icookie
 		, const ip_address *peer UNUSED
 		, msgid_t /*network order*/ msgid)
 {
-    struct state *st = *state_hash(icookie, rcookie);
+    struct state *st = *state_hash(icookie, rcookie, NULL);
 
     while (st != (struct state *) NULL)
     {
@@ -1317,7 +1292,7 @@ find_phase2_state_to_delete(const struct state *p1st
 	for (st = statetable[i]; st != NULL; st = st->st_hashchain_next)
 	{
 	    if (IS_IPSEC_SA_ESTABLISHED(st->st_state)
-	    && p1st->st_connection->host_pair == st->st_connection->host_pair
+	    && p1st->st_connection->IPhost_pair == st->st_connection->IPhost_pair
 	    && same_peer_ids(p1st->st_connection, st->st_connection, NULL))
 	    {
 		struct ipsec_proto_info *pr = protoid == PROTO_IPSEC_AH
@@ -1349,7 +1324,7 @@ find_phase1_state(const struct connection *c, lset_t ok_states)
     for (i = 0; i < STATE_TABLE_SIZE; i++) {
 	for (st = statetable[i]; st != NULL; st = st->st_hashchain_next) {
 	    if (LHAS(ok_states, st->st_state)
-		&& c->host_pair == st->st_connection->host_pair
+		&& c->IPhost_pair == st->st_connection->IPhost_pair
 		&& same_peer_ids(c, st->st_connection, NULL)
 		&& (best == NULL
 		    || best->st_serialno < st->st_serialno))
@@ -1403,6 +1378,15 @@ state_eroute_usage(ip_subnet *ours, ip_subnet *his
 	});
 }
 
+static long msgid_invalid(msgid_t thing)
+{
+    if(thing == INVALID_MSGID) {
+        return -1;
+    } else {
+        return thing;
+    }
+}
+
 void fmt_state(struct state *st, const time_t n
 , char *state_buf, const size_t state_buf_len
 , char *state_buf2, const size_t state_buf2_len)
@@ -1412,6 +1396,7 @@ void fmt_state(struct state *st, const time_t n
     long delta;
     char inst[CONN_INST_BUF];
     char dpdbuf[128];
+    char msgidbuf[128];
     const char *np1 = c->newest_isakmp_sa == st->st_serialno
 	? "; newest ISAKMP" : "";
     const char *np2 = c->newest_ipsec_sa == st->st_serialno
@@ -1431,6 +1416,7 @@ void fmt_state(struct state *st, const time_t n
 	delta = -1;
     }
 
+    msgidbuf[0]='\0';
     if (IS_IPSEC_SA_ESTABLISHED(st->st_state))
     {
 	dpdbuf[0]='\0';
@@ -1445,6 +1431,20 @@ void fmt_state(struct state *st, const time_t n
 	} else {
 	    snprintf(dpdbuf, sizeof(dpdbuf), "; nodpd");
 	}
+        if(st->st_ikev2) {
+            if(IS_PARENT_SA(st)) {
+                snprintf(msgidbuf, sizeof(msgidbuf), "; retranscnt=%ld,outorder=%ld,last=%ld,next=%ld,recv=%ld; msgid=%ld"
+                     , (long)st->st_msg_retransmitted
+                     , (long)st->st_msg_badmsgid_recv
+                         , msgid_invalid(st->st_msgid_lastack)
+                         , msgid_invalid(st->st_msgid_nextuse)
+                         , msgid_invalid(st->st_msgid_lastrecv)
+                         , msgid_invalid(st->st_msgid));
+            } else {
+                snprintf(msgidbuf, sizeof(msgidbuf), "; msgid=%ld"
+                         , msgid_invalid(st->st_msgid));
+            }
+        }
     }
 
     if(st->st_calculating) {
@@ -1456,15 +1456,16 @@ void fmt_state(struct state *st, const time_t n
     }
 
     snprintf(state_buf, state_buf_len
-	     , "#%lu: \"%s\"%s:%u %s (%s); %s in %lds%s%s%s%s; %s; %s"
+	     , "#%lu: \"%s\"%s:%u IKEv%u.%u %s (%s); %s in %lds%s%s%s%s%s; %s; %s"
 	     , st->st_serialno
 	     , c->name, inst
 	     , st->st_remoteport
+             , st->st_ike_maj, st->st_ike_min
 	     , enum_name(&state_names, st->st_state)
-	     , state_story[st->st_state - STATE_MAIN_R0]
+	     , enum_name(&state_stories, st->st_state)
 	     , st->st_event ? enum_name(&timer_event_names, st->st_event->ev_type) : "none"
 	     , delta
-	     , np1, np2, eo, dpdbuf
+	     , np1, np2, eo, dpdbuf, msgidbuf
 	     , idlestr
 	     , enum_name(&pluto_cryptoimportance_names, st->st_import));
 
@@ -1614,7 +1615,7 @@ show_states_status(void)
 	   }
         }
 
-         /* sort it! */
+         /* sort it --- XXXX might be a big deal for really big systems... */
          qsort(array, count, sizeof(struct state *), state_compare);
 
          /* now print sorted results */
@@ -1638,6 +1639,18 @@ show_states_status(void)
     }
 }
 
+void dump_one_state(struct state *st)
+{
+    char state_buf[LOG_WIDTH];
+    char state_buf2[LOG_WIDTH];
+
+    fmt_state(st, 1, state_buf, sizeof(state_buf)
+              , state_buf2, sizeof(state_buf2));
+    DBG_log("%s", state_buf);
+    if (state_buf2[0] != '\0')
+        DBG_log("%s", state_buf2);
+}
+
 /* Given that we've used up a range of unused CPI's,
  * search for a new range of currently unused ones.
  * Note: this is very expensive when not trivial!
@@ -1760,6 +1773,7 @@ void copy_quirks(struct isakmp_quirks *dq
     dq->xauth_ack_msgid   |= sq->xauth_ack_msgid;
     dq->modecfg_pull_mode |= sq->modecfg_pull_mode;
     dq->nat_traversal_vid |= sq->nat_traversal_vid;
+    dq->xauth_vid |= sq->xauth_vid;
 }
 
 void set_state_ike_endpoints(struct state *st
@@ -1767,7 +1781,7 @@ void set_state_ike_endpoints(struct state *st
 {
     /* reset our choice of interface */
     c->interface = NULL;
-    orient(c);
+    orient(c, pluto_port500);
 
     st->st_localaddr  = c->spd.this.host_addr;
     st->st_localport  = c->spd.this.host_port;
@@ -1779,6 +1793,7 @@ void set_state_ike_endpoints(struct state *st
 
 /*
  * Local Variables:
- * c-basic-offset:4
+ * c-style: pluto
+ * c-basic-offset: 4
  * End:
  */
