@@ -26,7 +26,8 @@
 #include <netinet/in.h>
 #include <time.h>
 #include <gmp.h>    /* GNU MP library */
-#include "quirks.h"
+#include "pluto/quirks.h"
+#include "id.h"
 
 #ifdef HAVE_LIBNSS
 # include <nss.h>
@@ -55,11 +56,13 @@
 #define INVALID_MSGID     0xffffffff
 
 struct state;	/* forward declaration of tag */
+
+/* used by IKEv1 only */
 extern void reserve_msgid(struct state *isakmp_sa, msgid_t msgid);
 extern bool unique_msgid(struct state *isakmp_sa, msgid_t msgid);
-
 extern msgid_t generate_msgid(struct state *isakmp_sa);
 
+
 #define XAUTH_USERNAME_LEN 64
 
 /* Oakley (Phase 1 / Main Mode) transform and attributes
@@ -127,9 +130,9 @@ struct ipsec_proto_info {
 };
 
 /*
- * internal state that
- * should get copied by god... to the child SA state.
- * this is to make Einstein happy.
+ * internal state that should get copied by god... to the child SA state.
+ * (this is to make Einstein happy)
+ * A smarter way in IKEv2 might be to make this a pointer in the child?
  */
 
 struct hidden_variables {
@@ -188,14 +191,20 @@ struct xfrm_user_sec_ctx_ike {
  * - each state object will be in statetable exactly once.
  * - each state object will always have a pending event.
  *   This prevents leaks.
+ *
+ * - should eventually move all ikev1 specific stuff into "ikev1" struct
+ *   and ikev2 stuff too.
  */
 struct state
 {
     so_serial_t        st_serialno;          /* serial number (for seniority)*/
     so_serial_t        st_clonedfrom;        /* serial number of parent */
+    so_serial_t        st_replaced;          /* what state are we rekey for? */
     int                st_usage;
 
     bool               st_ikev2;             /* is this an IKEv2 state? */
+    u_char             st_ike_maj;
+    u_char             st_ike_min;
     bool               st_rekeytov2;         /* true if this IKEv1 is about
 					      * to be replaced with IKEv2 */
 
@@ -250,6 +259,18 @@ struct state
     struct msgid_list  *st_used_msgids;        /* used-up msgids */
 
     /* IKEv2 things */
+    struct {
+        struct id      st_peer_id;             /* stores decoded peer ID */
+	char           st_peer_buf[IDTOA_BUF];
+        struct id      st_local_id;            /* stores decoded ID for me */
+	char           st_local_buf[IDTOA_BUF];
+    } ikev2;
+
+    /* counters */
+    unsigned           st_msg_retransmitted;   /* total number of retransmissions seen */
+    unsigned           st_msg_badmsgid_recv;   /* out of order messages */
+
+
     /* message ID sequence for things we send (as initiator) */
     msgid_t            st_msgid_lastack;       /* last one peer acknowledged */
     msgid_t            st_msgid_nextuse;       /* next one to use */
@@ -257,7 +278,6 @@ struct state
     /* message ID sequence for things we receive (as responder) */
     msgid_t            st_msgid_lastrecv;      /* last one peer sent */
 
-
     /* symmetric stuff */
 
     /* initiator stuff */
@@ -269,16 +289,15 @@ struct state
     chunk_t            st_gr;                  /* Responder public value */
     u_int8_t           st_rcookie[COOKIE_SIZE];/* Responder Cookie */
     chunk_t            st_nr;                  /* Nr nonce */
-    chunk_t			   st_dcookie;             /* DOS cookie of responder */
+    chunk_t            st_dcookie;             /* DOS cookie of responder */
 
     /* my stuff */
     chunk_t            st_tpacket;             /* Transmitted packet */
     chunk_t            st_firstpacket_me;      /* copy of my message 1 */
     chunk_t            st_firstpacket_him;     /* copy of his message 1 */
 
-#ifdef HAVE_LABELED_IPSEC
-     struct xfrm_user_sec_ctx_ike *sec_ctx;
-#endif
+    /* always present, but not used if feature compiled out */
+    struct xfrm_user_sec_ctx_ike *sec_ctx;
 
     /* Phase 2 ID payload info about my user */
     u_int8_t           st_myuserprotoid;       /* IDcx.protoid */
@@ -390,10 +409,7 @@ struct state
  					   * products */
 
 };
-
-/* global variables */
-
-extern u_int16_t pluto_port;	/* Pluto's port */
+#define NULL_STATE NULL
 
 extern bool states_use_connection(struct connection *c);
 
@@ -407,6 +423,7 @@ extern void rehash_state(struct state *st);
 extern void release_whack(struct state *st);
 extern void state_eroute_usage(ip_subnet *ours, ip_subnet *his
     , unsigned long count, time_t nw);
+extern void free_state(struct state *st);
 extern void delete_state(struct state *st);
 struct connection;	/* forward declaration of tag */
 extern void delete_states_by_connection(struct connection *c, bool relations);
@@ -459,6 +476,8 @@ extern void initialize_new_state(struct state *st
 			       , enum crypto_importance importance);
 
 extern void show_states_status(void);
+extern void dump_one_state(struct state *st);
+
 
 #if 1
 void for_each_state(void *(f)(struct state *, void *data), void *data);
