@@ -38,13 +38,13 @@
 #ifdef XAUTH_USEPAM
 #include <security/pam_appl.h>
 #endif
-#include "connections.h"	/* needs id.h */
+#include "pluto/connections.h"	/* needs id.h */
 #include "state.h"
 #include "packet.h"
 #include "demux.h"  /* needs packet.h */
 #include "ipsec_doi.h"	/* needs demux.h and state.h */
 #include "kernel.h"	/* needs connections.h */
-#include "server.h"
+#include "pluto/server.h"
 #include "log.h"
 #include "rnd.h"
 #include "timer.h"
@@ -76,6 +76,7 @@ void
 event_schedule(enum event_type type, time_t tm, struct state *st)
 {
     struct event *ev = alloc_thing(struct event, "struct event in event_schedule()");
+    const char *headqueue;
 
     passert(tm >= 0);
     ev->ev_type = type;
@@ -98,19 +99,24 @@ event_schedule(enum event_type type, time_t tm, struct state *st)
 	    }
     }
 
+    headqueue = "";
+    if (evlist == (struct event *) NULL
+	|| evlist->ev_time >= ev->ev_time) {
+        headqueue = "(head of queue)";
+    }
+
     DBG(DBG_CONTROL,
 	if (st == NULL)
-	    DBG_log("inserting event %s, timeout in %lu seconds"
-		, enum_show(&timer_event_names, type), (unsigned long)tm);
+	    DBG_log("inserting event %s, timeout in %lu seconds %s"
+                    , enum_show(&timer_event_names, type), (unsigned long)tm, headqueue);
 	else
-	    DBG_log("inserting event %s, timeout in %lu seconds for #%lu"
-		, enum_show(&timer_event_names, type), (unsigned long)tm
-		, ev->ev_state->st_serialno));
+	    DBG_log("inserting event %s, timeout in %lu seconds for #%lu %s"
+                    , enum_show(&timer_event_names, type), (unsigned long)tm
+                    , ev->ev_state->st_serialno, headqueue));
 
     if (evlist == (struct event *) NULL
 	|| evlist->ev_time >= ev->ev_time)
     {
-	DBG(DBG_CONTROLMORE, DBG_log("event added at head of queue"));
 	ev->ev_next = evlist;
 	evlist = ev;
     }
@@ -458,8 +464,10 @@ handle_next_timer_event(void)
     type = ev->ev_type;
     st = ev->ev_state;
 
-    DBG(DBG_CONTROL, DBG_log("handling event %s"
-			     , enum_show(&timer_event_names, type)));
+    if(DBGP(DBG_CONTROL)) {
+        DBG_log("at %s handling event %s", oswtimestr()
+                , enum_show(&timer_event_names, type));
+    }
 
     if(DBGP(DBG_CONTROL)) {
 	if (evlist != (struct event *) NULL) {
@@ -506,12 +514,10 @@ handle_next_timer_event(void)
 	    break;
 #endif
 
-#ifdef DYNAMICDNS
         case EVENT_PENDING_DDNS:
 	    passert(st == NULL);
 	    connection_check_ddns();
 	    break;
-#endif
 
         case EVENT_PENDING_PHASE2:
 	    passert(st == NULL);
@@ -658,6 +664,11 @@ handle_next_timer_event(void)
 	    delete_state(st);
 	    break;
 
+        case EVENT_SA_DELETE:
+	    passert(st == NULL);
+	    DBG(DBG_CONTROL, DBG_log("event EVENT_SA_DELETE --- no reply to delete"));
+            delete_state(st);
+	    break;
 
 	default:
 	    loglog(RC_LOG_SERIOUS, "INTERNAL ERROR: ignoring unknown expiring event %s"
@@ -689,11 +700,12 @@ next_event(void)
 	    DBG_log("next event %s in %ld seconds"
 		, enum_show(&timer_event_names, evlist->ev_type)
 		, (long)evlist->ev_time - (long)tm);
-	else
-	    DBG_log("next event %s in %ld seconds for #%lu"
+	else {
+	    DBG_log("next event %s in %ld seconds for #%lu (%s)"
 		, enum_show(&timer_event_names, evlist->ev_type)
 		, (long)evlist->ev_time - (long)tm
-		, evlist->ev_state->st_serialno));
+                    , evlist->ev_state->st_serialno, oswtimestr()));
+        }
 
     if (evlist->ev_time - tm <= 0)
 	return 0;
