@@ -46,7 +46,8 @@
 #include "keys.h"
 #include "packet.h"
 #include "demux.h"	/* needs packet.h */
-#include "connections.h"
+#include "pluto/connections.h"
+#include "hostpair.h"
 #include "state.h"
 #include "md2.h"
 #include "md5.h"
@@ -55,102 +56,9 @@
 #include "fetch.h"
 #include "ocsp.h"
 #include "pkcs.h"
+#include "kernel.h"
 #include "x509more.h"
 
-/* extract id and public key from x.509 certificate and
- * insert it into a pubkeyrec
- */
-void
-add_x509_public_key(struct id *keyid
-		    , x509cert_t *cert
-		    , time_t until
-		    , enum dns_auth_level dns_auth_level)
-{
-    generalName_t *gn;
-    struct pubkey *pk;
-    cert_t c = { FALSE, CERT_X509_SIGNATURE, {cert} };
-
-    /* we support RSA only */
-    if (cert->subjectPublicKeyAlgorithm != PUBKEY_ALG_RSA) return;
-
-    /* ID type: ID_DER_ASN1_DN  (X.509 subject field) */
-    pk = allocate_RSA_public_key(c);
-    passert(pk != NULL);
-    pk->id.kind = ID_DER_ASN1_DN;
-    pk->id.name = cert->subject;
-    pk->dns_auth_level = dns_auth_level;
-    pk->until_time = until;
-    pk->issuer = cert->issuer;
-    delete_public_keys(&pluto_pubkeys, &pk->id, pk->alg);
-    install_public_key(pk, &pluto_pubkeys);
-
-    gn = cert->subjectAltName;
-
-    while (gn != NULL) /* insert all subjectAltNames */
-    {
-	struct id id = empty_id;
-
-	gntoid(&id, gn);
-	if (id.kind != ID_NONE)
-	{
-	    pk = allocate_RSA_public_key(c);
-	    pk->id = id;
-	    pk->dns_auth_level = dns_auth_level;
-	    pk->until_time = until;
-	    pk->issuer = cert->issuer;
-	    delete_public_keys(&pluto_pubkeys, &pk->id, pk->alg);
-	    install_public_key(pk, &pluto_pubkeys);
-	}
-	gn = gn->next;
-    }
-
-    if(keyid != NULL &&
-       keyid->kind != ID_DER_ASN1_DN &&
-       keyid->kind != ID_DER_ASN1_GN) {
-	pk = allocate_RSA_public_key(c);
-	pk->id = *keyid;
-	
-	pk->dns_auth_level = dns_auth_level;
-	pk->until_time = until;
-	pk->issuer = cert->issuer;
-	delete_public_keys(&pluto_pubkeys, &pk->id, pk->alg);
-	install_public_key(pk, &pluto_pubkeys);
-    }
-}
-
-
-/*  when a X.509 certificate gets revoked, all instances of
- *  the corresponding public key must be removed
- */
-void
-remove_x509_public_key(/*const*/ x509cert_t *cert)
-{
-    const cert_t c = {FALSE, CERT_X509_SIGNATURE, {cert}};
-    struct pubkey_list *p, **pp;
-    struct pubkey *revoked_pk;
-
-    revoked_pk = allocate_RSA_public_key(c);
-    p          = pluto_pubkeys;
-    pp         = &pluto_pubkeys;
-
-    while(p != NULL)
-   {
-	if (same_RSA_public_key(&p->key->u.rsa, &revoked_pk->u.rsa))
-	{
-	    /* remove p from list and free memory */
-	    *pp = free_public_keyentry(p);
-	    loglog(RC_LOG_SERIOUS,
-		"invalid RSA public key deleted");
-	}
-	else
-	{
-	    pp = &p->next;
-	}
-	p =*pp;
-    }
-    free_public_key(revoked_pk);
-}
-
 /*
  * Decode the CERT payload of Phase 1.
  */
@@ -229,8 +137,8 @@ ikev2_decode_cert(struct msg_digest *md)
 		{
 		    DBG(DBG_X509 | DBG_PARSING,
 			DBG_log("Public key validated")
-		    )
-			add_x509_public_key(NULL, &cert2, valid_until, DAL_SIGNED);
+                        );
+                    add_x509_public_key(NULL, &cert2, valid_until, DAL_SIGNED);
 		}
 		else
 		{
@@ -274,7 +182,7 @@ decode_cr(struct msg_digest *md, generalName_t **requested_ca)
     {
 	struct isakmp_cr *const cr = &p->payload.cr;
 	chunk_t ca_name;
-	
+
 	ca_name.len = pbs_left(&p->pbs);
 	ca_name.ptr = (ca_name.len > 0)? p->pbs.cur : NULL;
 
@@ -286,7 +194,7 @@ decode_cr(struct msg_digest *md, generalName_t **requested_ca)
 	    if (ca_name.len > 0)
 	    {
 		generalName_t *gn;
-		
+
 		if (!is_asn1(ca_name))
 		    continue;
 
@@ -322,7 +230,7 @@ ikev2_decode_cr(struct msg_digest *md, generalName_t **requested_ca)
     {
 	struct ikev2_certreq *const cr = &p->payload.v2certreq;
 	chunk_t ca_name;
-	
+
 	ca_name.len = pbs_left(&p->pbs);
 	ca_name.ptr = (ca_name.len > 0)? p->pbs.cur : NULL;
 
@@ -334,7 +242,7 @@ ikev2_decode_cr(struct msg_digest *md, generalName_t **requested_ca)
 	    if (ca_name.len > 0)
 	    {
 		generalName_t *gn;
-		
+
 		if (!is_asn1(ca_name))
 		    continue;
 
@@ -402,13 +310,16 @@ ikev2_build_and_ship_CR(u_int8_t type, chunk_t ca, pb_stream *outs, u_int8_t np)
     close_output_pbs(&cr_pbs);
     return TRUE;
 }
+
 bool
 collect_rw_ca_candidates(struct msg_digest *md, generalName_t **top)
 {
-    struct connection *d = find_host_connection(&md->iface->ip_addr
-	, pluto_port, (ip_address*)NULL, md->sender_port, LEMPTY);
+    struct connection *d = find_host_connection(ANY_MATCH, &md->iface->ip_addr
+                                                , pluto_port500
+                                                , KH_ANY
+                                                ,(ip_address*)NULL, md->sender_port, LEMPTY, LEMPTY, NULL);
 
-    for (; d != NULL; d = d->hp_next)
+    for (; d != NULL; d = d->IPhp_next)
     {
 	/* must be a road warrior connection */
 	if (d->kind == CK_TEMPLATE && !(d->policy & POLICY_OPPO)
