@@ -21,8 +21,9 @@
 
 #include "packet.h"
 #include "demux.h"
-#include "server.h"
+#include "pluto/server.h"
 #include "secrets.h"
+#include "pluto/x509lists.h"
 
 /* forward reference */
 struct msg_digest;
