@@ -55,13 +55,13 @@
 #ifdef XAUTH_USEPAM
 #include <security/pam_appl.h>
 #endif
-#include "connections.h"	/* needs id.h */
+#include "pluto/connections.h"	/* needs id.h */
 #include "packet.h"
 #include "demux.h"	/* needs packet.h */
 #include "kernel.h"
 #include "log.h"
 #include "cookie.h"
-#include "server.h"
+#include "pluto/server.h"
 #include "spdb.h"
 #include "timer.h"
 #include "rnd.h"
@@ -75,7 +75,7 @@
 #include "ike_alg.h"
 
 #include "xauth.h"
-#include "virtual.h"
+#include "pluto/virtual.h"
 
 #ifdef HAVE_THREADS
 #include <pthread.h>
@@ -381,7 +381,7 @@ stf_status modecfg_resp(struct state *st
 	    dont_advance = FALSE;
 	    if(resp & 1)
 	    {
-		const unsigned char *byte_ptr;
+		unsigned char *byte_ptr;
 		unsigned int len;
 
 		/* ISAKMP attr out */
@@ -936,6 +936,7 @@ int do_pam_authentication(void *varg)
     pam_handle_t *pamh=NULL;
     int retval;
 
+    conv.conv = xauth_pam_conv;
     conv.appdata_ptr = varg;
 
     retval = pam_start("pluto", arg->name.ptr, &conv, &pamh);
@@ -1070,8 +1071,8 @@ int do_md5_authentication(void *varg)
 		openswan_log("XAUTH: checking user(%s:%s) " , szuser, szconnid);
 	    }
 
-           /* Ok then now password check */
-           if ( strcmp(cp, szpass ) == 0 )
+           /* Ok then now password check; Note: handle crypt() NULL returns */
+           if ( cp && strcmp(cp, szpass ) == 0 )
            {
              /* we have a winner */
              fclose( fp );
@@ -1786,7 +1787,7 @@ modecfg_inR1(struct msg_digest *md)
 
 		    if(addrbytesptr(&c->spd.this.host_srcip, NULL) == 0
 		       || isanyaddr(&c->spd.this.host_srcip)
-			|| c->remotepeertype == CISCO ) { 
+			|| c->remotepeertype == CISCO ) {
 			/*with remotepeertype == CISCO, overwrite the previous address with the new received address*/
 			openswan_log("setting ip source address to %s"
 				     , caddr);
@@ -1798,7 +1799,7 @@ modecfg_inR1(struct msg_digest *md)
 
 		case INTERNAL_IP4_NETMASK:
 		{
-		    //struct connection *c = st->st_connection;	
+		    //struct connection *c = st->st_connection;
 		    ip_address a;
 		    char caddr[SUBNETTOT_BUF];
 
@@ -1843,7 +1844,7 @@ modecfg_inR1(struct msg_digest *md)
 			     * concatenate only if the received DNS is not
 			     * already present in the current string.
 			     */
- 
+
 			    if( !strstr(c->cisco_dns_info, caddr) ) {
 			    size_t sz_old = strlen(old);
 			    size_t sz_added = strlen(caddr) + 1;
@@ -1899,7 +1900,7 @@ modecfg_inR1(struct msg_digest *md)
                     }
 
 					/* receiving remote subnets information again
-					* free the previous ones before proceeding. 
+					* free the previous ones before proceeding.
 					*/
 					tmp_spd = tmp_spd2->next;
 					tmp_spd2->next = NULL;
@@ -1907,7 +1908,7 @@ modecfg_inR1(struct msg_digest *md)
 					delete_sr(c, tmp_spd);
 					tmp_spd1 = tmp_spd->next;
 					pfree(tmp_spd);
-					tmp_spd = tmp_spd1;	    	
+					tmp_spd = tmp_spd1;
 					}
 
                     while (len > 0) {
@@ -2310,8 +2311,12 @@ xauth_inI0(struct msg_digest *md)
 		break;
 
 	    case XAUTH_MESSAGE:
-		if(len > 80) len=80;
-		memcpy(msgbuf, dat, len);
+		if(len > 80) {
+		    len=80;
+		}
+		if(dat) {
+		   memcpy(msgbuf, dat, len);
+		}
 		msgbuf[len]='\0';
 		loglog(RC_LOG_SERIOUS, "XAUTH: Bad Message: %s", msgbuf);
 		break;
